------------------------------------------------------------------------------------------
-- CalcStat file
-- Written by Giseldah
-- Version 2.3
-- ****************************** Start CalcStat ******************************

-- Floating point numbers bring errors into the calculation, both inside the Lotro-client and in this function collection. This is why a 100% match with the stats in Lotro is impossible.
-- Anyway, to compensate for some errors, we use a calculation deviation correction value. This makes for instance 24.49999999 round to 25, as it's assumed that 24.5 was intended as outcome of a formula.
DblCalcDev = 0.00000001;

function CalcStat(SName, SLvl, SParam)

	-- process parameters and parameter defaults

	if type(SName) ~= "string" then
		return "Missing stat name";
	end
	local SN = string.match(SName:upper(),"^%s*(-?%a+%d*%a+)%s*$"); -- allow nested digits in stat name and a starting -
	if SN == nil then
		SN = string.match(SName,"^%s*(-?%a+)%s*$"); -- to allow single character stat name
		if SN == nil then
			return "Illegal stat name";
		end
	end
	SN = string.upper(SN);

	local L = 1; -- default L
	if type(SLvl) == "number" then
		L = SLvl;
	elseif type(SLvl) ~= "nil" then
		return "Illegal level";
	end
	local Lm = L-DblCalcDev;
	local Lp = L+DblCalcDev;

	local N = 1; -- default N
	local C = ""; -- default C
	if type(SParam) == "number" then
		N = SParam;
	elseif type(SParam) == "string" then
		C = SParam;
	elseif type(SParam) ~= "nil" then
		return "Illegal N or C";
	end

	-- binary search tree (generated code)

	if SN < "MINSTRELCDBASEPOWER" then
		if SN < "CREEPAUDACITYDMGP" then
			if SN < "BRWAGGPOSTUREPHYMIT" then
				if SN < "BEORNINGCDBASEWILL" then
					if SN < "AWARDILVLF" then
						if SN < "ARMQTYMP" then
							if SN < "ARMOURADJBYTYPE" then
								if SN < "ADJUMBARTRAITMIT" then
									if SN < "ACIDMITT" then
										if SN > "-VERSION" then
											if SN == "ACIDMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN == "-VERSION" then
											return "2.3f";
										else
											return 0;
										end
									elseif SN > "ACIDMITT" then
										if SN < "ADJUMBARITEMMIT" then
											if SN == "ADJUMBARITEM" then
												if Lm <= 499 then
													return 1;
												else
													return 0.9;
												end
											else
												return 0;
											end
										elseif SN > "ADJUMBARITEMMIT" then
											if SN == "ADJUMBARTRAIT" then
												if Lm <= 140 then
													return 1;
												elseif Lm <= 150 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											if Lm <= 499 then
												return 1;
											elseif Lm <= 500 then
												return 0.78;
											else
												return 0.7;
											end
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								elseif SN > "ADJUMBARTRAITMIT" then
									if SN < "ARMCATMP" then
										if SN < "AGILITYC" then
											if SN == "AGILITY" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "AGILITYC" then
											if SN == "AGILITYT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MainC",L,N);
										end
									elseif SN > "ARMCATMP" then
										if SN < "ARMOUR" then
											if SN == "ARMCATPROGB" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("ProgBArmourHeavy",N);
												elseif Lm <= 2 then
													return CalcStat("ProgBArmourMedium",N);
												else
													return CalcStat("ProgBArmourLight",N);
												end
											else
												return 0;
											end
										elseif SN > "ARMOUR" then
											if SN == "ARMOURADJ" then
												return CalcStat("ArmourAdjByType",ArmCodeIndex(C,2),L);
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("ArmourPntMP","ItemProg","ArmourProgB","ArmourAdj",L,C,0));
										end
									else
										return CalcStat("PntMPArmour",L);
									end
								else
									if Lm <= 140 then
										return 1;
									elseif Lm <= 141 then
										return 0.78;
									else
										return 0.7;
									end
								end
							elseif SN > "ARMOURADJBYTYPE" then
								if SN < "ARMOURPENT" then
									if SN < "ARMOURC" then
										if SN < "ARMOURADJTYPEOTHER" then
											if SN == "ARMOURADJTYPECLOAK" then
												return CalcStat("AdjUmbarItemMit",L);
											else
												return 0;
											end
										elseif SN > "ARMOURADJTYPEOTHER" then
											if SN == "ARMOURADJTYPESHIELD" then
												return CalcStat("AdjUmbarItemMit",L);
											else
												return 0;
											end
										else
											return CalcStat("AdjUmbarItemMit",L);
										end
									elseif SN > "ARMOURC" then
										if SN < "ARMOURLOW" then
											if SN == "ARMOURCRAW" then
												return StatLinInter("PntMPArmourC","CreepProg","ProgBArmourC","",L,N,99);
											else
												return 0;
											end
										elseif SN > "ARMOURLOW" then
											if SN == "ARMOURLOWPNTMP" then
												return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyLowMP",ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("ArmourLowPntMP","ItemProg","ArmourProgB","ArmourAdj",L,C,0));
										end
									else
										return StatLinInter("PntMPArmourC","CreepProg","ProgBArmourC","",L,N,0);
									end
								elseif SN > "ARMOURPENT" then
									if SN < "ARMQTYCOMMMP" then
										if SN < "ARMOURPROGB" then
											if SN == "ARMOURPNTMP" then
												return CalcStat("ArmTypeMP",ArmCodeIndex(C,2))*CalcStat("ArmQtyMP",ArmCodeIndex(C,3),ArmCodeIndex(C,2))*CalcStat("ArmCatMP",ArmCodeIndex(C,1));
											else
												return 0;
											end
										elseif SN > "ARMOURPROGB" then
											if SN == "ARMOURT" then
												return StatLinInter("PntMPArmourT","TraitProg","ProgBArmour","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("ArmCatProgB",ArmCodeIndex(C,1),L);
										end
									elseif SN > "ARMQTYCOMMMP" then
										if SN < "ARMQTYINCOMPMP" then
											if SN == "ARMQTYEPICMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1,1,1,1,1.002,1,1},L);
												end
											else
												return 0;
											end
										elseif SN > "ARMQTYINCOMPMP" then
											if SN == "ARMQTYLOWMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.3635,0.3635,0.3635,0.3635,0.3635,0.36215,0.3635,0.3635},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({28/30,28/30,0.95,0.95,0.94,0.948,0.95,0.95},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({22/30,22/30,0.8,0.8,0.76,0.804,0.8,0.7},L);
										end
									end
								else
									return StatLinInter("PntMPArmourPenT","TraitProg","ProgBArmour","",L,N,0);
								end
							else
								if 3 <= Lp and Lm <= 3 then
									return CalcStat("ArmourAdjTypeCloak",N);
								elseif 8 <= Lp and Lm <= 8 then
									return CalcStat("ArmourAdjTypeShield",N);
								else
									return CalcStat("ArmourAdjTypeOther",N);
								end
							end
						elseif SN > "ARMQTYMP" then
							if SN < "AWARDILVLCC" then
								if SN < "AWARDILVLAC" then
									if SN < "AUTOLVLTOILVL" then
										if SN < "ARMQTYUNCOMMP" then
											if SN == "ARMQTYRAREMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({26/30,26/30,0.9,0.9,0.88,0.9,0.9,0.8},L);
												end
											else
												return 0;
											end
										elseif SN > "ARMQTYUNCOMMP" then
											if SN == "ARMTYPEMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({9,9,20,30,15,25,12,32.726},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({22/30,22/30,0.85,0.85,0.82,0.852,0.85,0.75},L);
											end
										end
									elseif SN > "AUTOLVLTOILVL" then
										if SN < "AWARDILVLAA" then
											if SN == "AWARDILVLA" then
												if Lm <= 75 then
													return L;
												elseif Lm <= 94 then
													return 5*L-304;
												elseif Lm <= 95 then
													return 173;
												elseif Lm <= 100 then
													return 5*L-304;
												elseif Lm <= 101 then
													return 197;
												elseif Lm <= 104 then
													return 4*L-206;
												elseif Lm <= 105 then
													return 218;
												elseif Lm <= 110 then
													return RoundDbl(LinFmod(1,295,303.8,106,110,L));
												elseif Lm <= 115 then
													return 304;
												elseif Lm <= 119 then
													return RoundDbl(LinFmod(1,345,357,116,119,L));
												elseif Lm <= 120 then
													return 359;
												elseif Lm <= 130 then
													return 395;
												elseif Lm <= 140 then
													return RoundDbl(LinFmod(1,445.4,455.4,131,140,L));
												elseif Lm <= 150 then
													return RoundDbl(LinFmod(1,479.4,491.4,141,150,L));
												else
													return CalcStat("AwardILvlA",150);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLAA" then
											if SN == "AWARDILVLAB" then
												if 75 <= Lp and Lm <= 75 then
													return CalcStat("AwardILvlA",L)+1;
												else
													return CalcStat("AwardILvlA",L);
												end
											else
												return 0;
											end
										else
											if 75 <= Lp and Lm <= 75 then
												return CalcStat("AwardILvlA",L)+3;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									else
										return CalcStat("AwardILvlCC",L);
									end
								elseif SN > "AWARDILVLAC" then
									if SN < "AWARDILVLBOLD" then
										if SN < "AWARDILVLB" then
											if SN == "AWARDILVLAOLD" then
												return CalcStat("AwardILvlA",L);
											else
												return 0;
											end
										elseif SN > "AWARDILVLB" then
											if SN == "AWARDILVLBA" then
												return CalcStat("AwardILvlB",L);
											else
												return 0;
											end
										else
											if Lm <= 4 then
												return CalcStat("AwardILvlA",1);
											elseif Lm <= 50 then
												return CalcStat("AwardILvlA",L-4);
											elseif Lm <= 54 then
												return CalcStat("AwardILvlA",51);
											elseif Lm <= 75 then
												return CalcStat("AwardILvlA",L-4);
											elseif Lm <= 95 then
												return CalcStat("AwardILvlA",L)+4;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									elseif SN > "AWARDILVLBOLD" then
										if SN < "AWARDILVLCA" then
											if SN == "AWARDILVLC" then
												if Lm <= 75 then
													return CalcStat("AwardILvlB",L);
												else
													return CalcStat("AwardILvlA",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLCA" then
											if SN == "AWARDILVLCB" then
												if 96 <= Lp and Lm <= 99 then
													return CalcStat("AwardILvlC",L)+4;
												elseif 100 <= Lp and Lm <= 100 then
													return CalcStat("AwardILvlC",L)+6;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										else
											if 96 <= Lp and Lm <= 100 then
												return CalcStat("AwardILvlC",L)+4;
											else
												return CalcStat("AwardILvlC",L);
											end
										end
									else
										return CalcStat("AwardILvlB",L);
									end
								else
									if 75 <= Lp and Lm <= 75 then
										return CalcStat("AwardILvlA",L)+2;
									else
										return CalcStat("AwardILvlA",L);
									end
								end
							elseif SN > "AWARDILVLCC" then
								if SN < "AWARDILVLDF" then
									if SN < "AWARDILVLDB" then
										if SN < "AWARDILVLD" then
											if SN == "AWARDILVLCD" then
												if 101 <= Lp and Lm <= 105 then
													return CalcStat("AwardILvlC",L)+4;
												else
													return CalcStat("AwardILvlC",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLD" then
											if SN == "AWARDILVLDA" then
												if 106 <= Lp and Lm <= 115 then
													return RoundDbl(LinFmod(1,300,336.5,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 115 then
												return 300;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									elseif SN > "AWARDILVLDB" then
										if SN < "AWARDILVLDD" then
											if SN == "AWARDILVLDC" then
												if 106 <= Lp and Lm <= 110 then
													return RoundDbl(LinFmod(1,300,320,106,115,L));
												elseif 111 <= Lp and Lm <= 115 then
													return CalcStat("AwardILvlDC",110);
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLDD" then
											if SN == "AWARDILVLDE" then
												if 106 <= Lp and Lm <= 115 then
													return RoundDbl(LinFmod(1,300.4,326.4,106,115,L));
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										else
											if 106 <= Lp and Lm <= 114 then
												return RoundDbl(LinFmod(1,300,327,106,115,L));
											elseif 115 <= Lp and Lm <= 115 then
												return CalcStat("AwardILvlD",115)+30;
											else
												return CalcStat("AwardILvlD",L);
											end
										end
									else
										if 106 <= Lp and Lm <= 115 then
											return RoundDbl(LinFmod(1,300,345,106,115,L));
										else
											return CalcStat("AwardILvlD",L);
										end
									end
								elseif SN > "AWARDILVLDF" then
									if SN < "AWARDILVLEB" then
										if SN < "AWARDILVLE" then
											if SN == "AWARDILVLDG" then
												if 106 <= Lp and Lm <= 114 then
													return RoundDbl(LinFmod(1,300,336,106,115,L));
												elseif 115 <= Lp and Lm <= 115 then
													return CalcStat("AwardILvlD",115)+40;
												else
													return CalcStat("AwardILvlD",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLE" then
											if SN == "AWARDILVLEA" then
												if 116 <= Lp and Lm <= 119 then
													return RoundDbl(LinFmod(1,350,378,116,120,L));
												elseif 120 <= Lp and Lm <= 120 then
													return CalcStat("AwardILvlE",120)+26;
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 115 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 120 then
												return 350;
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									elseif SN > "AWARDILVLEB" then
										if SN < "AWARDILVLED" then
											if SN == "AWARDILVLEC" then
												if 116 <= Lp and Lm <= 120 then
													return RoundDbl(LinFmod(1,350,382,116,120,L));
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLED" then
											if SN == "AWARDILVLEE" then
												if 116 <= Lp and Lm <= 119 then
													return RoundDbl(LinFmod(1,350,366,116,120,L));
												elseif 120 <= Lp and Lm <= 120 then
													return CalcStat("AwardILvlE",120)+14;
												else
													return CalcStat("AwardILvlE",L);
												end
											else
												return 0;
											end
										else
											if 116 <= Lp and Lm <= 120 then
												return RoundDbl(LinFmod(1,350,370,116,120,L));
											else
												return CalcStat("AwardILvlE",L);
											end
										end
									else
										if 116 <= Lp and Lm <= 119 then
											return RoundDbl(LinFmod(1,350,390,116,120,L));
										elseif 120 <= Lp and Lm <= 120 then
											return CalcStat("AwardILvlE",120)+38;
										else
											return CalcStat("AwardILvlE",L);
										end
									end
								else
									if 106 <= Lp and Lm <= 110 then
										return CalcStat("AwardILvlDF",111);
									elseif 111 <= Lp and Lm <= 115 then
										return RoundDbl(LinFmod(1,299.5,320.5,106,115,L));
									else
										return CalcStat("AwardILvlD",L);
									end
								end
							else
								return CalcStat("AwardILvlC",L);
							end
						else
							if Lm <= 0 then
								return 0;
							elseif Lm <= 1 then
								return CalcStat("ArmQtyCommMP",N);
							elseif Lm <= 2 then
								return CalcStat("ArmQtyUncomMP",N);
							elseif Lm <= 3 then
								return CalcStat("ArmQtyRareMP",N);
							elseif Lm <= 4 then
								return CalcStat("ArmQtyIncompMP",N);
							else
								return CalcStat("ArmQtyEpicMP",N);
							end
						end
					elseif SN > "AWARDILVLF" then
						if SN < "BASEMORALE" then
							if SN < "AWARDILVLJB" then
								if SN < "AWARDILVLGD" then
									if SN < "AWARDILVLG" then
										if SN > "AWARDILVLFA" then
											if SN == "AWARDILVLFB" then
												if Lm <= 140 then
													return CalcStat("AwardILvlF",L);
												elseif Lm <= 150 then
													return RoundDbl(LinFmod(1,500.4,510.4,141,150,L));
												else
													return CalcStat("AwardILvlFB",150);
												end
											else
												return 0;
											end
										elseif SN == "AWARDILVLFA" then
											if Lm <= 140 then
												return CalcStat("AwardILvlF",L);
											else
												return 500;
											end
										else
											return 0;
										end
									elseif SN > "AWARDILVLG" then
										if SN < "AWARDILVLGB" then
											if SN == "AWARDILVLGA" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,412.4,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLGB" then
											if SN == "AWARDILVLGC" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,408.3,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										else
											if 121 <= Lp and Lm <= 130 then
												return RoundDbl(LinFmod(1,400.4,418.4,121,130,L));
											else
												return CalcStat("AwardILvlG",L);
											end
										end
									else
										if Lm <= 120 then
											return CalcStat("AwardILvlC",L);
										elseif Lm <= 130 then
											return 400;
										else
											return CalcStat("AwardILvlA",L);
										end
									end
								elseif SN > "AWARDILVLGD" then
									if SN < "AWARDILVLI" then
										if SN < "AWARDILVLGF" then
											if SN == "AWARDILVLGE" then
												if 121 <= Lp and Lm <= 130 then
													return RoundDbl(LinFmod(1,400.4,434.4,121,130,L));
												else
													return CalcStat("AwardILvlG",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLGF" then
											if SN == "AWARDILVLH" then
												if Lm <= 4 then
													return CalcStat("AwardILvlA",1);
												elseif Lm <= 50 then
													return CalcStat("AwardILvlA",L-4);
												elseif Lm <= 54 then
													return CalcStat("AwardILvlA",51);
												elseif Lm <= 75 then
													return CalcStat("AwardILvlA",L-4);
												else
													return L-4;
												end
											else
												return 0;
											end
										else
											if 121 <= Lp and Lm <= 130 then
												return RoundDbl(LinFmod(1,400.4,440.4,121,130,L));
											else
												return CalcStat("AwardILvlG",L);
											end
										end
									elseif SN > "AWARDILVLI" then
										if SN < "AWARDILVLJ" then
											if SN == "AWARDILVLIA" then
												if Lm <= 44 then
													return CalcStat("AwardILvlI",L);
												elseif Lm <= 55 then
													return RoundDblDown((L-45)/6)+51;
												elseif Lm <= 90 then
													return CalcStat("AwardILvlI",L);
												elseif Lm <= 104 then
													return RoundDblDown((L-91)/5)*25+165;
												elseif Lm <= 105 then
													return CalcStat("LvlToILvl",L);
												elseif Lm <= 110 then
													return CalcStat("LvlToILvl",106)+15;
												elseif Lm <= 115 then
													return CalcStat("LvlToILvl",115)-20;
												elseif Lm <= 120 then
													return CalcStat("LvlToILvl",116)+15;
												elseif Lm <= 125 then
													return CalcStat("LvlToILvl",121)+15;
												elseif Lm <= 130 then
													return CalcStat("LvlToILvl",130)-20;
												elseif Lm <= 135 then
													return CalcStat("LvlToILvl",131)+15;
												elseif Lm <= 140 then
													return CalcStat("LvlToILvl",140)-4;
												elseif Lm <= 150 then
													return CalcStat("LvlToILvl",141)+15;
												else
													return CalcStat("AwardILvlIA",150);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLJ" then
											if SN == "AWARDILVLJA" then
												if 131 <= Lp and Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,465.4,131,140,L));
												else
													return CalcStat("AwardILvlJ",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 130 then
												return CalcStat("AwardILvlC",L);
											elseif Lm <= 140 then
												return RoundDbl(LinFmod(1,450.4,460.4,131,140,L));
											else
												return CalcStat("AwardILvlA",L);
											end
										end
									else
										if Lm <= 44 then
											return 1;
										elseif Lm <= 55 then
											return 52;
										elseif Lm <= 75 then
											return RoundDblDown((L-56)/5)*5+60;
										elseif Lm <= 85 then
											return RoundDblDown((L-76)/5)*29+100;
										elseif Lm <= 95 then
											return RoundDblDown((L-86)/5)*20+155;
										elseif Lm <= 100 then
											return RoundDblDown((L-96)/4)*10+190;
										elseif Lm <= 105 then
											return RoundDblDown((L-101)/4)*35+215;
										elseif Lm <= 110 then
											return CalcStat("LvlToILvl",106)+15;
										elseif Lm <= 115 then
											return CalcStat("LvlToILvl",115);
										elseif Lm <= 119 then
											return CalcStat("LvlToILvl",116)+15;
										elseif Lm <= 120 then
											return CalcStat("LvlToILvl",120);
										elseif Lm <= 125 then
											return CalcStat("LvlToILvl",121)+15;
										elseif Lm <= 130 then
											return CalcStat("LvlToILvl",130);
										elseif Lm <= 135 then
											return CalcStat("LvlToILvl",131)+15;
										elseif Lm <= 140 then
											return CalcStat("LvlToILvl",140);
										elseif Lm <= 145 then
											return CalcStat("LvlToILvl",141)+15;
										elseif Lm <= 150 then
											return CalcStat("LvlToILvl",150);
										else
											return CalcStat("AwardILvlI",150);
										end
									end
								else
									if 121 <= Lp and Lm <= 130 then
										return RoundDbl(LinFmod(1,400.4,426.4,121,130,L));
									else
										return CalcStat("AwardILvlG",L);
									end
								end
							elseif SN > "AWARDILVLJB" then
								if SN < "BALANCEOFMANEVADE" then
									if SN < "AWARDLVLTOILVL" then
										if SN < "AWARDILVLJD" then
											if SN == "AWARDILVLJC" then
												if 131 <= Lp and Lm <= 140 then
													return RoundDbl(LinFmod(1,450.4,475.4,131,140,L));
												else
													return CalcStat("AwardILvlJ",L);
												end
											else
												return 0;
											end
										elseif SN > "AWARDILVLJD" then
											if SN == "AWARDLVLCAP" then
												return 150;
											else
												return 0;
											end
										else
											if 131 <= Lp and Lm <= 140 then
												return RoundDbl(LinFmod(1,450.4,480.4,131,140,L));
											else
												return CalcStat("AwardILvlJ",L);
											end
										end
									elseif SN > "AWARDLVLTOILVL" then
										if SN < "AXEARMRENDPOS" then
											if SN == "AXEARMOURREND" then
												return -CalcStat("AxeArmRendPos",L);
											else
												return 0;
											end
										elseif SN > "AXEARMRENDPOS" then
											if SN == "BALANCEOFMANBLOCK" then
												return CalcStat("BlockT",L,0.8);
											else
												return 0;
											end
										else
											if Lm <= 7 then
												return RoundDbl(LinFmod(1,0.5,3.25,1,7,L));
											elseif Lm <= 14 then
												return RoundDbl(LinFmod(1,3.75,8.5,8,14,L));
											elseif Lm <= 30 then
												return RoundDbl(LinFmod(1,9,24.7,15,30,L));
											elseif Lm <= 39 then
												return RoundDbl(LinFmod(1,26.5,37,31,39,L));
											elseif Lm <= 50 then
												return RoundDbl(LinFmod(1,38.5,53,40,50,L));
											elseif Lm <= 52 then
												return 53;
											elseif Lm <= 60 then
												return RoundDbl(LinFmod(1,53.2,64.8,53,60,L));
											elseif Lm <= 77 then
												return RoundDbl(LinFmod(1,66.15,94.8,61,77,L));
											elseif Lm <= 113 then
												return RoundDbl(RoundDbl(L/3-0.5)*1.95+44.1);
											elseif Lm <= 141 then
												return 116;
											elseif Lm <= 181 then
												return RoundDbl(RoundDbl(L/4)*2+46);
											elseif Lm <= 217 then
												return RoundDbl(RoundDbl(L/4)*2+45);
											elseif Lm <= 221 then
												return 153;
											elseif Lm <= 299 then
												return 155;
											elseif Lm <= 308 then
												return RoundDbl(RoundDbl(L/2.6-0.4)*2-73);
											elseif Lm <= 317 then
												return RoundDbl(RoundDbl(L/2.74-0.75)*2-60);
											elseif Lm <= 325 then
												return 172;
											elseif Lm <= 326 then
												return 174;
											elseif Lm <= 599 then
												return ExpFmod(CalcStat("AxeArmRendPos",326),327,1,L,1);
											else
												return CalcStat("AxeArmRendPos",599);
											end
										end
									else
										return CalcStat("AwardILvlC",L);
									end
								elseif SN > "BALANCEOFMANEVADE" then
									if SN < "BASEFATE" then
										if SN < "BASEAGILITY" then
											if SN == "BALANCEOFMANPARRY" then
												return CalcStat("ParryT",L,0.8);
											else
												return 0;
											end
										elseif SN > "BASEAGILITY" then
											if SN == "BASEARMOUR" then
												if Lm <= 50 then
													return RoundDblUp(LinFmod(1,1,10,1,50,L)*N);
												elseif Lm <= 60 then
													return RoundDblUp(LinFmod(1,10,15,50,60,L)*N);
												elseif Lm <= 65 then
													return RoundDblUp(LinFmod(1,15,20,60,65,L)*N);
												elseif Lm <= 75 then
													return RoundDblUp(LinFmod(1,20,30,65,75,L)*N);
												elseif Lm <= 85 then
													return RoundDblUp(LinFmod(1,30,45,75,85,L)*N);
												elseif Lm <= 95 then
													return RoundDblUp(LinFmod(1,45,65,85,95,L)*N);
												elseif Lm <= 100 then
													return RoundDblUp(LinFmod(1,65,100,95,100,L)*N);
												elseif Lm <= 105 then
													return RoundDblUp(LinFmod(1,100,130,100,105,L)*N);
												elseif Lm <= 115 then
													return RoundDblUp(LinFmod(1,143,200,106,115,L,0)*N);
												elseif Lm <= 120 then
													return RoundDblUp(LinFmod(1,230,250,116,120,L,0)*N);
												elseif Lm <= 130 then
													return RoundDblUp(LinFmod(1,288,380,121,130,L,0)*N);
												elseif Lm <= 140 then
													return RoundDblUp(LinFmod(1,440,750,131,140,L,0)*N);
												else
													return CalcStat("BaseArmour",140,N);
												end
											else
												return 0;
											end
										else
											return CalcStat("BaseMain",L,N);
										end
									elseif SN > "BASEFATE" then
										if SN < "BASEMAINPROG" then
											if SN == "BASEMAIN" then
												if Lm <= 50 then
													return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
												elseif Lm <= 95 then
													return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
												else
													return RoundDbl(CalcStat("BaseMainProg",L)*1*N);
												end
											else
												return 0;
											end
										elseif SN > "BASEMAINPROG" then
											if SN == "BASEMIGHT" then
												return CalcStat("BaseMain",L,N);
											else
												return 0;
											end
										else
											if Lm <= 95 then
												return CalcStat("StdProg",L,1.0);
											elseif Lm <= 140 then
												return RoundDbl(CalcStat("StdProg",L,1.0));
											else
												return RoundDbl(LinFmod(1,780,1360,141,150,L));
											end
										end
									else
										if Lm <= 50 then
											return RoundDbl(CalcStat("ProgBEnergy",L)*3.5);
										elseif Lm <= 95 then
											return RoundDbl(CalcStat("ProgBEnergy",L)*3.5);
										else
											return RoundDbl(CalcStat("BasePower",L)*0.7);
										end
									end
								else
									return CalcStat("EvadeT",L,1.0);
								end
							else
								if 131 <= Lp and Lm <= 140 then
									return RoundDbl(LinFmod(1,450.4,470.4,131,140,L));
								else
									return CalcStat("AwardILvlJ",L);
								end
							end
						elseif SN > "BASEMORALE" then
							if SN < "BEORNINGCDAGILITYTOOUTHEAL" then
								if SN < "BEOEMISSARYFATE" then
									if SN < "BATTLELOREMAS" then
										if SN < "BASEVITALITY" then
											if SN == "BASEPOWER" then
												if Lm <= 50 then
													return RoundDblDown(CalcStat("ProgBEnergy",L)*5);
												elseif Lm <= 95 then
													return RoundDblDown(CalcStat("ProgBEnergy",L)*5);
												else
													return RoundDbl(CalcStat("ProgBEnergy",L)*5);
												end
											else
												return 0;
											end
										elseif SN > "BASEVITALITY" then
											if SN == "BASEWILL" then
												return CalcStat("BaseMain",L,N);
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return RoundDbl(CalcStat("ProgBHealth",L)*0.75);
											elseif Lm <= 95 then
												return RoundDbl(CalcStat("ProgBHealth",L)*0.75);
											else
												return RoundDbl(CalcStat("BaseMorale",L)*0.15);
											end
										end
									elseif SN > "BATTLELOREMAS" then
										if SN < "BATTLELORETACMAS" then
											if SN == "BATTLELOREPHYMAS" then
												return CalcStat("BattleLoreMas",L);
											else
												return 0;
											end
										elseif SN > "BATTLELORETACMAS" then
											if SN == "BEOBEARFORMCRITDEF" then
												return CalcStat("CritDefT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("BattleLoreMas",L);
										end
									else
										if Lm <= 105 then
											return CalcStat("Mastery",L,2.5);
										elseif Lm <= 119 then
											return CalcStat("MasteryT",L,1.2);
										elseif Lm <= 120 then
											return CalcStat("MasteryT",L,1.6);
										elseif Lm <= 129 then
											return CalcStat("MasteryT",L,1.2);
										elseif Lm <= 130 then
											return CalcStat("MasteryT",L,1.6);
										elseif Lm <= 139 then
											return CalcStat("MasteryT",L,1.2);
										else
											return CalcStat("MasteryT",L,1.6);
										end
									end
								elseif SN > "BEOEMISSARYFATE" then
									if SN < "BEOMIGHTOFTHEWILDMIGHT" then
										if SN < "BEOFERALPRESFATE" then
											if SN == "BEOFATE" then
												return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "BEOFERALPRESFATE" then
											if SN == "BEOFEWINNUMBERFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("FateT",L,1.0);
										end
									elseif SN > "BEOMIGHTOFTHEWILDMIGHT" then
										if SN < "BEORNINGCDAGILITYTOEVADE" then
											if SN == "BEORNINGCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDAGILITYTOEVADE" then
											if SN == "BEORNINGCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("MightT",L,1.0);
									end
								else
									return CalcStat("FateT",L,1.0);
								end
							elseif SN > "BEORNINGCDAGILITYTOOUTHEAL" then
								if SN < "BEORNINGCDBASEICMR" then
									if SN < "BEORNINGCDARMOURTOTACMIT" then
										if SN < "BEORNINGCDARMOURTOCOMPHYMIT" then
											if SN == "BEORNINGCDAGILITYTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDARMOURTOCOMPHYMIT" then
											if SN == "BEORNINGCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGCDARMOURTOTACMIT" then
										if SN < "BEORNINGCDBASEAGILITY" then
											if SN == "BEORNINGCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEAGILITY" then
											if SN == "BEORNINGCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityM",L);
										end
									else
										return 0.2;
									end
								elseif SN > "BEORNINGCDBASEICMR" then
									if SN < "BEORNINGCDBASENCMR" then
										if SN < "BEORNINGCDBASEMIGHT" then
											if SN == "BEORNINGCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEMIGHT" then
											if SN == "BEORNINGCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMightM",L);
										end
									elseif SN > "BEORNINGCDBASENCMR" then
										if SN < "BEORNINGCDBASEPOWER" then
											if SN == "BEORNINGCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										elseif SN > "BEORNINGCDBASEPOWER" then
											if SN == "BEORNINGCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										else
											return 10;
										end
									else
										return CalcStat("ClassBaseNCMRM",L);
									end
								else
									return CalcStat("ClassBaseICMRM",L);
								end
							else
								return 2;
							end
						else
							if Lm <= 50 then
								return RoundDblDown(CalcStat("ProgBHealth",L)*5);
							elseif Lm <= 95 then
								return RoundDbl(CalcStat("ProgBHealth",L)*5);
							else
								return RoundDbl(CalcStat("ProgBHealth",L)*5,-1);
							end
						end
					else
						if Lm <= 40 then
							return LinFmod(1,1,40,1,40,L);
						elseif Lm <= 41 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 49 then
							return LinFmod(1,41,48,42,49,L);
						elseif Lm <= 50 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 55 then
							return LinFmod(1,51,55,51,55,L);
						elseif Lm <= 56 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 59 then
							return LinFmod(1,56,58,57,59,L);
						elseif Lm <= 60 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 64 then
							return RoundDbl(LinFmod(1,60.6,63,61,64,L));
						elseif Lm <= 65 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 70 then
							return LinFmod(1,66,70,66,70,L);
						elseif Lm <= 71 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 74 then
							return LinFmod(1,71,73,72,74,L);
						elseif Lm <= 75 then
							return CalcStat("AwardILvlF",L-1);
						elseif Lm <= 85 then
							return RoundDbl(LinFmod(1,80.4,123.4,76,85,L));
						elseif Lm <= 94 then
							return RoundDbl(LinFmod(1,130,169,86,94,L));
						elseif Lm <= 100 then
							return RoundDbl(LinFmod(1,175,198.4,95,100,L));
						elseif Lm <= 104 then
							return LinFmod(1,201,213,101,104,L);
						elseif Lm <= 105 then
							return 220;
						elseif Lm <= 114 then
							return RoundDbl(LinFmod(1,300,323,106,114,L));
						elseif Lm <= 115 then
							return 324;
						elseif Lm <= 119 then
							return RoundDbl(LinFmod(1,349.6,368.4,116,120,L));
						elseif Lm <= 120 then
							return 380;
						elseif Lm <= 125 then
							return RoundDbl(LinFmod(1,400.4,416.4,121,125,L));
						elseif Lm <= 129 then
							return RoundDbl(LinFmod(1,418.4,430.4,126,129,L));
						elseif Lm <= 130 then
							return 432;
						elseif Lm <= 135 then
							return RoundDbl(LinFmod(1,450.4,463.8,131,135,L));
						elseif Lm <= 138 then
							return RoundDbl(LinFmod(1,466,472.5,136,138,L));
						else
							return 475;
						end
					end
				elseif SN > "BEORNINGCDBASEWILL" then
					if SN < "BRATDEVHIT" then
						if SN < "BEORNINGRDTRAITMIGHT" then
							if SN < "BEORNINGCDPHYMITTOCOMPHYMIT" then
								if SN < "BEORNINGCDFATETONCMR" then
									if SN < "BEORNINGCDCALCTYPETACMIT" then
										if SN > "BEORNINGCDCALCTYPECOMPHYMIT" then
											if SN == "BEORNINGCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN == "BEORNINGCDCALCTYPECOMPHYMIT" then
											return 14;
										else
											return 0;
										end
									elseif SN > "BEORNINGCDCALCTYPETACMIT" then
										if SN < "BEORNINGCDFATETOICMR" then
											if SN == "BEORNINGCDCANBLOCK" then
												if Lm <= 5 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "BEORNINGCDFATETOICMR" then
											if SN == "BEORNINGCDFATETOMORALE" then
												return 2.5;
											else
												return 0;
											end
										else
											return 0.04;
										end
									else
										return 27;
									end
								elseif SN > "BEORNINGCDFATETONCMR" then
									if SN < "BEORNINGCDMIGHTTOPARRY" then
										if SN < "BEORNINGCDMIGHTTOEVADE" then
											if SN == "BEORNINGCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDMIGHTTOEVADE" then
											if SN == "BEORNINGCDMIGHTTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "BEORNINGCDMIGHTTOPARRY" then
										if SN < "BEORNINGCDMIGHTTOPHYMIT" then
											if SN == "BEORNINGCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDMIGHTTOPHYMIT" then
											if SN == "BEORNINGCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 0.12;
								end
							elseif SN > "BEORNINGCDPHYMITTOCOMPHYMIT" then
								if SN < "BEORNINGCDWILLTOPHYMAS" then
									if SN < "BEORNINGCDVITALITYTOMORALE" then
										if SN < "BEORNINGCDTACMASTOOUTHEAL" then
											if SN == "BEORNINGCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDTACMASTOOUTHEAL" then
											if SN == "BEORNINGCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGCDVITALITYTOMORALE" then
										if SN < "BEORNINGCDWILLTOFINESSE" then
											if SN == "BEORNINGCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDWILLTOFINESSE" then
											if SN == "BEORNINGCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 4.5;
									end
								elseif SN > "BEORNINGCDWILLTOPHYMAS" then
									if SN < "BEORNINGRDPSVONEFATE" then
										if SN < "BEORNINGCDWILLTORESIST" then
											if SN == "BEORNINGCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "BEORNINGCDWILLTORESIST" then
											if SN == "BEORNINGCDWILLTOTACMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "BEORNINGRDPSVONEFATE" then
										if SN < "BEORNINGRDPSVTWONAME" then
											if SN == "BEORNINGRDPSVONENAME" then
												return "Emissary";
											else
												return 0;
											end
										elseif SN > "BEORNINGRDPSVTWONAME" then
											if SN == "BEORNINGRDTRAITFATE" then
												return CalcStat("BeoFewinNumberFate",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return CalcStat("BeoEmissaryFate",L);
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "BEORNINGRDTRAITMIGHT" then
							if SN < "BLOCKPRATPC" then
								if SN < "BLACKARROWCDHASPOWER" then
									if SN < "BLACKARROWCANBLOCK" then
										if SN < "BEOTHICKHIDEVITALITY" then
											if SN == "BEORNINGRDTRAITVITALITY" then
												return CalcStat("BeoThickHideVitality",L);
											else
												return 0;
											end
										elseif SN > "BEOTHICKHIDEVITALITY" then
											if SN == "BEOVITALITYINCREASE" then
												return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									elseif SN > "BLACKARROWCANBLOCK" then
										if SN < "BLACKARROWCDCALCTYPENONPHYMIT" then
											if SN == "BLACKARROWCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "BLACKARROWCDCALCTYPENONPHYMIT" then
											if SN == "BLACKARROWCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return 1;
									end
								elseif SN > "BLACKARROWCDHASPOWER" then
									if SN < "BLOCKPPRAT" then
										if SN < "BLOCKC" then
											if SN == "BLOCK" then
												return CalcStat("BPE",L,N);
											else
												return 0;
											end
										elseif SN > "BLOCKC" then
											if SN == "BLOCKPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEC",L,N);
										end
									elseif SN > "BLOCKPPRAT" then
										if SN < "BLOCKPRATPA" then
											if SN == "BLOCKPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "BLOCKPRATPA" then
											if SN == "BLOCKPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPA",L);
										end
									else
										return CalcStat("BPEPPRat",L,N);
									end
								else
									return 1;
								end
							elseif SN > "BLOCKPRATPC" then
								if SN < "BPEPRATP" then
									if SN < "BPE" then
										if SN < "BLOCKPRATPCAPR" then
											if SN == "BLOCKPRATPCAP" then
												return CalcStat("BPEPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "BLOCKPRATPCAPR" then
											if SN == "BLOCKT" then
												return CalcStat("BPET",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPCapR",L);
										end
									elseif SN > "BPE" then
										if SN < "BPEPBONUS" then
											if SN == "BPEC" then
												return StatLinInter("PntMPBPEC","CreepProg","ProgBBPEC","",L,N,0);
											else
												return 0;
											end
										elseif SN > "BPEPBONUS" then
											if SN == "BPEPPRAT" then
												return CalcRatAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return StatLinInter("PntMPBPE","ItemProg","ProgBBPE","AdjUmbarItem",L,N,0);
									end
								elseif SN > "BPEPRATP" then
									if SN < "BPEPRATPCAP" then
										if SN < "BPEPRATPB" then
											if SN == "BPEPRATPA" then
												return 39;
											else
												return 0;
											end
										elseif SN > "BPEPRATPB" then
											if SN == "BPEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									elseif SN > "BPEPRATPCAP" then
										if SN < "BPET" then
											if SN == "BPEPRATPCAPR" then
												return CalcStat("BPEPRatPB",L)*CalcStat("BPEPRatPC",L);
											else
												return 0;
											end
										elseif SN > "BPET" then
											if SN == "BRATCRITMAGN" then
												return CalcStat("StdProg",L,600);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPBPE","TraitProg","ProgBBPE","AdjUmbarTrait",L,N,0);
										end
									else
										return 13;
									end
								else
									return CalcPercAB(CalcStat("BPEPRatPA",L),CalcStat("BPEPRatPB",L),CalcStat("BPEPRatPCap",L),N);
								end
							else
								return CalcStat("BPEPRatPC",L);
							end
						else
							return CalcStat("BeoMightoftheWildMight",L);
						end
					elseif SN > "BRATDEVHIT" then
						if SN < "BRAWLERCDCALCTYPECOMPHYMIT" then
							if SN < "BRAWLERCDAGILITYTOPHYMAS" then
								if SN < "BRATPARTBPE" then
									if SN < "BRATMITIGATIONS" then
										if SN > "BRATEXTRA" then
											if SN == "BRATMITHEAVY" then
												return CalcStat("BRatStandard",L);
											else
												return 0;
											end
										elseif SN == "BRATEXTRA" then
											return CalcStat("StdProg",L,300);
										else
											return 0;
										end
									elseif SN > "BRATMITIGATIONS" then
										if SN < "BRATMITMEDIUM" then
											if SN == "BRATMITLIGHT" then
												return CalcStat("BRatMitigations",L,0.666);
											else
												return 0;
											end
										elseif SN > "BRATMITMEDIUM" then
											if SN == "BRATOUTHEAL" then
												return CalcStat("StdProg",L,450);
											else
												return 0;
											end
										else
											return CalcStat("BRatMitigations",L,0.833);
										end
									else
										if Lm <= 50 then
											return LinFmod(1,(N*CalcStat("BRatStandard",1))*7/6-50.4,N*CalcStat("BRatStandard",50),1,50,L,"P");
										elseif Lm <= 60 then
											return LinFmod(N,CalcStat("BRatStandard",50),CalcStat("BRatStandard",60),50,60,L,"P");
										elseif Lm <= 65 then
											return LinFmod(N,CalcStat("BRatStandard",60),CalcStat("BRatStandard",65),60,65,L,"P");
										elseif Lm <= 75 then
											return LinFmod(N,CalcStat("BRatStandard",65),CalcStat("BRatStandard",75),65,75,L,"P");
										elseif Lm <= 85 then
											return LinFmod(N,CalcStat("BRatStandard",75),CalcStat("BRatStandard",85),75,85,L,"P");
										elseif Lm <= 95 then
											return LinFmod(N,CalcStat("BRatStandard",85),CalcStat("BRatStandard",95),85,95,L,"P");
										elseif Lm <= 100 then
											return LinFmod(N,CalcStat("BRatStandard",95),CalcStat("BRatStandard",100),95,100,L,"P");
										elseif Lm <= 105 then
											return LinFmod(N,CalcStat("BRatStandard",100),CalcStat("BRatStandard",105),100,105,L,"P");
										elseif Lm <= 115 then
											return LinFmod(N,CalcStat("BRatStandard",106),CalcStat("BRatStandard",115),106,115,L,"P");
										elseif Lm <= 120 then
											return LinFmod(N,CalcStat("BRatStandard",116),CalcStat("BRatStandard",120),116,120,L,"P");
										elseif Lm <= 130 then
											return LinFmod(N,CalcStat("BRatStandard",121),CalcStat("BRatStandard",130),121,130,L,"P");
										elseif Lm <= 140 then
											return LinFmod(N,CalcStat("BRatStandard",131),CalcStat("BRatStandard",140),131,140,L,"P");
										elseif Lm <= 150 then
											return LinFmod(N,CalcStat("BRatStandard",141),CalcStat("BRatStandard",150),141,150,L,"P");
										else
											return LinFmod(N,CalcStat("BRatStandard",151),CalcStat("BRatStandard",160),151,160,L,"P");
										end
									end
								elseif SN > "BRATPARTBPE" then
									if SN < "BRAWLERCDAGILITYTOCRITHIT" then
										if SN < "BRATROUNDED" then
											if SN == "BRATPROGB" then
												if Lm <= 50 then
													return RoundDbl(CalcStat(C,L));
												else
													return CalcStat(C,L);
												end
											else
												return 0;
											end
										elseif SN > "BRATROUNDED" then
											if SN == "BRATSTANDARD" then
												return CalcStat("StdProg",L,200);
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return RoundDbl(CalcStat(C,L));
											elseif Lm <= 105 then
												return RoundDbl(CalcStat(C,L),-1);
											elseif Lm <= 115 then
												return RoundDbl(CalcStat(C,L),-2);
											elseif Lm <= 130 then
												return RoundDbl(CalcStat(C,L),-1);
											elseif Lm <= 150 then
												return RoundDbl(CalcStat(C,L),-2);
											else
												return RoundDbl(CalcStat(C,L));
											end
										end
									elseif SN > "BRAWLERCDAGILITYTOCRITHIT" then
										if SN < "BRAWLERCDAGILITYTOFINESSE" then
											if SN == "BRAWLERCDAGILITYTOEVADE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDAGILITYTOFINESSE" then
											if SN == "BRAWLERCDAGILITYTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								else
									return CalcStat("StdProg",L,350);
								end
							elseif SN > "BRAWLERCDAGILITYTOPHYMAS" then
								if SN < "BRAWLERCDBASEICPR" then
									if SN < "BRAWLERCDARMOURTYPE" then
										if SN < "BRAWLERCDARMOURTONONPHYMIT" then
											if SN == "BRAWLERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDARMOURTONONPHYMIT" then
											if SN == "BRAWLERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "BRAWLERCDARMOURTYPE" then
										if SN < "BRAWLERCDBASEFATE" then
											if SN == "BRAWLERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEFATE" then
											if SN == "BRAWLERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									else
										return 3;
									end
								elseif SN > "BRAWLERCDBASEICPR" then
									if SN < "BRAWLERCDBASENCPR" then
										if SN < "BRAWLERCDBASEMORALE" then
											if SN == "BRAWLERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEMORALE" then
											if SN == "BRAWLERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									elseif SN > "BRAWLERCDBASENCPR" then
										if SN < "BRAWLERCDBASEVITALITY" then
											if SN == "BRAWLERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "BRAWLERCDBASEVITALITY" then
											if SN == "BRAWLERCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									else
										return CalcStat("ClassBaseNCPR",L);
									end
								else
									return CalcStat("ClassBaseICPR",L);
								end
							else
								return 2;
							end
						elseif SN > "BRAWLERCDCALCTYPECOMPHYMIT" then
							if SN < "BRAWLERCDVITALITYTOICMR" then
								if SN < "BRAWLERCDMIGHTTOOUTHEAL" then
									if SN < "BRAWLERCDFATETOPOWER" then
										if SN < "BRAWLERCDCALCTYPETACMIT" then
											if SN == "BRAWLERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDCALCTYPETACMIT" then
											if SN == "BRAWLERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "BRAWLERCDFATETOPOWER" then
										if SN < "BRAWLERCDMIGHTTOCRITHIT" then
											if SN == "BRAWLERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDMIGHTTOCRITHIT" then
											if SN == "BRAWLERCDMIGHTTOEVADE" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "BRAWLERCDMIGHTTOOUTHEAL" then
									if SN < "BRAWLERCDMIGHTTOTACMIT" then
										if SN < "BRAWLERCDMIGHTTOPHYMAS" then
											if SN == "BRAWLERCDMIGHTTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDMIGHTTOPHYMAS" then
											if SN == "BRAWLERCDMIGHTTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "BRAWLERCDMIGHTTOTACMIT" then
										if SN < "BRAWLERCDPHYMITTONONPHYMIT" then
											if SN == "BRAWLERCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDPHYMITTONONPHYMIT" then
											if SN == "BRAWLERCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 3;
								end
							elseif SN > "BRAWLERCDVITALITYTOICMR" then
								if SN < "BRAWLERCDWILLTOTACMIT" then
									if SN < "BRAWLERCDWILLTOOUTHEAL" then
										if SN < "BRAWLERCDVITALITYTONCMR" then
											if SN == "BRAWLERCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDVITALITYTONCMR" then
											if SN == "BRAWLERCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.12;
										end
									elseif SN > "BRAWLERCDWILLTOOUTHEAL" then
										if SN < "BRAWLERCDWILLTOPHYMIT" then
											if SN == "BRAWLERCDWILLTOPHYMAS" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BRAWLERCDWILLTOPHYMIT" then
											if SN == "BRAWLERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1;
									end
								elseif SN > "BRAWLERCDWILLTOTACMIT" then
									if SN < "BRGREVWEAKNFINESSE" then
										if SN < "BRGALLINONEXPFINESSE" then
											if SN == "BRGALLINFINESSE" then
												return CalcStat("Finesse",L,2);
											else
												return 0;
											end
										elseif SN > "BRGALLINONEXPFINESSE" then
											if SN == "BRGREVWEAKNCRITDEF" then
												return -CalcStat("CritDefT",L,0.4);
											else
												return 0;
											end
										else
											return -CalcStat("Finesse",L,2);
										end
									elseif SN > "BRGREVWEAKNFINESSE" then
										if SN < "BRGTRICKCNTDEFBPE" then
											if SN == "BRGREVWEAKNRESIST" then
												return -CalcStat("ResistT",L,0.4);
											else
												return 0;
											end
										elseif SN > "BRGTRICKCNTDEFBPE" then
											if SN == "BRGTRICKCNTDEFCRITDEF" then
												return -CalcStat("CritDefT",L,0.8);
											else
												return 0;
											end
										else
											return -CalcStat("BPET",L,1.2);
										end
									else
										return -CalcStat("FinesseT",L,0.4);
									end
								else
									return 1.5;
								end
							else
								return 0.012;
							end
						else
							return 14;
						end
					else
						return CalcStat("StdProg",L,400);
					end
				else
					return CalcStat("ClassBaseWillM",L);
				end
			elseif SN > "BRWAGGPOSTUREPHYMIT" then
				if SN < "CHAMPIONCDCALCTYPECOMPHYMIT" then
					if SN < "CAPTAINCDAGILITYTOTACMAS" then
						if SN < "BURGLARCDBASENCPR" then
							if SN < "BURGLARCDAGILITYTOOUTHEAL" then
								if SN < "BRWRETPRECISIONFINESSE" then
									if SN < "BRWINNSTRPRECISIONCRITHIT" then
										if SN > "BRWDEFPOSTUREPHYMAS" then
											if SN == "BRWINNSTRCLEVTECHFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait13510Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN == "BRWDEFPOSTUREPHYMAS" then
											return -CalcStat("PhyMasT",L,4);
										else
											return 0;
										end
									elseif SN > "BRWINNSTRPRECISIONCRITHIT" then
										if SN < "BRWMIGHTINCREASE" then
											if SN == "BRWMAELSTROMMIGHT" then
												return CalcStat("MightT",L,2);
											else
												return 0;
											end
										elseif SN > "BRWMIGHTINCREASE" then
											if SN == "BRWRETINTENSITYCRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait234Choice",N));
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return CalcStat("CritHitT",L,CalcStat("Trait13510Choice",N)*0.2);
									end
								elseif SN > "BRWRETPRECISIONFINESSE" then
									if SN < "BRWTACMIT" then
										if SN < "BRWSHAREISHEAVYCRITHIT" then
											if SN == "BRWSHAREISBALANCEFINESSE" then
												return CalcStat("BrwInnStrClevTechFinesse",L,3);
											else
												return 0;
											end
										elseif SN > "BRWSHAREISHEAVYCRITHIT" then
											if SN == "BRWSHATTFISTARMOUR" then
												return -CalcStat("ArmourT",L,1.6);
											else
												return 0;
											end
										else
											return CalcStat("BrwInnStrPrecisionCritHit",L,3);
										end
									elseif SN > "BRWTACMIT" then
										if SN < "BURGLARCDAGILITYTOCRITHIT" then
											if SN == "BRWVITALITYINCREASE" then
												return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "BURGLARCDAGILITYTOCRITHIT" then
											if SN == "BURGLARCDAGILITYTOEVADE" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.4);
									end
								else
									return CalcStat("FinesseT",L,CalcStat("Trait234Choice",N));
								end
							elseif SN > "BURGLARCDAGILITYTOOUTHEAL" then
								if SN < "BURGLARCDARMOURTYPE" then
									if SN < "BURGLARCDAGILITYTOTACMIT" then
										if SN < "BURGLARCDAGILITYTOPHYMAS" then
											if SN == "BURGLARCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDAGILITYTOPHYMAS" then
											if SN == "BURGLARCDAGILITYTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "BURGLARCDAGILITYTOTACMIT" then
										if SN < "BURGLARCDARMOURTONONPHYMIT" then
											if SN == "BURGLARCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDARMOURTONONPHYMIT" then
											if SN == "BURGLARCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "BURGLARCDARMOURTYPE" then
									if SN < "BURGLARCDBASEICPR" then
										if SN < "BURGLARCDBASEFATE" then
											if SN == "BURGLARCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityH",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASEFATE" then
											if SN == "BURGLARCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									elseif SN > "BURGLARCDBASEICPR" then
										if SN < "BURGLARCDBASEMORALE" then
											if SN == "BURGLARCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASEMORALE" then
											if SN == "BURGLARCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								else
									return 2;
								end
							else
								return 3;
							end
						elseif SN > "BURGLARCDBASENCPR" then
							if SN < "BURGLARCDPHYMITTONONPHYMIT" then
								if SN < "BURGLARCDFATETOPOWER" then
									if SN < "BURGLARCDCALCTYPECOMPHYMIT" then
										if SN < "BURGLARCDBASEVITALITY" then
											if SN == "BURGLARCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "BURGLARCDBASEVITALITY" then
											if SN == "BURGLARCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									elseif SN > "BURGLARCDCALCTYPECOMPHYMIT" then
										if SN < "BURGLARCDCALCTYPETACMIT" then
											if SN == "BURGLARCDCALCTYPENONPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "BURGLARCDCALCTYPETACMIT" then
											if SN == "BURGLARCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											return 26;
										end
									else
										return 13;
									end
								elseif SN > "BURGLARCDFATETOPOWER" then
									if SN < "BURGLARCDMIGHTTOFINESSE" then
										if SN < "BURGLARCDMIGHTTOCRITHIT" then
											if SN == "BURGLARCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDMIGHTTOCRITHIT" then
											if SN == "BURGLARCDMIGHTTOEVADE" then
												return 1;
											else
												return 0;
											end
										else
											return 1.5;
										end
									elseif SN > "BURGLARCDMIGHTTOFINESSE" then
										if SN < "BURGLARCDMIGHTTOPHYMAS" then
											if SN == "BURGLARCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "BURGLARCDMIGHTTOPHYMAS" then
											if SN == "BURGLARCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 1.5;
									end
								else
									return 1;
								end
							elseif SN > "BURGLARCDPHYMITTONONPHYMIT" then
								if SN < "BURGLARCDWILLTOPHYMAS" then
									if SN < "BURGLARCDVITALITYTONCMR" then
										if SN < "BURGLARCDVITALITYTOICMR" then
											if SN == "BURGLARCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDVITALITYTOICMR" then
											if SN == "BURGLARCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 0.012;
										end
									elseif SN > "BURGLARCDVITALITYTONCMR" then
										if SN < "BURGLARCDWILLTOFINESSE" then
											if SN == "BURGLARCDWILLTOCRITHIT" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "BURGLARCDWILLTOFINESSE" then
											if SN == "BURGLARCDWILLTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 0.12;
									end
								elseif SN > "BURGLARCDWILLTOPHYMAS" then
									if SN < "CAPTAINCDAGILITYTOCRITHIT" then
										if SN < "BURGLARCDWILLTORESIST" then
											if SN == "BURGLARCDWILLTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "BURGLARCDWILLTORESIST" then
											if SN == "C" then
												return C;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CAPTAINCDAGILITYTOCRITHIT" then
										if SN < "CAPTAINCDAGILITYTOPARRY" then
											if SN == "CAPTAINCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDAGILITYTOPARRY" then
											if SN == "CAPTAINCDAGILITYTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								else
									return 2;
								end
							else
								return 1;
							end
						else
							return CalcStat("ClassBaseNCPR",L);
						end
					elseif SN > "CAPTAINCDAGILITYTOTACMAS" then
						if SN < "CAPTAINCDPHYMITTONONPHYMIT" then
							if SN < "CAPTAINCDBASEWILL" then
								if SN < "CAPTAINCDBASEICMR" then
									if SN < "CAPTAINCDARMOURTOTACMIT" then
										if SN > "CAPTAINCDARMOURTOCOMPHYMIT" then
											if SN == "CAPTAINCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN == "CAPTAINCDARMOURTOCOMPHYMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "CAPTAINCDARMOURTOTACMIT" then
										if SN < "CAPTAINCDBASEAGILITY" then
											if SN == "CAPTAINCDARMOURTYPE" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEAGILITY" then
											if SN == "CAPTAINCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityL",L);
										end
									else
										return 0.2;
									end
								elseif SN > "CAPTAINCDBASEICMR" then
									if SN < "CAPTAINCDBASENCMR" then
										if SN < "CAPTAINCDBASEMIGHT" then
											if SN == "CAPTAINCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEMIGHT" then
											if SN == "CAPTAINCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMightH",L);
										end
									elseif SN > "CAPTAINCDBASENCMR" then
										if SN < "CAPTAINCDBASEPOWER" then
											if SN == "CAPTAINCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										elseif SN > "CAPTAINCDBASEPOWER" then
											if SN == "CAPTAINCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBasePower",L);
										end
									else
										return CalcStat("ClassBaseNCMRM",L);
									end
								else
									return CalcStat("ClassBaseICMRM",L);
								end
							elseif SN > "CAPTAINCDBASEWILL" then
								if SN < "CAPTAINCDMIGHTTOBLOCK" then
									if SN < "CAPTAINCDCANBLOCK" then
										if SN < "CAPTAINCDCALCTYPENONPHYMIT" then
											if SN == "CAPTAINCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDCALCTYPENONPHYMIT" then
											if SN == "CAPTAINCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									elseif SN > "CAPTAINCDCANBLOCK" then
										if SN < "CAPTAINCDFATETOPOWER" then
											if SN == "CAPTAINCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDFATETOPOWER" then
											if SN == "CAPTAINCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										if Lm <= 14 then
											return 0;
										else
											return 1;
										end
									end
								elseif SN > "CAPTAINCDMIGHTTOBLOCK" then
									if SN < "CAPTAINCDMIGHTTOPHYMIT" then
										if SN < "CAPTAINCDMIGHTTOPARRY" then
											if SN == "CAPTAINCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDMIGHTTOPARRY" then
											if SN == "CAPTAINCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CAPTAINCDMIGHTTOPHYMIT" then
										if SN < "CAPTAINCDMIGHTTOTACMIT" then
											if SN == "CAPTAINCDMIGHTTOTACMAS" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDMIGHTTOTACMIT" then
											if SN == "CAPTAINCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 2;
								end
							else
								return CalcStat("ClassBaseWillM",L);
							end
						elseif SN > "CAPTAINCDPHYMITTONONPHYMIT" then
							if SN < "CHAMPIONCDAGILITYTOPHYMAS" then
								if SN < "CAPTAINCDWILLTORESIST" then
									if SN < "CAPTAINCDVITALITYTONCMR" then
										if SN < "CAPTAINCDVITALITYTOICMR" then
											if SN == "CAPTAINCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDVITALITYTOICMR" then
											if SN == "CAPTAINCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 0.012;
										end
									elseif SN > "CAPTAINCDVITALITYTONCMR" then
										if SN < "CAPTAINCDWILLTOPHYMAS" then
											if SN == "CAPTAINCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDWILLTOPHYMAS" then
											if SN == "CAPTAINCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.12;
									end
								elseif SN > "CAPTAINCDWILLTORESIST" then
									if SN < "CHAMPIONCDAGILITYTOCRITHIT" then
										if SN < "CAPTAINCDWILLTOTACMIT" then
											if SN == "CAPTAINCDWILLTOTACMAS" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CAPTAINCDWILLTOTACMIT" then
											if SN == "CATMINTBEARARMOUR" then
												return CalcStat("LMCatmintBearArmour",L);
											else
												return 0;
											end
										else
											return 1.5;
										end
									elseif SN > "CHAMPIONCDAGILITYTOCRITHIT" then
										if SN < "CHAMPIONCDAGILITYTOOUTHEAL" then
											if SN == "CHAMPIONCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDAGILITYTOOUTHEAL" then
											if SN == "CHAMPIONCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return 2;
									end
								else
									return 1;
								end
							elseif SN > "CHAMPIONCDAGILITYTOPHYMAS" then
								if SN < "CHAMPIONCDBASEICPR" then
									if SN < "CHAMPIONCDARMOURTYPE" then
										if SN < "CHAMPIONCDARMOURTONONPHYMIT" then
											if SN == "CHAMPIONCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDARMOURTONONPHYMIT" then
											if SN == "CHAMPIONCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "CHAMPIONCDARMOURTYPE" then
										if SN < "CHAMPIONCDBASEFATE" then
											if SN == "CHAMPIONCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASEFATE" then
											if SN == "CHAMPIONCDBASEICMR" then
												return CalcStat("ClassBaseICMRH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									else
										return 3;
									end
								elseif SN > "CHAMPIONCDBASEICPR" then
									if SN < "CHAMPIONCDBASENCPR" then
										if SN < "CHAMPIONCDBASEMORALE" then
											if SN == "CHAMPIONCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASEMORALE" then
											if SN == "CHAMPIONCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									elseif SN > "CHAMPIONCDBASENCPR" then
										if SN < "CHAMPIONCDBASEVITALITY" then
											if SN == "CHAMPIONCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDBASEVITALITY" then
											if SN == "CHAMPIONCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseVitality",L);
										end
									else
										return CalcStat("ClassBaseNCPR",L);
									end
								else
									return CalcStat("ClassBaseICPR",L);
								end
							else
								return 2;
							end
						else
							return 1;
						end
					else
						return 2;
					end
				elseif SN > "CHAMPIONCDCALCTYPECOMPHYMIT" then
					if SN < "CLASSBASEWILLL" then
						if SN < "CHISELCRITHITHOLD" then
							if SN < "CHAMPIONCDTACMASTOOUTHEAL" then
								if SN < "CHAMPIONCDMIGHTTOCRITHIT" then
									if SN < "CHAMPIONCDCANBLOCK" then
										if SN > "CHAMPIONCDCALCTYPENONPHYMIT" then
											if SN == "CHAMPIONCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN == "CHAMPIONCDCALCTYPENONPHYMIT" then
											return 14;
										else
											return 0;
										end
									elseif SN > "CHAMPIONCDCANBLOCK" then
										if SN < "CHAMPIONCDFATETOPOWER" then
											if SN == "CHAMPIONCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDFATETOPOWER" then
											if SN == "CHAMPIONCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										if Lm <= 5 then
											return 0;
										else
											return 1;
										end
									end
								elseif SN > "CHAMPIONCDMIGHTTOCRITHIT" then
									if SN < "CHAMPIONCDMIGHTTOPHYMIT" then
										if SN < "CHAMPIONCDMIGHTTOPARRY" then
											if SN == "CHAMPIONCDMIGHTTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDMIGHTTOPARRY" then
											if SN == "CHAMPIONCDMIGHTTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "CHAMPIONCDMIGHTTOPHYMIT" then
										if SN < "CHAMPIONCDPHYMITTOCOMPHYMIT" then
											if SN == "CHAMPIONCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDPHYMITTOCOMPHYMIT" then
											if SN == "CHAMPIONCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "CHAMPIONCDTACMASTOOUTHEAL" then
								if SN < "CHAMPIONCDWILLTORESIST" then
									if SN < "CHAMPIONCDWILLTOFINESSE" then
										if SN < "CHAMPIONCDVITALITYTOMORALE" then
											if SN == "CHAMPIONCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDVITALITYTOMORALE" then
											if SN == "CHAMPIONCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 4.5;
										end
									elseif SN > "CHAMPIONCDWILLTOFINESSE" then
										if SN < "CHAMPIONCDWILLTOPHYMAS" then
											if SN == "CHAMPIONCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CHAMPIONCDWILLTOPHYMAS" then
											if SN == "CHAMPIONCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "CHAMPIONCDWILLTORESIST" then
									if SN < "CHICKENCDCALCTYPENONPHYMIT" then
										if SN < "CHICKENCANBLOCK" then
											if SN == "CHAMPIONCDWILLTOTACMIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "CHICKENCANBLOCK" then
											if SN == "CHICKENCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "CHICKENCDCALCTYPENONPHYMIT" then
										if SN < "CHICKENCDHASPOWER" then
											if SN == "CHICKENCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN > "CHICKENCDHASPOWER" then
											if SN == "CHISELCRITHITH" then
												if Lm <= 140 then
													return CalcStat("U371LegacyStatFix",L,"ChiselCritHitHOld");
												else
													return CalcStat("ChiselCritHitH",140);
												end
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 14;
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "CHISELCRITHITHOLD" then
							if SN < "CLASSBASEICMRM" then
								if SN < "CHPSTALWBLADEVITALITY" then
									if SN < "CHPCONTRBURNICPR" then
										if SN < "CHISELCRITHITLOLD" then
											if SN == "CHISELCRITHITL" then
												if Lm <= 140 then
													return CalcStat("U371LegacyStatFix",L,"ChiselCritHitLOld");
												else
													return CalcStat("ChiselCritHitL",140);
												end
											else
												return 0;
											end
										elseif SN > "CHISELCRITHITLOLD" then
											if SN == "CHP2HWPNBLOCK" then
												return CalcStat("BlockT",L,4);
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return RoundDbl(16.16*L);
											else
												return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitLOld",105));
											end
										end
									elseif SN > "CHPCONTRBURNICPR" then
										if SN < "CHPFLURRYINCRCRITHIT" then
											if SN == "CHPFINESSEINCREASE" then
												return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "CHPFLURRYINCRCRITHIT" then
											if SN == "CHPMIGHTINCREASE" then
												return CalcStat("MightT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("CritHitT",L,2.4);
										end
									else
										return CalcStat("ICPRT",L,0.4);
									end
								elseif SN > "CHPSTALWBLADEVITALITY" then
									if SN < "CLASSBASEAGILITYM" then
										if SN < "CLASSBASEAGILITYH" then
											if SN == "CHPUNBREAKTACMIT" then
												return CalcStat("TacMitT",L,0.4);
											else
												return 0;
											end
										elseif SN > "CLASSBASEAGILITYH" then
											if SN == "CLASSBASEAGILITYL" then
												return CalcStat("BaseAgility",L,0.5);
											else
												return 0;
											end
										else
											return CalcStat("BaseAgility",L,1.5);
										end
									elseif SN > "CLASSBASEAGILITYM" then
										if SN < "CLASSBASEICMRH" then
											if SN == "CLASSBASEFATE" then
												return CalcStat("BaseFate",L);
											else
												return 0;
											end
										elseif SN > "CLASSBASEICMRH" then
											if SN == "CLASSBASEICMRL" then
												return 0.15;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return CalcStat("BaseAgility",L,1.0);
									end
								else
									return CalcStat("VitalityT",L,CalcStat("Trait567810Choice",N)*0.4);
								end
							elseif SN > "CLASSBASEICMRM" then
								if SN < "CLASSBASENCMRL" then
									if SN < "CLASSBASEMIGHTL" then
										if SN < "CLASSBASEICPRADJ" then
											if SN == "CLASSBASEICPR" then
												return StatLinInter("PntMPClassBaseICPR","ClassBaseProgPowerRegen","ProgBICPR","ClassBaseICPRAdj",L,N,99);
											else
												return 0;
											end
										elseif SN > "CLASSBASEICPRADJ" then
											if SN == "CLASSBASEMIGHTH" then
												return CalcStat("BaseMight",L,1.5);
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 1.5*N;
											elseif Lm <= 20 then
												return 1.1*N;
											else
												return N;
											end
										end
									elseif SN > "CLASSBASEMIGHTL" then
										if SN < "CLASSBASEMORALE" then
											if SN == "CLASSBASEMIGHTM" then
												return CalcStat("BaseMight",L,1.0);
											else
												return 0;
											end
										elseif SN > "CLASSBASEMORALE" then
											if SN == "CLASSBASENCMRH" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("BaseMorale",L);
										end
									else
										return CalcStat("BaseMight",L,0.5);
									end
								elseif SN > "CLASSBASENCMRL" then
									if SN < "CLASSBASEPOWER" then
										if SN < "CLASSBASENCPR" then
											if SN == "CLASSBASENCMRM" then
												return 1;
											else
												return 0;
											end
										elseif SN > "CLASSBASENCPR" then
											if SN == "CLASSBASENCPRADJ" then
												if Lm <= 1 then
													return 1.5*N;
												elseif Lm <= 20 then
													return 1.1*N;
												else
													return N;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPClassBaseNCPR","ClassBaseProgPowerRegen","ProgBNCPR","ClassBaseNCPRAdj",L,N,99);
										end
									elseif SN > "CLASSBASEPOWER" then
										if SN < "CLASSBASEVITALITY" then
											if SN == "CLASSBASEPROGPOWERREGEN" then
												return {{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150},{1,20,50,60,65,75,85,95,100,105,115,120,130,140,150}};
											else
												return 0;
											end
										elseif SN > "CLASSBASEVITALITY" then
											if SN == "CLASSBASEWILLH" then
												return CalcStat("BaseWill",L,1.5);
											else
												return 0;
											end
										else
											return CalcStat("BaseVitality",L);
										end
									else
										return CalcStat("BasePower",L);
									end
								else
									return 1;
								end
							else
								return 0.175;
							end
						else
							if Lm <= 105 then
								return RoundDbl(24.24*L);
							else
								return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("ChiselCritHitHOld",105));
							end
						end
					elseif SN > "CLASSBASEWILLL" then
						if SN < "COMBATDAMAGEMODNPCS" then
							if SN < "COMBATBASETACHPSBYLEVEL" then
								if SN < "COMBATBASEL" then
									if SN < "COMBATBASE" then
										if SN > "CLASSBASEWILLM" then
											if SN == "CLOTHARMOUR" then
												if Lm <= 50 then
													return L;
												else
													return 50;
												end
											else
												return 0;
											end
										elseif SN == "CLASSBASEWILLM" then
											return CalcStat("BaseWill",L,1.0);
										else
											return 0;
										end
									elseif SN > "COMBATBASE" then
										if SN < "COMBATBASEH" then
											if SN == "COMBATBASECAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseH",N);
												else
													return CalcStat("CombatBaseL",N);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASEH" then
											if SN == "COMBATBASEHQTYMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.02,1.04,1.08,1.12},L);
												end
											else
												return 0;
											end
										else
											return 1.0*CalcStat("CombatBasePhyDPS",L)*1.08;
										end
									else
										return CalcStat("CombatBaseCat",WpnCodeIndex(C,1),L)*CalcStat("CombatBaseTypeMP",WpnCodeIndex(C,2))*CalcStat("CombatBaseQtyMPCat",WpnCodeIndex(C,1),WpnCodeIndex(C,3));
									end
								elseif SN > "COMBATBASEL" then
									if SN < "COMBATBASETACDPS" then
										if SN < "COMBATBASEPHYDPS" then
											if SN == "COMBATBASELQTYMP" then
												return CalcStat("CombatBaseHQtyMP",L);
											else
												return 0;
											end
										elseif SN > "COMBATBASEPHYDPS" then
											if SN == "COMBATBASEQTYMPCAT" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return CalcStat("CombatBaseHQtyMP",N);
												else
													return CalcStat("CombatBaseLQtyMP",N);
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,2,20,1,50,L);
											elseif Lm <= 60 then
												return LinFmod(1,20,26.6,50,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,26.6,29.1,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,29.1,39,65,75,L);
											elseif Lm <= 125 then
												return LinFmod(1,39,52,75,125,L);
											elseif Lm <= 175 then
												return LinFmod(1,52,69,125,175,L);
											elseif Lm <= 200 then
												return LinFmod(1,69,86,175,200,L);
											elseif Lm <= 222 then
												return LinFmod(1,86,108,200,222,L);
											elseif Lm <= 300 then
												return LinFmod(1,108,108,222,300,L);
											elseif Lm <= 350 then
												return LinFmod(1,119,144,301,350,L);
											elseif Lm <= 400 then
												return LinFmod(1,158,180,351,400,L);
											elseif Lm <= 450 then
												return LinFmod(1,198,239,401,450,L);
											elseif Lm <= 500 then
												return LinFmod(1,263,360,451,500,L);
											elseif Lm <= 550 then
												return LinFmod(1,400,540,501,550,L);
											elseif Lm <= 600 then
												return LinFmod(1,590,810,551,600,L);
											elseif Lm <= 650 then
												return RoundDblDown(RoundDblUp((L-50)/5.5)*36.7-2811,-1);
											else
												return CalcStat("CombatBasePhyDPS",650);
											end
										end
									elseif SN > "COMBATBASETACDPS" then
										if SN < "COMBATBASETACDPSBYLEVELMAX" then
											if SN == "COMBATBASETACDPSBYLEVEL" then
												if Lm <= 50 then
													return LinFmod(1,2,CalcStat("CombatBaseTacDPSByLevelMax",L),1,50,L);
												else
													return CalcStat("CombatBaseTacDPSByLevelMax",L);
												end
											else
												return 0;
											end
										elseif SN > "COMBATBASETACDPSBYLEVELMAX" then
											if SN == "COMBATBASETACHPS" then
												if Lm <= 46 then
													return 0;
												elseif Lm <= 49 then
													return CalcStat("CombatBaseTacHPSCurves",L)-CalcStat("CombatBaseTacHPSByLevelMax",L);
												elseif Lm <= 55 then
													return 0.5;
												elseif Lm <= 474 then
													return CalcStat("CommonTacHPS",L)-CalcStat("CombatBaseTacHPSByLevelMax",L);
												elseif Lm <= 475 then
													return 460.62656;
												elseif Lm <= 599 then
													return CalcStat("CommonTacHPS",L)-CalcStat("CombatBaseTacHPSByLevelMax",L);
												elseif Lm <= 649 then
													return (RoundDblUp((4*L+2)/22)*678-RoundDblUp((L+1)/22))*0.175-11734.1-CalcStat("CombatBaseTacHPSByLevelMax",L);
												else
													return CalcStat("CombatBaseTacHPS",649);
												end
											else
												return 0;
											end
										else
											return 20;
										end
									else
										if Lm <= 47 then
											return LinFmod(1,0.5,0.5,1,47,L);
										elseif Lm <= 50 then
											return LinFmod(1,0.5,0.8,47,50,L);
										elseif Lm <= 51 then
											return LinFmod(1,1,1,51,51,L);
										elseif Lm <= 60 then
											return LinFmod(1,1.32,6.6,52,60,L);
										elseif Lm <= 65 then
											return LinFmod(1,6.6,9.1,60,65,L);
										elseif Lm <= 75 then
											return LinFmod(1,9.1,19,65,75,L);
										elseif Lm <= 125 then
											return LinFmod(1,19,32,75,125,L);
										elseif Lm <= 175 then
											return LinFmod(1,32,49,125,175,L);
										elseif Lm <= 200 then
											return LinFmod(1,49,66,175,200,L);
										elseif Lm <= 222 then
											return LinFmod(1,66,88,200,222,L);
										elseif Lm <= 299 then
											return LinFmod(1,88,88,222,299,L);
										elseif Lm <= 349 then
											return LinFmod(1,99,124,300,349,L);
										elseif Lm <= 399 then
											return LinFmod(1,138,160,350,399,L);
										elseif Lm <= 449 then
											return LinFmod(1,178,219,400,449,L);
										elseif Lm <= 499 then
											return LinFmod(1,243,340,450,499,L);
										elseif Lm <= 549 then
											return LinFmod(1,380,520,500,549,L);
										elseif Lm <= 599 then
											return LinFmod(1,570,790,550,599,L);
										elseif Lm <= 649 then
											return RoundDblDown(RoundDblUp((L-49)/5.5)*36.7-2831,-1);
										else
											return CalcStat("CombatBaseTacDPS",649);
										end
									end
								else
									return 0.9*CalcStat("CombatBasePhyDPS",L)*1.08;
								end
							elseif SN > "COMBATBASETACHPSBYLEVEL" then
								if SN < "COMBATDAMAGEMODHEALTH" then
									if SN < "COMBATBASETACHPSNOCLASS" then
										if SN < "COMBATBASETACHPSCURVES" then
											if SN == "COMBATBASETACHPSBYLEVELMAX" then
												return 5.659142856;
											else
												return 0;
											end
										elseif SN > "COMBATBASETACHPSCURVES" then
											if SN == "COMBATBASETACHPSLVLTOILVL" then
												if Lm <= 75 then
													return LinFmod(1,1,75,1,75,L);
												elseif Lm <= 100 then
													return LinFmod(1,75,200,75,100,L);
												elseif Lm <= 104 then
													return RoundDbl(LinFmod(1,201.3,214.3,101,104,L));
												elseif Lm <= 105 then
													return 222;
												else
													return RoundDbl(CalcStat("LvlToILvl",L));
												end
											else
												return 0;
											end
										else
											if Lm <= 25 then
												return -0.0033215*L*L+0.2470730225*L+1.7562484775;
											else
												return -0.003714298*L*L+0.254286886*L+1.82140064;
											end
										end
									elseif SN > "COMBATBASETACHPSNOCLASS" then
										if SN < "COMBATDAMAGEMOD" then
											if SN == "COMBATBASETYPEMP" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.4,1.4},L);
												end
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMOD" then
											if SN == "COMBATDAMAGEMODENERGY" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBEnergy","",L,N,2);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","",L,N,2);
										end
									else
										return CalcStat("CombatBaseTacHPSByLevel",L)+CalcStat("CombatBaseTacHPS",CalcStat("CombatBaseTacHPSLvlToILvl",L));
									end
								elseif SN > "COMBATDAMAGEMODHEALTH" then
									if SN < "COMBATDAMAGEMODHEALTHMEDIUM" then
										if SN < "COMBATDAMAGEMODHEALTHLOW" then
											if SN == "COMBATDAMAGEMODHEALTHADJ" then
												if Lm <= 141 then
													return 1;
												else
													return 0.63;
												end
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODHEALTHLOW" then
											if SN == "COMBATDAMAGEMODHEALTHLOWADJ" then
												if Lm <= 1 then
													return 0.144*N;
												elseif Lm <= 25 then
													return 0.324*N;
												elseif Lm <= 50 then
													return 0.576*N;
												else
													return 0.9*N;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","CombatDamageModHealthLowAdj",L,N,99);
										end
									elseif SN > "COMBATDAMAGEMODHEALTHMEDIUM" then
										if SN < "COMBATDAMAGEMODNERFED" then
											if SN == "COMBATDAMAGEMODHEALTHMEDIUMADJ" then
												if Lm <= 1 then
													return 0.36*N;
												elseif Lm <= 25 then
													return 0.54*N;
												elseif Lm <= 50 then
													return 0.72*N;
												else
													return 0.9*N;
												end
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODNERFED" then
											if SN == "COMBATDAMAGEMODNERFEDADJ" then
												if Lm <= 141 then
													return 1;
												else
													return 0.82;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModNerfedAdj",L,N,2);
										end
									else
										return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","CombatDamageModHealthMediumAdj",L,N,99);
									end
								else
									return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBHealth","CombatDamageModHealthAdj",L,N,2);
								end
							else
								if Lm <= 46 then
									return CalcStat("CombatBaseTacHPSCurves",L);
								else
									return CalcStat("CombatBaseTacHPSByLevelMax",L);
								end
							end
						elseif SN > "COMBATDAMAGEMODNPCS" then
							if SN < "CPTCOVVITALITY" then
								if SN < "COMMONTACHPSITEMPROG" then
									if SN < "COMBATDAMAGEMODPLAYERS" then
										if SN < "COMBATDAMAGEMODPETSADJ" then
											if SN == "COMBATDAMAGEMODPETS" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,99);
											else
												return 0;
											end
										elseif SN > "COMBATDAMAGEMODPETSADJ" then
											if SN == "COMBATDAMAGEMODPETSRND" then
												return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPetsAdj",L,N,2);
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return N;
											elseif Lm <= 25 then
												return 0.4*N;
											elseif Lm <= 50 then
												return 0.5*N;
											elseif Lm <= 60 then
												return 0.6*N;
											elseif Lm <= 65 then
												return 0.7*N;
											elseif Lm <= 75 then
												return 0.8*N;
											elseif Lm <= 85 then
												return 0.85*N;
											elseif Lm <= 95 then
												return 0.9*N;
											elseif Lm <= 100 then
												return 0.95*N;
											else
												return N;
											end
										end
									elseif SN > "COMBATDAMAGEMODPLAYERS" then
										if SN < "COMBATINHEAL" then
											if SN == "COMBATDAMAGEMODPLAYERSADJ" then
												if Lm <= 1 then
													return 0.8*N;
												elseif Lm <= 25 then
													return 0.9*N;
												else
													return N;
												end
											else
												return 0;
											end
										elseif SN > "COMBATINHEAL" then
											if SN == "COMMONTACHPS" then
												return StatLinInter("PntMPTacHPS","CommonTacHPSItemProg","ProgBTacHPS","",L,N,99);
											else
												return 0;
											end
										else
											return CalcStat("InHeal",L,2.0);
										end
									else
										return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","CombatDamageModPlayersAdj",L,N,99);
									end
								elseif SN > "COMMONTACHPSITEMPROG" then
									if SN < "CPTBLADETACMAS" then
										if SN < "CPTBLADECRITDEF" then
											if SN == "CONSTSTATC" then
												return CalcStat(C,1,L);
											else
												return 0;
											end
										elseif SN > "CPTBLADECRITDEF" then
											if SN == "CPTBLADEPHYMAS" then
												return CalcStat("PhyMasT",L,1.2);
											else
												return 0;
											end
										else
											return CalcStat("CritDefT",L,0.8);
										end
									elseif SN > "CPTBLADETACMAS" then
										if SN < "CPTCOVMAIN" then
											if SN == "CPTCOVFATE" then
												return CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										elseif SN > "CPTCOVMAIN" then
											if SN == "CPTCOVPHYMIT" then
												return CalcStat("PhyMitT",L,2.4);
											else
												return 0;
											end
										else
											return CalcStat("MainT",L,0.4);
										end
									else
										return CalcStat("TacMasT",L,0.8);
									end
								else
									return {{1,50,60,65,75,125,175,200,222,299,300,349,350,399,400,449,450,499,500,549,550,599},{1,50,60,65,75,85,95,100,105,105,106,115,116,120,121,130,131,140,141,150,151,160}};
								end
							elseif SN > "CPTCOVVITALITY" then
								if SN < "CPTSONGCRITDEF" then
									if SN < "CPTIDOMEVITALITY" then
										if SN < "CPTIDOMEFATE" then
											if SN == "CPTCRITDEF" then
												return CalcStat("CritDef",L,0.6);
											else
												return 0;
											end
										elseif SN > "CPTIDOMEFATE" then
											if SN == "CPTIDOMEMAIN" then
												return CalcStat("MainT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("FateT",L,0.4);
										end
									elseif SN > "CPTIDOMEVITALITY" then
										if SN < "CPTSHIELDPHYMAS" then
											if SN == "CPTSHIELDCRITDEF" then
												return CalcStat("CritDefT",L,1.2);
											else
												return 0;
											end
										elseif SN > "CPTSHIELDPHYMAS" then
											if SN == "CPTSHIELDTACMAS" then
												return CalcStat("TacMasT",L,0.8);
											else
												return 0;
											end
										else
											return CalcStat("PhyMasT",L,0.8);
										end
									else
										return CalcStat("VitalityT",L,0.4);
									end
								elseif SN > "CPTSONGCRITDEF" then
									if SN < "CPTSTANDALONETACMAS" then
										if SN < "CPTSONGTACMAS" then
											if SN == "CPTSONGPHYMAS" then
												return CalcStat("PhyMasT",L,0.8);
											else
												return 0;
											end
										elseif SN > "CPTSONGTACMAS" then
											if SN == "CPTSTANDALONEPHYMAS" then
												return CalcStat("PhyMasT",L,1.2);
											else
												return 0;
											end
										else
											return CalcStat("TacMasT",L,1.2);
										end
									elseif SN > "CPTSTANDALONETACMAS" then
										if SN < "CREEPAUDACITYCOST" then
											if SN == "CREEPAUDACITYCCDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return 0.5;
												else
													return 0.4;
												end
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYCOST" then
											if SN == "CREEPAUDACITYCOSTBASE" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 2 then
													return 2*L;
												elseif Lm <= 9 then
													return 2*L-1;
												elseif Lm <= 15 then
													return 3*L;
												elseif Lm <= 25 then
													return 6*L;
												elseif Lm <= 29 then
													return 9*L;
												elseif Lm <= 36 then
													return 300;
												else
													return 0;
												end
											else
												return 0;
											end
										else
											return CalcStat("CreepAudacityCostBase",L)*25;
										end
									else
										return CalcStat("TacMasT",L,0.8);
									end
								else
									return CalcStat("CritDefT",L,0.8);
								end
							else
								return CalcStat("VitalityT",L,0.4);
							end
						else
							return StatLinInter("PntMPCombatDamageMod","TraitProgVital","ProgBCombatDamageMod","",L,N,99);
						end
					else
						return CalcStat("BaseWill",L,0.5);
					end
				else
					return 14;
				end
			else
				return -CalcStat("PhyMitT",L,3);
			end
		elseif SN > "CREEPAUDACITYDMGP" then
			if SN < "HUNTERCDAGILITYTOEVADE" then
				if SN < "FINESSEPRATPB" then
					if SN < "DEVHITPRATPCAP" then
						if SN < "CRITHIT" then
							if SN < "CREEPBATPROMRNGDMGP" then
								if SN < "CREEPAUDACITYTACREDP" then
									if SN < "CREEPAUDACITYREDP" then
										if SN > "CREEPAUDACITYMELDMGP" then
											if SN == "CREEPAUDACITYMELREDP" then
												return CalcStat("CreepAudacityRedP",L);
											else
												return 0;
											end
										elseif SN == "CREEPAUDACITYMELDMGP" then
											return CalcStat("CreepAudacityDmgP",L);
										else
											return 0;
										end
									elseif SN > "CREEPAUDACITYREDP" then
										if SN < "CREEPAUDACITYRNGREDP" then
											if SN == "CREEPAUDACITYRNGDMGP" then
												return CalcStat("CreepAudacityDmgP",L);
											else
												return 0;
											end
										elseif SN > "CREEPAUDACITYRNGREDP" then
											if SN == "CREEPAUDACITYTACDMGP" then
												return CalcStat("CreepAudacityDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("CreepAudacityRedP",L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 10 then
											return LinFmod(1,0.55,0.55,1,10,L);
										elseif Lm <= 36 then
											return LinFmod(1,0.55,0.55,10,36,L);
										elseif Lm <= 41 then
											return LinFmod(1,0.55,0.55,36,41,L);
										elseif Lm <= 60 then
											return LinFmod(1,0.55,0.55,41,60,L);
										else
											return CalcStat("CreepAudacityRedP",60);
										end
									end
								elseif SN > "CREEPAUDACITYTACREDP" then
									if SN < "CREEPBATPROMINHEALP" then
										if SN < "CREEPBATPROMFINESSE" then
											if SN == "CREEPBATPROMDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 5 then
													return LinFmod(1,0.005,0.02,1,5,L);
												elseif Lm <= 10 then
													return LinFmod(1,0.02,0.045,5,10,L);
												elseif Lm <= 15 then
													return LinFmod(1,0.045,0.075,10,15,L);
												else
													return CalcStat("CreepBatPromDmgP",15);
												end
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMFINESSE" then
											if SN == "CREEPBATPROMHEALTHP" then
												return CalcStat("CreepBatPromVitalP",L);
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "CREEPBATPROMINHEALP" then
										if SN < "CREEPBATPROMOUTHEALP" then
											if SN == "CREEPBATPROMMELDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMOUTHEALP" then
											if SN == "CREEPBATPROMPOWERP" then
												return CalcStat("CreepBatPromVitalP",L);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 5 then
												return LinFmod(1,0.02,0.1,1,5,L);
											elseif Lm <= 10 then
												return LinFmod(1,0.1,0.15,5,10,L);
											elseif Lm <= 15 then
												return LinFmod(1,0.15,0.2,10,15,L);
											else
												return CalcStat("CreepBatPromOutHealP",15);
											end
										end
									else
										return 0;
									end
								else
									return CalcStat("CreepAudacityRedP",L);
								end
							elseif SN > "CREEPBATPROMRNGDMGP" then
								if SN < "CRITDEFPPRAT" then
									if SN < "CREEPPROGB" then
										if SN < "CREEPBATPROMVITALP" then
											if SN == "CREEPBATPROMTACDMGP" then
												return CalcStat("CreepBatPromDmgP",L);
											else
												return 0;
											end
										elseif SN > "CREEPBATPROMVITALP" then
											if SN == "CREEPPROG" then
												return {{1,150},{1,150}};
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 5 then
												return LinFmod(1,1.02,1.1,1,5,L);
											elseif Lm <= 10 then
												return LinFmod(1,1.1,1.15,5,10,L);
											elseif Lm <= 15 then
												return LinFmod(1,1.15,1.2,10,15,L);
											else
												return CalcStat("CreepBatPromVitalP",15);
											end
										end
									elseif SN > "CREEPPROGB" then
										if SN < "CRITDEFC" then
											if SN == "CRITDEF" then
												return StatLinInter("PntMPCritDef","ItemProg","ProgBCritDef","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										elseif SN > "CRITDEFC" then
											if SN == "CRITDEFPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPCritDefC","CreepProg","ProgBCritDefC","",L,N,0);
										end
									else
										return LinFmod(1,0.01,1,1,CalcStat("LevelCap",L),L);
									end
								elseif SN > "CRITDEFPPRAT" then
									if SN < "CRITDEFPRATPC" then
										if SN < "CRITDEFPRATPA" then
											if SN == "CRITDEFPRATP" then
												return CalcPercAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN > "CRITDEFPRATPA" then
											if SN == "CRITDEFPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										else
											return 240;
										end
									elseif SN > "CRITDEFPRATPC" then
										if SN < "CRITDEFPRATPCAPR" then
											if SN == "CRITDEFPRATPCAP" then
												return 80;
											else
												return 0;
											end
										elseif SN > "CRITDEFPRATPCAPR" then
											if SN == "CRITDEFT" then
												return StatLinInter("PntMPCritDef","TraitProg","ProgBCritDef","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("CritDefPRatPB",L)*CalcStat("CritDefPRatPC",L);
										end
									else
										return 0.5;
									end
								else
									return CalcRatAB(CalcStat("CritDefPRatPA",L),CalcStat("CritDefPRatPB",L),CalcStat("CritDefPRatPCapR",L),N);
								end
							else
								return CalcStat("CreepBatPromDmgP",L);
							end
						elseif SN > "CRITHIT" then
							if SN < "CRITMAGNPRATPB" then
								if SN < "CRITHITPRATPC" then
									if SN < "CRITHITPPRAT" then
										if SN < "CRITHITOLD" then
											if SN == "CRITHITC" then
												return StatLinInter("PntMPCritHitC","CreepProg","ProgBCritHitC","",L,N,0);
											else
												return 0;
											end
										elseif SN > "CRITHITOLD" then
											if SN == "CRITHITPBONUS" then
												return 0;
											else
												return 0;
											end
										else
											return CalcStat("CritHit",L,N);
										end
									elseif SN > "CRITHITPPRAT" then
										if SN < "CRITHITPRATPA" then
											if SN == "CRITHITPRATP" then
												return CalcPercAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCap",L),N);
											else
												return 0;
											end
										elseif SN > "CRITHITPRATPA" then
											if SN == "CRITHITPRATPB" then
												return CalcStat("BRatRounded",L,"BRatExtra");
											else
												return 0;
											end
										else
											return 75;
										end
									else
										return CalcRatAB(CalcStat("CritHitPRatPA",L),CalcStat("CritHitPRatPB",L),CalcStat("CritHitPRatPCapR",L),N);
									end
								elseif SN > "CRITHITPRATPC" then
									if SN < "CRITMAGNPBONUS" then
										if SN < "CRITHITPRATPCAPR" then
											if SN == "CRITHITPRATPCAP" then
												return 25;
											else
												return 0;
											end
										elseif SN > "CRITHITPRATPCAPR" then
											if SN == "CRITHITT" then
												return StatLinInter("PntMPCritHit","TraitProg","ProgBCritHit","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("CritHitPRatPB",L)*CalcStat("CritHitPRatPC",L);
										end
									elseif SN > "CRITMAGNPBONUS" then
										if SN < "CRITMAGNPRATP" then
											if SN == "CRITMAGNPPRAT" then
												return CalcRatAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "CRITMAGNPRATP" then
											if SN == "CRITMAGNPRATPA" then
												return 225;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("CritMagnPRatPA",L),CalcStat("CritMagnPRatPB",L),CalcStat("CritMagnPRatPCap",L),N);
										end
									else
										return 0;
									end
								else
									return 0.5;
								end
							elseif SN > "CRITMAGNPRATPB" then
								if SN < "DEFILERCDCALCTYPETACMIT" then
									if SN < "CRYRESISTT" then
										if SN < "CRITMAGNPRATPCAP" then
											if SN == "CRITMAGNPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "CRITMAGNPRATPCAP" then
											if SN == "CRITMAGNPRATPCAPR" then
												return CalcStat("CritMagnPRatPB",L)*CalcStat("CritMagnPRatPC",L);
											else
												return 0;
											end
										else
											return 75;
										end
									elseif SN > "CRYRESISTT" then
										if SN < "DEFILERCDCALCTYPECOMPHYMIT" then
											if SN == "DEFILERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										elseif SN > "DEFILERCDCALCTYPECOMPHYMIT" then
											if SN == "DEFILERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ResistAddT",L,N);
									end
								elseif SN > "DEFILERCDCALCTYPETACMIT" then
									if SN < "DEVHITPRATP" then
										if SN < "DEVHITPBONUS" then
											if SN == "DEFILERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "DEVHITPBONUS" then
											if SN == "DEVHITPPRAT" then
												return CalcRatAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									elseif SN > "DEVHITPRATP" then
										if SN < "DEVHITPRATPB" then
											if SN == "DEVHITPRATPA" then
												return 30;
											else
												return 0;
											end
										elseif SN > "DEVHITPRATPB" then
											if SN == "DEVHITPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatDevHit");
										end
									else
										return CalcPercAB(CalcStat("DevHitPRatPA",L),CalcStat("DevHitPRatPB",L),CalcStat("DevHitPRatPCap",L),N);
									end
								else
									return 27;
								end
							else
								return CalcStat("BRatRounded",L,"BRatCritMagn");
							end
						else
							return StatLinInter("PntMPCritHit","ItemProg","ProgBCritHit","AdjUmbarItem",L,N,0);
						end
					elseif SN > "DEVHITPRATPCAP" then
						if SN < "ELFFADINGFIRSTBORNFATE" then
							if SN < "DWARFRDTRAITICPR" then
								if SN < "DWARFLOSTDWARFKDSFATE" then
									if SN < "DMGTYPEMIT" then
										if SN > "DEVHITPRATPCAPR" then
											if SN == "DISEASERESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										elseif SN == "DEVHITPRATPCAPR" then
											return CalcStat("DevHitPRatPB",L)*CalcStat("DevHitPRatPC",L);
										else
											return 0;
										end
									elseif SN > "DMGTYPEMIT" then
										if SN < "DWARFENDURVITALITY" then
											if SN == "DMGTYPEMITT" then
												return StatLinInter("PntMPDmgTypeMit","TraitProg","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										elseif SN > "DWARFENDURVITALITY" then
											if SN == "DWARFFATEFULDWARFFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									else
										return StatLinInter("PntMPDmgTypeMit","ItemProg","ProgBMitigation","AdjUmbarItemMit",L,N,0);
									end
								elseif SN > "DWARFLOSTDWARFKDSFATE" then
									if SN < "DWARFRDPSVTWONAME" then
										if SN < "DWARFRDPSVONENAME" then
											if SN == "DWARFRDPSVONEFATE" then
												return CalcStat("DwarfFatefulDwarfFate",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDPSVONENAME" then
											if SN == "DWARFRDPSVTWOBLOCK" then
												return CalcStat("DwarfShieldBrwlBlock",L);
											else
												return 0;
											end
										else
											return "Fateful Dwarf";
										end
									elseif SN > "DWARFRDPSVTWONAME" then
										if SN < "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITAGILITY" then
												return CalcStat("DwarfStockyAgility",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITFATE" then
											if SN == "DWARFRDTRAITICMR" then
												return CalcStat("DwarfUnwearBattleICMR",L);
											else
												return 0;
											end
										else
											return CalcStat("DwarfLostDwarfKdsFate",L);
										end
									else
										return "Shield Brawler";
									end
								else
									return -CalcStat("FateT",L,0.4);
								end
							elseif SN > "DWARFRDTRAITICPR" then
								if SN < "DWARFSTURDINESSMIGHT" then
									if SN < "DWARFRDTRAITPHYMITP" then
										if SN < "DWARFRDTRAITNCMR" then
											if SN == "DWARFRDTRAITMIGHT" then
												return CalcStat("DwarfSturdinessMight",L);
											else
												return 0;
											end
										elseif SN > "DWARFRDTRAITNCMR" then
											if SN == "DWARFRDTRAITNCPR" then
												return CalcStat("DwarfUnwearBattleNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("DwarfUnwearBattleNCMR",L);
										end
									elseif SN > "DWARFRDTRAITPHYMITP" then
										if SN < "DWARFSHIELDBRWLBLOCK" then
											if SN == "DWARFRDTRAITVITALITY" then
												return CalcStat("DwarfSturdinessVitality",L);
											else
												return 0;
											end
										elseif SN > "DWARFSHIELDBRWLBLOCK" then
											if SN == "DWARFSTOCKYAGILITY" then
												return CalcStat("AgilityT",L,0.4);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,0.8);
										end
									else
										return CalcStat("DwarfSturdinessPhyMitP",L);
									end
								elseif SN > "DWARFSTURDINESSMIGHT" then
									if SN < "DWARFUNWEARBATTLEICPR" then
										if SN < "DWARFSTURDINESSVITALITY" then
											if SN == "DWARFSTURDINESSPHYMITP" then
												return 1;
											else
												return 0;
											end
										elseif SN > "DWARFSTURDINESSVITALITY" then
											if SN == "DWARFUNWEARBATTLEICMR" then
												return CalcStat("ICMRT",L,0.6);
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,1.0);
										end
									elseif SN > "DWARFUNWEARBATTLEICPR" then
										if SN < "DWARFUNWEARBATTLENCPR" then
											if SN == "DWARFUNWEARBATTLENCMR" then
												return -CalcStat("NCMRT",L,0.4);
											else
												return 0;
											end
										elseif SN > "DWARFUNWEARBATTLENCPR" then
											if SN == "ELFAGILITYWOODSAGILITY" then
												return CalcStat("AgilityT",L,1.0);
											else
												return 0;
											end
										else
											return -CalcStat("NCPRT",L,0.4);
										end
									else
										return CalcStat("ICPRT",L,0.6);
									end
								else
									return CalcStat("MightT",L,1.0);
								end
							else
								return CalcStat("DwarfUnwearBattleICPR",L);
							end
						elseif SN > "ELFFADINGFIRSTBORNFATE" then
							if SN < "EVADEPRATPA" then
								if SN < "ELFRDTRAITNCMR" then
									if SN < "ELFRDPSVTWONAME" then
										if SN < "ELFRDPSVONEFATE" then
											if SN == "ELFFRIENDOFMANFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										elseif SN > "ELFRDPSVONEFATE" then
											if SN == "ELFRDPSVONENAME" then
												return "Friend Of Man";
											else
												return 0;
											end
										else
											return CalcStat("ElfFriendOfManFate",L);
										end
									elseif SN > "ELFRDPSVTWONAME" then
										if SN < "ELFRDTRAITFATE" then
											if SN == "ELFRDTRAITAGILITY" then
												return CalcStat("ElfAgilityWoodsAgility",L);
											else
												return 0;
											end
										elseif SN > "ELFRDTRAITFATE" then
											if SN == "ELFRDTRAITMORALE" then
												return CalcStat("ElfSorrowFirstbornMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ElfFadingFirstbornFate",L);
										end
									else
										return "";
									end
								elseif SN > "ELFRDTRAITNCMR" then
									if SN < "EVADEC" then
										if SN < "ELFSORROWFIRSTBORNNCMR" then
											if SN == "ELFSORROWFIRSTBORNMORALE" then
												return -CalcStat("MoraleT",L,0.4);
											else
												return 0;
											end
										elseif SN > "ELFSORROWFIRSTBORNNCMR" then
											if SN == "EVADE" then
												return CalcStat("BPE",L,N);
											else
												return 0;
											end
										else
											return -CalcStat("NCMRT",L,0.4);
										end
									elseif SN > "EVADEC" then
										if SN < "EVADEPPRAT" then
											if SN == "EVADEPBONUS" then
												return CalcStat("BPEPBonus",L);
											else
												return 0;
											end
										elseif SN > "EVADEPPRAT" then
											if SN == "EVADEPRATP" then
												return CalcStat("BPEPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPEPPRat",L,N);
										end
									else
										return CalcStat("BPEC",L,N);
									end
								else
									return CalcStat("ElfSorrowFirstbornNCMR",L);
								end
							elseif SN > "EVADEPRATPA" then
								if SN < "FATET" then
									if SN < "EVADEPRATPCAPR" then
										if SN < "EVADEPRATPC" then
											if SN == "EVADEPRATPB" then
												return CalcStat("BPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "EVADEPRATPC" then
											if SN == "EVADEPRATPCAP" then
												return CalcStat("BPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPC",L);
										end
									elseif SN > "EVADEPRATPCAPR" then
										if SN < "FATE" then
											if SN == "EVADET" then
												return CalcStat("BPET",L,N);
											else
												return 0;
											end
										elseif SN > "FATE" then
											if SN == "FATEC" then
												return RoundDblDown(StatLinInter("PntMPFateC","CreepProg","ProgBFateC","",L,N,0));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPFate","ItemProgVital","ProgBFate","",L,N,0));
										end
									else
										return CalcStat("BPEPRatPCapR",L);
									end
								elseif SN > "FATET" then
									if SN < "FINESSEPBONUS" then
										if SN < "FINESSE" then
											if SN == "FEARRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										elseif SN > "FINESSE" then
											if SN == "FINESSEC" then
												return StatLinInter("PntMPFinesseC","CreepProg","ProgBFinesseC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPFinesse","ItemProg","ProgBFinesse","AdjUmbarItem",L,N,0);
										end
									elseif SN > "FINESSEPBONUS" then
										if SN < "FINESSEPRATP" then
											if SN == "FINESSEPPRAT" then
												return CalcRatAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "FINESSEPRATP" then
											if SN == "FINESSEPRATPA" then
												return 150;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("FinessePRatPA",L),CalcStat("FinessePRatPB",L),CalcStat("FinessePRatPCap",L),N);
										end
									else
										return 0;
									end
								else
									return RoundDblDown(StatLinInter("PntMPFate","TraitProgVital","ProgBFate","",L,N,0));
								end
							else
								return CalcStat("BPEPRatPA",L);
							end
						else
							return -CalcStat("FateT",L,0.4);
						end
					else
						return 10;
					end
				elseif SN > "FINESSEPRATPB" then
					if SN < "GUARDIANCDBASEVITALITY" then
						if SN < "FREEPBATPROMPOWERP" then
							if SN < "FOODNCPRM" then
								if SN < "FOODNCMRBASEPROG" then
									if SN < "FINESSEPRATPCAPR" then
										if SN > "FINESSEPRATPC" then
											if SN == "FINESSEPRATPCAP" then
												return 50;
											else
												return 0;
											end
										elseif SN == "FINESSEPRATPC" then
											return 0.5;
										else
											return 0;
										end
									elseif SN > "FINESSEPRATPCAPR" then
										if SN < "FIREMIT" then
											if SN == "FINESSET" then
												return StatLinInter("PntMPFinesseT","TraitProg","ProgBFinesse","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										elseif SN > "FIREMIT" then
											if SN == "FIREMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("DmgTypeMit",L,N);
										end
									else
										return CalcStat("FinessePRatPB",L)*CalcStat("FinessePRatPC",L);
									end
								elseif SN > "FOODNCMRBASEPROG" then
									if SN < "FOODNCMRPROG" then
										if SN < "FOODNCMRL" then
											if SN == "FOODNCMRH" then
												return CalcStat("FoodNCMRProg",L,1)*N;
											else
												return 0;
											end
										elseif SN > "FOODNCMRL" then
											if SN == "FOODNCMRM" then
												return CalcStat("FoodNCMRProg",L,3)*N;
											else
												return 0;
											end
										else
											return CalcStat("FoodNCMRProg",L,2)*N;
										end
									elseif SN > "FOODNCMRPROG" then
										if SN < "FOODNCPRH" then
											if SN == "FOODNCPRBASEPROG" then
												if Lm <= 1 then
													return (N*75+300)/60;
												elseif Lm <= 2 then
													return (N*100+350)/120;
												else
													return (N*90+337.5)/90;
												end
											else
												return 0;
											end
										elseif SN > "FOODNCPRH" then
											if SN == "FOODNCPRL" then
												return CalcStat("FoodNCPRProg",L,2)*N;
											else
												return 0;
											end
										else
											return CalcStat("FoodNCPRProg",L,1)*N;
										end
									else
										if Lm <= 50 then
											return CalcStat("FoodNCMRBaseProg",N,L);
										elseif Lm <= 52 then
											return CalcStat("FoodNCMRBaseProg",N,50);
										elseif Lm <= 77 then
											return CalcStat("FoodNCMRBaseProg",N,L-3);
										elseif Lm <= 111 then
											return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+143)/3));
										elseif Lm <= 140 then
											return CalcStat("FoodNCMRBaseProg",N,85);
										elseif Lm <= 217 then
											return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L+200)/4));
										elseif Lm <= 221 then
											return CalcStat("FoodNCMRBaseProg",N,104);
										elseif Lm <= 299 then
											return CalcStat("FoodNCMRBaseProg",N,105);
										elseif Lm <= 320 then
											return CalcStat("FoodNCMRBaseProg",N,RoundDbl((L-53)/(7/3)));
										elseif Lm <= 325 then
											return CalcStat("FoodNCMRBaseProg",N,114);
										elseif Lm <= 326 then
											return CalcStat("FoodNCMRBaseProg",N,115);
										else
											return ExpFmod(CalcStat("FoodNCMRProg",326,N),327,1,L,1);
										end
									end
								else
									if Lm <= 1 then
										return (N*70.2+210)/60;
									elseif Lm <= 2 then
										return (N*84+216)/120;
									else
										return (N*83.7+270)/90;
									end
								end
							elseif SN > "FOODNCPRM" then
								if SN < "FREEPAUDACITYREDP" then
									if SN < "FREEPAUDACITYDMGP" then
										if SN < "FOODRESIST" then
											if SN == "FOODNCPRPROG" then
												if Lm <= 1 then
													return CalcStat("FoodNCPRBaseProg",N,L);
												elseif Lm <= 6 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.25;
												elseif Lm <= 24 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.5;
												elseif Lm <= 48 then
													return CalcStat("FoodNCPRBaseProg",N,L)-0.75;
												elseif Lm <= 61 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+1;
												elseif Lm <= 66 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+2;
												elseif Lm <= 72 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+3;
												elseif Lm <= 326 then
													return RoundDblDown(CalcStat("FoodNCPRProg",48,N))+4;
												else
													return ExpFmod(CalcStat("FoodNCPRProg",326,N),327,1,L,1);
												end
											else
												return 0;
											end
										elseif SN > "FOODRESIST" then
											if SN == "FREEPAUDACITYCCDP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return 0.5;
												else
													return 0.4;
												end
											else
												return 0;
											end
										else
											return StatLinInter("PntMPFoodResist","ItemProg","ProgBResist","AdjUmbarItem",L,N,0);
										end
									elseif SN > "FREEPAUDACITYDMGP" then
										if SN < "FREEPAUDACITYMELREDP" then
											if SN == "FREEPAUDACITYMELDMGP" then
												return CalcStat("FreepAudacityDmgP",L);
											else
												return 0;
											end
										elseif SN > "FREEPAUDACITYMELREDP" then
											if SN == "FREEPAUDACITYMORALEP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 16 then
													return LinFmod(1,0.5,1.4,1,16,L);
												else
													return 1.5;
												end
											else
												return 0;
											end
										else
											return CalcStat("FreepAudacityRedP",L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 16 then
											return 0.75;
										elseif Lm <= 36 then
											return LinFmod(1,1.2,1.25,17,36,L);
										else
											return 1.25;
										end
									end
								elseif SN > "FREEPAUDACITYREDP" then
									if SN < "FREEPAUDACITYTACREDP" then
										if SN < "FREEPAUDACITYRNGREDP" then
											if SN == "FREEPAUDACITYRNGDMGP" then
												return CalcStat("FreepAudacityDmgP",L);
											else
												return 0;
											end
										elseif SN > "FREEPAUDACITYRNGREDP" then
											if SN == "FREEPAUDACITYTACDMGP" then
												return CalcStat("FreepAudacityDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepAudacityRedP",L);
										end
									elseif SN > "FREEPAUDACITYTACREDP" then
										if SN < "FREEPBATPROMHEALTHP" then
											if SN == "FREEPBATPROMDMGP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 1 then
													return 0.005;
												elseif Lm <= 14 then
													return LinFmod(1,0.005,0.065,2,14,L);
												else
													return 0.075;
												end
											else
												return 0;
											end
										elseif SN > "FREEPBATPROMHEALTHP" then
											if SN == "FREEPBATPROMMELDMGP" then
												return CalcStat("FreepBatPromDmgP",L);
											else
												return 0;
											end
										else
											return CalcStat("FreepBatPromVitalP",L);
										end
									else
										return CalcStat("FreepAudacityRedP",L);
									end
								else
									if Lm <= 0 then
										return 0;
									elseif Lm <= 16 then
										return 1.5;
									else
										return 0.55;
									end
								end
							else
								return CalcStat("FoodNCPRProg",L,3)*N;
							end
						elseif SN > "FREEPBATPROMPOWERP" then
							if SN < "GUARDIANCDAGILITYTOOUTHEAL" then
								if SN < "GRDTENDERIZECRITHIT" then
									if SN < "FROSTMIT" then
										if SN < "FREEPBATPROMTACDMGP" then
											if SN == "FREEPBATPROMRNGDMGP" then
												return CalcStat("FreepBatPromDmgP",L);
											else
												return 0;
											end
										elseif SN > "FREEPBATPROMTACDMGP" then
											if SN == "FREEPBATPROMVITALP" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 5 then
													return LinFmod(1,1.02,1.1,1,5,L);
												elseif Lm <= 10 then
													return LinFmod(1,1.1,1.15,5,10,L);
												elseif Lm <= 15 then
													return LinFmod(1,1.15,1.2,10,15,L);
												else
													return CalcStat("FreepBatPromVitalP",15);
												end
											else
												return 0;
											end
										else
											return CalcStat("FreepBatPromDmgP",L);
										end
									elseif SN > "FROSTMIT" then
										if SN < "GRDCRITDEF" then
											if SN == "FROSTMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										elseif SN > "GRDCRITDEF" then
											if SN == "GRDRELENTLASSFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.2);
											else
												return 0;
											end
										else
											return CalcStat("CritDef",L);
										end
									else
										return CalcStat("DmgTypeMit",L,N);
									end
								elseif SN > "GRDTENDERIZECRITHIT" then
									if SN < "GRDTENDERIZET5CRITHIT" then
										if SN < "GRDTENDERIZET3CRITHIT" then
											if SN == "GRDTENDERIZET2CRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "GRDTENDERIZET3CRITHIT" then
											if SN == "GRDTENDERIZET4CRITHIT" then
												return CalcStat("CritHitT",L,CalcStat("Trait47101316Choice",N)*0.2);
											else
												return 0;
											end
										else
											return CalcStat("CritHitT",L,CalcStat("Trait357912Choice",N)*0.2);
										end
									elseif SN > "GRDTENDERIZET5CRITHIT" then
										if SN < "GUARDIANCDAGILITYTOCRITHIT" then
											if SN == "GRDWARDTACTTACMIT" then
												return CalcStat("TacMitT",L,CalcStat("Trait12345Choice",N)*0.2);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDAGILITYTOCRITHIT" then
											if SN == "GUARDIANCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									else
										return CalcStat("CritHitT",L,CalcStat("Trait58121620Choice",N)*0.2);
									end
								else
									return CalcStat("CritHitT",L,CalcStat("Trait12345Choice",N)*0.2);
								end
							elseif SN > "GUARDIANCDAGILITYTOOUTHEAL" then
								if SN < "GUARDIANCDBASEFATE" then
									if SN < "GUARDIANCDARMOURTONONPHYMIT" then
										if SN < "GUARDIANCDAGILITYTOPHYMAS" then
											if SN == "GUARDIANCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDAGILITYTOPHYMAS" then
											if SN == "GUARDIANCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "GUARDIANCDARMOURTONONPHYMIT" then
										if SN < "GUARDIANCDARMOURTYPE" then
											if SN == "GUARDIANCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDARMOURTYPE" then
											if SN == "GUARDIANCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 0.2;
									end
								elseif SN > "GUARDIANCDBASEFATE" then
									if SN < "GUARDIANCDBASEMORALE" then
										if SN < "GUARDIANCDBASEICPR" then
											if SN == "GUARDIANCDBASEICMR" then
												return CalcStat("ClassBaseICMRH",L);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASEICPR" then
											if SN == "GUARDIANCDBASEMIGHT" then
												return CalcStat("ClassBaseMightH",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									elseif SN > "GUARDIANCDBASEMORALE" then
										if SN < "GUARDIANCDBASENCPR" then
											if SN == "GUARDIANCDBASENCMR" then
												return CalcStat("ClassBaseNCMRH",L);
											else
												return 0;
											end
										elseif SN > "GUARDIANCDBASENCPR" then
											if SN == "GUARDIANCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseFate",L);
								end
							else
								return 2;
							end
						else
							return CalcStat("FreepBatPromVitalP",L);
						end
					elseif SN > "GUARDIANCDBASEVITALITY" then
						if SN < "HELFSORROWUNDYINGWILL" then
							if SN < "GUARDIANCDMIGHTTOTACMIT" then
								if SN < "GUARDIANCDFATETOPOWER" then
									if SN < "GUARDIANCDCALCTYPENONPHYMIT" then
										if SN > "GUARDIANCDBASEWILL" then
											if SN == "GUARDIANCDCALCTYPECOMPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN == "GUARDIANCDBASEWILL" then
											return CalcStat("ClassBaseWillL",L);
										else
											return 0;
										end
									elseif SN > "GUARDIANCDCALCTYPENONPHYMIT" then
										if SN < "GUARDIANCDCANBLOCK" then
											if SN == "GUARDIANCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDCANBLOCK" then
											if SN == "GUARDIANCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 14;
									end
								elseif SN > "GUARDIANCDFATETOPOWER" then
									if SN < "GUARDIANCDMIGHTTOOUTHEAL" then
										if SN < "GUARDIANCDMIGHTTOBLOCK" then
											if SN == "GUARDIANCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDMIGHTTOBLOCK" then
											if SN == "GUARDIANCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "GUARDIANCDMIGHTTOOUTHEAL" then
										if SN < "GUARDIANCDMIGHTTOPHYMAS" then
											if SN == "GUARDIANCDMIGHTTOPARRY" then
												return 2;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDMIGHTTOPHYMAS" then
											if SN == "GUARDIANCDMIGHTTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 3;
									end
								else
									return 1;
								end
							elseif SN > "GUARDIANCDMIGHTTOTACMIT" then
								if SN < "GUARDIANCDWILLTOOUTHEAL" then
									if SN < "GUARDIANCDVITALITYTOICMR" then
										if SN < "GUARDIANCDPHYMITTONONPHYMIT" then
											if SN == "GUARDIANCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDPHYMITTONONPHYMIT" then
											if SN == "GUARDIANCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "GUARDIANCDVITALITYTOICMR" then
										if SN < "GUARDIANCDVITALITYTONCMR" then
											if SN == "GUARDIANCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDVITALITYTONCMR" then
											if SN == "GUARDIANCDWILLTOFINESSE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 0.012;
									end
								elseif SN > "GUARDIANCDWILLTOOUTHEAL" then
									if SN < "GUARDIANCDWILLTOTACMIT" then
										if SN < "GUARDIANCDWILLTOPHYMIT" then
											if SN == "GUARDIANCDWILLTOPHYMAS" then
												return 1;
											else
												return 0;
											end
										elseif SN > "GUARDIANCDWILLTOPHYMIT" then
											if SN == "GUARDIANCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1.5;
										end
									elseif SN > "GUARDIANCDWILLTOTACMIT" then
										if SN < "HELFPEACEELDARMORALE" then
											if SN == "HELFFADINGFIRSTBORNFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										elseif SN > "HELFPEACEELDARMORALE" then
											if SN == "HELFPEACEELDARNCMR" then
												return CalcStat("NCMRT",L,0.6);
											else
												return 0;
											end
										else
											return CalcStat("MoraleT",L,1.0);
										end
									else
										return 1.5;
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "HELFSORROWUNDYINGWILL" then
							if SN < "HNTPURGEPOISONRESIST" then
								if SN < "HIGHELFRDTRAITWILL" then
									if SN < "HIGHELFRDPSVTWONAME" then
										if SN < "HIGHELFRDPSVONENAME" then
											if SN == "HELFTHOSEWHOREMAINWILL" then
												return CalcStat("WillT",L,1.0);
											else
												return 0;
											end
										elseif SN > "HIGHELFRDPSVONENAME" then
											if SN == "HIGHELFRDPSVONEWILL" then
												return CalcStat("HElfThoseWhoRemainWill",L);
											else
												return 0;
											end
										else
											return "Those Who Remain";
										end
									elseif SN > "HIGHELFRDPSVTWONAME" then
										if SN < "HIGHELFRDTRAITMORALE" then
											if SN == "HIGHELFRDTRAITFATE" then
												return CalcStat("HElfFadingFirstbornFate",L);
											else
												return 0;
											end
										elseif SN > "HIGHELFRDTRAITMORALE" then
											if SN == "HIGHELFRDTRAITNCMR" then
												return CalcStat("HElfPeaceEldarNCMR",L);
											else
												return 0;
											end
										else
											return CalcStat("HElfPeaceEldarMorale",L);
										end
									else
										return "";
									end
								elseif SN > "HIGHELFRDTRAITWILL" then
									if SN < "HNTBREACHFINDERRNGMIT" then
										if SN < "HNTARMOURRENDEVADE" then
											if SN == "HNTARMOURRENDBLOCK" then
												return -CalcStat("ShieldBlock",L,CalcStat("Trait23456Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "HNTARMOURRENDEVADE" then
											if SN == "HNTARMOURRENDPARRY" then
												return -CalcStat("ParryT",L,CalcStat("Trait23456Choice",N)*0.4);
											else
												return 0;
											end
										else
											return -CalcStat("EvadeT",L,CalcStat("Trait23456Choice",N)*0.4);
										end
									elseif SN > "HNTBREACHFINDERRNGMIT" then
										if SN < "HNTCAMPFIRENCPR" then
											if SN == "HNTCAMPFIRENCMR" then
												return CalcStat("NCMRT",L,1.2);
											else
												return 0;
											end
										elseif SN > "HNTCAMPFIRENCPR" then
											if SN == "HNTPRECISIONSTANCEFINESSE" then
												return CalcStat("FinesseT",L,1.6);
											else
												return 0;
											end
										else
											return CalcStat("NCPRT",L,1.2);
										end
									else
										return (-20)*L;
									end
								else
									return CalcStat("HElfSorrowUndyingWill",L);
								end
							elseif SN > "HNTPURGEPOISONRESIST" then
								if SN < "HOBBITRDTRAITNCMR" then
									if SN < "HOBBITRDPSVONEMIGHT" then
										if SN < "HNTRAPIDFIRESEL" then
											if SN == "HNTRAPIDFIREPHYMAS" then
												return CalcStat("PhyMasT",L,CalcStat("HntRapidFireSel",N)*0.2);
											else
												return 0;
											end
										elseif SN > "HNTRAPIDFIRESEL" then
											if SN == "HNTSTRENGTHSTANCECRITHIT" then
												return CalcStat("CritHitT",L,4);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 8 then
												return L+2;
											elseif Lm <= 10 then
												return 2*L-6;
											else
												return CalcStat("HntRapidFireSel",10);
											end
										end
									elseif SN > "HOBBITRDPSVONEMIGHT" then
										if SN < "HOBBITRDPSVTWONAME" then
											if SN == "HOBBITRDPSVONENAME" then
												return "Hobbit-stature";
											else
												return 0;
											end
										elseif SN > "HOBBITRDPSVTWONAME" then
											if SN == "HOBBITRDTRAITMIGHT" then
												return CalcStat("HobSmallSizeMight",L);
											else
												return 0;
											end
										else
											return "";
										end
									else
										return CalcStat("HobHobbitStatureMight",L);
									end
								elseif SN > "HOBBITRDTRAITNCMR" then
									if SN < "HOBRAPIDRECOVERYNCMR" then
										if SN < "HOBHOBBITSTATUREMIGHT" then
											if SN == "HOBBITRDTRAITVITALITY" then
												return CalcStat("HobHobbitToughnVitality",L);
											else
												return 0;
											end
										elseif SN > "HOBHOBBITSTATUREMIGHT" then
											if SN == "HOBHOBBITTOUGHNVITALITY" then
												return CalcStat("VitalityT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,1.0);
										end
									elseif SN > "HOBRAPIDRECOVERYNCMR" then
										if SN < "HOPEMORALEP" then
											if SN == "HOBSMALLSIZEMIGHT" then
												return -CalcStat("MightT",L,0.4);
											else
												return 0;
											end
										elseif SN > "HOPEMORALEP" then
											if SN == "HUNTERCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 5 then
												return DataTableValue({-0.99,-0.97,-0.95,-0.9,-0.85,-0.8,-0.65,-0.6,-0.5,-0.4,-0.3,-0.2,-0.15,-0.1,-0.05,0,0.01,0.02,0.03,0.04,0.05},L+16);
											else
												return 0.05;
											end
										end
									else
										return CalcStat("NCMRT",L,0.6);
									end
								else
									return CalcStat("HobRapidRecoveryNCMR",L);
								end
							else
								return CalcStat("PoisonResistT",L,4);
							end
						else
							return -CalcStat("WillT",L,0.4);
						end
					else
						return CalcStat("ClassBaseVitality",L);
					end
				else
					return CalcStat("BRatRounded",L,"BRatStandard");
				end
			elseif SN > "HUNTERCDAGILITYTOEVADE" then
				if SN < "LOREMASTERCDCALCTYPENONPHYMIT" then
					if SN < "INHEALPRATP" then
						if SN < "HUNTERCDMIGHTTOPHYMAS" then
							if SN < "HUNTERCDBASEMORALE" then
								if SN < "HUNTERCDARMOURTONONPHYMIT" then
									if SN < "HUNTERCDAGILITYTOPHYMAS" then
										if SN > "HUNTERCDAGILITYTOOUTHEAL" then
											if SN == "HUNTERCDAGILITYTOPARRY" then
												return 1;
											else
												return 0;
											end
										elseif SN == "HUNTERCDAGILITYTOOUTHEAL" then
											return 3;
										else
											return 0;
										end
									elseif SN > "HUNTERCDAGILITYTOPHYMAS" then
										if SN < "HUNTERCDAGILITYTOTACMIT" then
											if SN == "HUNTERCDAGILITYTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDAGILITYTOTACMIT" then
											if SN == "HUNTERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 3;
									end
								elseif SN > "HUNTERCDARMOURTONONPHYMIT" then
									if SN < "HUNTERCDBASEFATE" then
										if SN < "HUNTERCDARMOURTYPE" then
											if SN == "HUNTERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "HUNTERCDARMOURTYPE" then
											if SN == "HUNTERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityH",L);
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "HUNTERCDBASEFATE" then
										if SN < "HUNTERCDBASEICPR" then
											if SN == "HUNTERCDBASEICMR" then
												return CalcStat("ClassBaseICMRM",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASEICPR" then
											if SN == "HUNTERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									else
										return CalcStat("ClassBaseFate",L);
									end
								else
									return 0.2;
								end
							elseif SN > "HUNTERCDBASEMORALE" then
								if SN < "HUNTERCDCALCTYPETACMIT" then
									if SN < "HUNTERCDBASEVITALITY" then
										if SN < "HUNTERCDBASENCPR" then
											if SN == "HUNTERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRM",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDBASENCPR" then
											if SN == "HUNTERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									elseif SN > "HUNTERCDBASEVITALITY" then
										if SN < "HUNTERCDCALCTYPECOMPHYMIT" then
											if SN == "HUNTERCDBASEWILL" then
												return CalcStat("ClassBaseWillL",L);
											else
												return 0;
											end
										elseif SN > "HUNTERCDCALCTYPECOMPHYMIT" then
											if SN == "HUNTERCDCALCTYPENONPHYMIT" then
												return 13;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ClassBaseVitality",L);
									end
								elseif SN > "HUNTERCDCALCTYPETACMIT" then
									if SN < "HUNTERCDMIGHTTOCRITHIT" then
										if SN < "HUNTERCDFATETOPOWER" then
											if SN == "HUNTERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "HUNTERCDFATETOPOWER" then
											if SN == "HUNTERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "HUNTERCDMIGHTTOCRITHIT" then
										if SN < "HUNTERCDMIGHTTOFINESSE" then
											if SN == "HUNTERCDMIGHTTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDMIGHTTOFINESSE" then
											if SN == "HUNTERCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1.5;
									end
								else
									return 26;
								end
							else
								return CalcStat("ClassBaseMorale",L);
							end
						elseif SN > "HUNTERCDMIGHTTOPHYMAS" then
							if SN < "ICMRT" then
								if SN < "HUNTERCDWILLTOFINESSE" then
									if SN < "HUNTERCDVITALITYTOICMR" then
										if SN < "HUNTERCDPHYMITTONONPHYMIT" then
											if SN == "HUNTERCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "HUNTERCDPHYMITTONONPHYMIT" then
											if SN == "HUNTERCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "HUNTERCDVITALITYTOICMR" then
										if SN < "HUNTERCDVITALITYTONCMR" then
											if SN == "HUNTERCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										elseif SN > "HUNTERCDVITALITYTONCMR" then
											if SN == "HUNTERCDWILLTOCRITHIT" then
												return 0.5;
											else
												return 0;
											end
										else
											return 0.12;
										end
									else
										return 0.012;
									end
								elseif SN > "HUNTERCDWILLTOFINESSE" then
									if SN < "HUNTERCDWILLTORESIST" then
										if SN < "HUNTERCDWILLTOPHYMAS" then
											if SN == "HUNTERCDWILLTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "HUNTERCDWILLTOPHYMAS" then
											if SN == "HUNTERCDWILLTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "HUNTERCDWILLTORESIST" then
										if SN < "ICMRC" then
											if SN == "ICMR" then
												return StatLinInter("PntMPICMR","ItemProgVital","ProgBICMR","",L,N,1);
											else
												return 0;
											end
										elseif SN > "ICMRC" then
											if SN == "ICMRCRAW" then
												return StatLinInter("PntMPICMRC","CreepProg","ProgBICMRC","",L,N,99);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPICMRC","CreepProg","ProgBICMRC","",L,N,0);
										end
									else
										return 1;
									end
								else
									return 1.5;
								end
							elseif SN > "ICMRT" then
								if SN < "INDMGPRATPA" then
									if SN < "ICPRT" then
										if SN < "ICPRC" then
											if SN == "ICPR" then
												return StatLinInter("PntMPICPR","ItemProgVital","ProgBICPR","",L,N,99);
											else
												return 0;
											end
										elseif SN > "ICPRC" then
											if SN == "ICPRCRAW" then
												return StatLinInter("PntMPICPRC","CreepProg","ProgBICPRC","",L,N,99);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPICPRC","CreepProg","ProgBICPRC","",L,N,0);
										end
									elseif SN > "ICPRT" then
										if SN < "INDMGPPRAT" then
											if SN == "INDMGPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "INDMGPPRAT" then
											if SN == "INDMGPRATP" then
												return CalcPercAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("InDmgPRatPA",L),CalcStat("InDmgPRatPB",L),CalcStat("InDmgPRatPCapR",L),N);
										end
									else
										return StatLinInter("PntMPICPR","TraitProgVital","ProgBICPR","",L,N,99);
									end
								elseif SN > "INDMGPRATPA" then
									if SN < "INDMGPRATPCAPR" then
										if SN < "INDMGPRATPC" then
											if SN == "INDMGPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "INDMGPRATPC" then
											if SN == "INDMGPRATPCAP" then
												return 400;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "INDMGPRATPCAPR" then
										if SN < "INHEALPBONUS" then
											if SN == "INHEAL" then
												return StatLinInter("PntMPInHeal","ItemProg","ProgBInHeal","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										elseif SN > "INHEALPBONUS" then
											if SN == "INHEALPPRAT" then
												return CalcRatAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("InDmgPRatPB",L)*CalcStat("InDmgPRatPC",L);
									end
								else
									return 1200;
								end
							else
								return StatLinInter("PntMPICMR","TraitProgVital","ProgBICMR","",L,N,1);
							end
						else
							return 2;
						end
					elseif SN > "INHEALPRATP" then
						if SN < "LMPREPFORWARPETTACMAS" then
							if SN < "LEVELCAP" then
								if SN < "INSTRMORALE" then
									if SN < "INHEALPRATPC" then
										if SN > "INHEALPRATPA" then
											if SN == "INHEALPRATPB" then
												return CalcStat("BRatRounded",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN == "INHEALPRATPA" then
											return 75;
										else
											return 0;
										end
									elseif SN > "INHEALPRATPC" then
										if SN < "INHEALPRATPCAPR" then
											if SN == "INHEALPRATPCAP" then
												return 25;
											else
												return 0;
											end
										elseif SN > "INHEALPRATPCAPR" then
											if SN == "INHEALT" then
												return StatLinInter("PntMPInHeal","TraitProg","ProgBInHeal","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("InHealPRatPB",L)*CalcStat("InHealPRatPC",L);
										end
									else
										return 0.5;
									end
								elseif SN > "INSTRMORALE" then
									if SN < "ITEMPROGVIRTUEMASTERY" then
										if SN < "ITEMPROG" then
											if SN == "INSTRPOWER" then
												if Lm <= 9 then
													return RoundDbl(0.95*L+12.6);
												elseif Lm <= 18 then
													return RoundDbl(1.95*L+3);
												elseif Lm <= 26 then
													return RoundDbl(2.95*L-15.55);
												elseif Lm <= 35 then
													return RoundDbl(3.95*L-42.15);
												elseif Lm <= 42 then
													return RoundDbl(4.95*L-77.7);
												elseif Lm <= 45 then
													return RoundDbl(5.95*L-120.35);
												elseif Lm <= 105 then
													return RoundDbl(6*L-124);
												else
													return CalcStat("ProgExtMpExpRnd",L,CalcStat("InstrPower",105));
												end
											else
												return 0;
											end
										elseif SN > "ITEMPROG" then
											if SN == "ITEMPROGCLASSIC" then
												return {{1,25,50,60,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,50,60,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										else
											return {{1,25,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
										end
									elseif SN > "ITEMPROGVIRTUEMASTERY" then
										if SN < "ITEMPROGVITAL" then
											if SN == "ITEMPROGVIRTUEMORALE" then
												return {{0,1,2,50,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{0,1,2,50,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
											else
												return 0;
											end
										elseif SN > "ITEMPROGVITAL" then
											if SN == "L" then
												return L;
											else
												return 0;
											end
										else
											return {{1,25,50,60,79,80,200,225,300,349,350,399,400,449,450,499,500,549,550,599},{1,25,50,60,75,76,100,105,106,115,116,120,121,130,131,140,141,150,151,160}};
										end
									else
										return {{0,1,25,79,80,200,225,300,349,350,400,449,450,499,500,549,550,599},{0,1,25,75,76,100,105,106,115,116,121,130,131,140,141,150,151,160}};
									end
								else
									if Lm <= 105 then
										return RoundDbl(4.04*L);
									else
										return CalcStat("ProgExtLowExpRnd",L,CalcStat("InstrMorale",105));
									end
								end
							elseif SN > "LEVELCAP" then
								if SN < "LMCATMINTBEARARMOUR" then
									if SN < "LIGHTMITT" then
										if SN < "LI2REFORGEILVL" then
											if SN == "LI2ILVLCAP" then
												return 515;
											else
												return 0;
											end
										elseif SN > "LI2REFORGEILVL" then
											if SN == "LIGHTMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										else
											if Lm <= 145 then
												return CalcStat("AwardILvlI",L);
											else
												return N;
											end
										end
									elseif SN > "LIGHTMITT" then
										if SN < "LIGHTNINGMITT" then
											if SN == "LIGHTNINGMIT" then
												return CalcStat("DmgTypeMit",L,N);
											else
												return 0;
											end
										elseif SN > "LIGHTNINGMITT" then
											if SN == "LMANCIENTWISDOMWILL" then
												if Lm <= 105 then
													return RoundDbl(1.1*L);
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmAncientWisdomWill",105));
												end
											else
												return 0;
											end
										else
											return CalcStat("DmgTypeMitT",L,N);
										end
									else
										return CalcStat("DmgTypeMitT",L,N);
									end
								elseif SN > "LMCATMINTBEARARMOUR" then
									if SN < "LMKNOWLEDGELMCRYRES" then
										if SN < "LMEVASIONRAVENARMOUR" then
											if SN == "LMEAGLENOBILITYICPR" then
												return CalcStat("ICPRT",L,0.8);
											else
												return 0;
											end
										elseif SN > "LMEVASIONRAVENARMOUR" then
											if SN == "LMHEARTYDIETMORALE" then
												return CalcStat("Morale",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("ArmourT",L,0.8);
										end
									elseif SN > "LMKNOWLEDGELMCRYRES" then
										if SN < "LMKNOWLEDGELMSONGRES" then
											if SN == "LMKNOWLEDGELMPHYRES" then
												return -CalcStat("PhyResistT",L,0.6);
											else
												return 0;
											end
										elseif SN > "LMKNOWLEDGELMSONGRES" then
											if SN == "LMKNOWLEDGELMTACRES" then
												return -CalcStat("TacResistT",L,0.6);
											else
												return 0;
											end
										else
											return -CalcStat("SongResistT",L,0.6);
										end
									else
										return -CalcStat("CryResistT",L,0.6);
									end
								else
									return CalcStat("BaseArmour",L,15);
								end
							else
								return 150;
							end
						elseif SN > "LMPREPFORWARPETTACMAS" then
							if SN < "LOREMASTERCDARMOURTOCOMPHYMIT" then
								if SN < "LMSWSTAFFBUGPARRY" then
									if SN < "LMRINGOFFIREBLOCK" then
										if SN < "LMRAVENSHOTRAVWINGFIXTACMIT" then
											if SN == "LMPREPFORWARTACMAS" then
												return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "LMRAVENSHOTRAVWINGFIXTACMIT" then
											if SN == "LMRAVENSHOTRAVWINGTACMIT" then
												return CalcStat("TacMit",L,2);
											else
												return 0;
											end
										else
											return CalcStat("TacMitT",L,2);
										end
									elseif SN > "LMRINGOFFIREBLOCK" then
										if SN < "LMRINGOFFIREPARRY" then
											if SN == "LMRINGOFFIREEVADE" then
												return -CalcStat("EvadeT",L,2.4);
											else
												return 0;
											end
										elseif SN > "LMRINGOFFIREPARRY" then
											if SN == "LMSWSTAFFBUGMORALE" then
												if Lm <= 105 then
													return RoundDbl(LinFmod(1,16.2,696.9,1,105,L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("LmSwStaffBugMorale",105));
												end
											else
												return 0;
											end
										else
											return -CalcStat("ParryT",L,2.4);
										end
									else
										return -CalcStat("BlockT",L,2.4);
									end
								elseif SN > "LMSWSTAFFBUGPARRY" then
									if SN < "LOREMASTERCDAGILITYTOCRITHIT" then
										if SN < "LOE" then
											if SN == "LMSWSTAFFBUGPARRYOLD" then
												if Lm <= 105 then
													return RoundDbl(44.44*L);
												else
													return CalcStat("ProgExtHighLinExpRnd",L,CalcStat("LmSwStaffBugParryOld",105));
												end
											else
												return 0;
											end
										elseif SN > "LOE" then
											if SN == "LOEPASSIVE" then
												if Lm <= 115 then
													return 0;
												elseif Lm <= 119 then
													return 20*L-2300;
												else
													return CalcStat("LoEPassive",119);
												end
											else
												return 0;
											end
										else
											return 2*N;
										end
									elseif SN > "LOREMASTERCDAGILITYTOCRITHIT" then
										if SN < "LOREMASTERCDAGILITYTOFINESSE" then
											if SN == "LOREMASTERCDAGILITYTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDAGILITYTOFINESSE" then
											if SN == "LOREMASTERCDAGILITYTOTACMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								else
									if Lm <= 140 then
										return CalcStat("U371LegacyStatFix",L,"LmSwStaffBugParryOld");
									else
										return CalcStat("LMSwStaffBugParry",140);
									end
								end
							elseif SN > "LOREMASTERCDARMOURTOCOMPHYMIT" then
								if SN < "LOREMASTERCDBASEMIGHT" then
									if SN < "LOREMASTERCDBASEAGILITY" then
										if SN < "LOREMASTERCDARMOURTOTACMIT" then
											if SN == "LOREMASTERCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDARMOURTOTACMIT" then
											if SN == "LOREMASTERCDARMOURTYPE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.2;
										end
									elseif SN > "LOREMASTERCDBASEAGILITY" then
										if SN < "LOREMASTERCDBASEICMR" then
											if SN == "LOREMASTERCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEICMR" then
											if SN == "LOREMASTERCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRL",L);
										end
									else
										return CalcStat("ClassBaseAgilityL",L);
									end
								elseif SN > "LOREMASTERCDBASEMIGHT" then
									if SN < "LOREMASTERCDBASEPOWER" then
										if SN < "LOREMASTERCDBASENCMR" then
											if SN == "LOREMASTERCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASENCMR" then
											if SN == "LOREMASTERCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									elseif SN > "LOREMASTERCDBASEPOWER" then
										if SN < "LOREMASTERCDBASEWILL" then
											if SN == "LOREMASTERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDBASEWILL" then
											if SN == "LOREMASTERCDCALCTYPECOMPHYMIT" then
												return 12;
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseWillH",L);
										end
									else
										return CalcStat("ClassBasePower",L);
									end
								else
									return CalcStat("ClassBaseMightM",L);
								end
							else
								return 1;
							end
						else
							return CalcStat("TacMas",L,3.0);
						end
					else
						return CalcPercAB(CalcStat("InHealPRatPA",L),CalcStat("InHealPRatPB",L),CalcStat("InHealPRatPCap",L),N);
					end
				elseif SN > "LOREMASTERCDCALCTYPENONPHYMIT" then
					if SN < "MARINERCDBASENCPR" then
						if SN < "MANDIMMANKINDWILL" then
							if SN < "LOREMASTERCDWILLTOCRITHIT" then
								if SN < "LOREMASTERCDMIGHTTOPARRY" then
									if SN < "LOREMASTERCDFATETOPOWER" then
										if SN > "LOREMASTERCDCALCTYPETACMIT" then
											if SN == "LOREMASTERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN == "LOREMASTERCDCALCTYPETACMIT" then
											return 25;
										else
											return 0;
										end
									elseif SN > "LOREMASTERCDFATETOPOWER" then
										if SN < "LOREMASTERCDMIGHTTOCRITHIT" then
											if SN == "LOREMASTERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDMIGHTTOCRITHIT" then
											if SN == "LOREMASTERCDMIGHTTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1;
									end
								elseif SN > "LOREMASTERCDMIGHTTOPARRY" then
									if SN < "LOREMASTERCDTACMASTOOUTHEAL" then
										if SN < "LOREMASTERCDPHYMITTOCOMPHYMIT" then
											if SN == "LOREMASTERCDMIGHTTOTACMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDPHYMITTOCOMPHYMIT" then
											if SN == "LOREMASTERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "LOREMASTERCDTACMASTOOUTHEAL" then
										if SN < "LOREMASTERCDVITALITYTOMORALE" then
											if SN == "LOREMASTERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDVITALITYTOMORALE" then
											if SN == "LOREMASTERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 4.5;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "LOREMASTERCDWILLTOCRITHIT" then
								if SN < "LVLBONUSPOWRES" then
									if SN < "LOREMASTERCDWILLTOTACMAS" then
										if SN < "LOREMASTERCDWILLTOPHYMIT" then
											if SN == "LOREMASTERCDWILLTOEVADE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "LOREMASTERCDWILLTOPHYMIT" then
											if SN == "LOREMASTERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "LOREMASTERCDWILLTOTACMAS" then
										if SN < "LVLBONUSMORRES" then
											if SN == "LOREMASTERCDWILLTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "LVLBONUSMORRES" then
											if SN == "LVLBONUSPHYDMG" then
												return 0.1;
											else
												return 0;
											end
										else
											return 0.1;
										end
									else
										return 3;
									end
								elseif SN > "LVLBONUSPOWRES" then
									if SN < "LVLTOILVL" then
										if SN < "LVLEXPCOST" then
											if SN == "LVLBONUSTACDMG" then
												return 0.1;
											else
												return 0;
											end
										elseif SN > "LVLEXPCOST" then
											if SN == "LVLEXPCOSTTOT" then
												if Lm <= 1 then
													return 0;
												else
													return CalcStat("LvlExpCostTot",L-1)+CalcStat("LvlExpCost",L);
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 0;
											elseif Lm <= 5 then
												return RoundDbl(12.5*L*L+12.5666666666667*L+24.8666666666667);
											elseif Lm <= 10 then
												return RoundDbl(33.8*L*L-179.48*L+452.6);
											elseif Lm <= 15 then
												return RoundDbl(55.05*L*L-583.77*L+2370.5);
											elseif Lm <= 20 then
												return RoundDbl(76.2*L*L-1196.96*L+6809);
											elseif Lm <= 25 then
												return RoundDbl(97.4*L*L-2023*L+14849.8);
											elseif Lm <= 30 then
												return RoundDbl(118.7*L*L-3066.02 *L+27612.8);
											elseif Lm <= 35 then
												return RoundDbl(139.95*L*L-4319.23*L+46084.1);
											elseif Lm <= 40 then
												return RoundDbl(161.2*L*L-5785.04*L+71356.2);
											elseif Lm <= 45 then
												return RoundDbl(182.5*L*L-7467.38*L+104569.8);
											elseif Lm <= 50 then
												return RoundDbl(203.8*L*L-9363.48*L+146761.8);
											elseif Lm <= 55 then
												return RoundDbl(225.05*L*L-11467.77*L+198851.3);
											elseif Lm <= 60 then
												return RoundDbl(246.3*L*L-13784.46*L+261988);
											elseif Lm <= 70 then
												return RoundDbl(ExpFmod(CalcStat("LvlExpCost",60),61,5.071,L,nil,3.485));
											elseif Lm <= 75 then
												return RoundDbl(ExpFmod(CalcStat("LvlExpCost",70),71,5.072,L,nil,-0.95));
											else
												return ExpFmod(CalcStat("LvlExpCost",75),76,5,L,0,-0.5);
											end
										end
									elseif SN > "LVLTOILVL" then
										if SN < "MAINC" then
											if SN == "MAIN" then
												return RoundDblDown(StatLinInter("PntMPMain","ItemProg","ProgBMain","AdjUmbarItem",L,N,0));
											else
												return 0;
											end
										elseif SN > "MAINC" then
											if SN == "MAINT" then
												return RoundDblDown(StatLinInter("PntMPMain","TraitProg","ProgBMain","AdjUmbarTrait",L,N,0));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPMainC","CreepProg","ProgBMainC","",L,N,0));
										end
									else
										if Lm <= 106 then
											return LinFmod(1,225,300,105,106,L);
										elseif Lm <= 115 then
											return LinFmod(1,300,349,106,115,L);
										elseif Lm <= 116 then
											return LinFmod(1,349,350,115,116,L);
										elseif Lm <= 120 then
											return LinFmod(1,350,399,116,120,L);
										elseif Lm <= 121 then
											return LinFmod(1,399,400,120,121,L);
										elseif Lm <= 130 then
											return LinFmod(1,400,449,121,130,L);
										elseif Lm <= 131 then
											return LinFmod(1,449,450,130,131,L);
										elseif Lm <= 140 then
											return LinFmod(1,450,499,131,140,L);
										elseif Lm <= 141 then
											return LinFmod(1,499,500,140,141,L);
										else
											return LinFmod(1,500,549,141,150,L);
										end
									end
								else
									return CalcStat("SkillPowerCost",L,N);
								end
							else
								return 1;
							end
						elseif SN > "MANDIMMANKINDWILL" then
							if SN < "MARINERCDAGILITYTOOUTHEAL" then
								if SN < "MANRDPSVTWONAME" then
									if SN < "MANRDPSVONENAME" then
										if SN < "MANFOURTHAGEWILL" then
											if SN == "MANEASILYINSPINHEALP" then
												return 5;
											else
												return 0;
											end
										elseif SN > "MANFOURTHAGEWILL" then
											if SN == "MANGIFTOFMENFATE" then
												return CalcStat("FateT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									elseif SN > "MANRDPSVONENAME" then
										if SN < "MANRDPSVTWOBLOCK" then
											if SN == "MANRDPSVONEWILL" then
												return CalcStat("ManFourthAgeWill",L);
											else
												return 0;
											end
										elseif SN > "MANRDPSVTWOBLOCK" then
											if SN == "MANRDPSVTWOEVADE" then
												return CalcStat("BalanceOfManEvade",L);
											else
												return 0;
											end
										else
											return CalcStat("BalanceOfManBlock",L);
										end
									else
										return "Man of the Fourth Age";
									end
								elseif SN > "MANRDPSVTWONAME" then
									if SN < "MANRDTRAITMIGHT" then
										if SN < "MANRDTRAITFATE" then
											if SN == "MANRDPSVTWOPARRY" then
												return CalcStat("BalanceOfManParry",L);
											else
												return 0;
											end
										elseif SN > "MANRDTRAITFATE" then
											if SN == "MANRDTRAITINHEALP" then
												return CalcStat("ManEasilyInspInHealP",L);
											else
												return 0;
											end
										else
											return CalcStat("ManGiftOfMenFate",L);
										end
									elseif SN > "MANRDTRAITMIGHT" then
										if SN < "MANSTRONGMENMIGHT" then
											if SN == "MANRDTRAITWILL" then
												return CalcStat("ManDimMankindWill",L);
											else
												return 0;
											end
										elseif SN > "MANSTRONGMENMIGHT" then
											if SN == "MARINERCDAGILITYTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("MightT",L,1.0);
										end
									else
										return CalcStat("ManStrongMenMight",L);
									end
								else
									return "Balance of Man";
								end
							elseif SN > "MARINERCDAGILITYTOOUTHEAL" then
								if SN < "MARINERCDARMOURTYPE" then
									if SN < "MARINERCDAGILITYTOTACMIT" then
										if SN < "MARINERCDAGILITYTOPHYMAS" then
											if SN == "MARINERCDAGILITYTOPARRY" then
												return 3;
											else
												return 0;
											end
										elseif SN > "MARINERCDAGILITYTOPHYMAS" then
											if SN == "MARINERCDAGILITYTOPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "MARINERCDAGILITYTOTACMIT" then
										if SN < "MARINERCDARMOURTONONPHYMIT" then
											if SN == "MARINERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDARMOURTONONPHYMIT" then
											if SN == "MARINERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "MARINERCDARMOURTYPE" then
									if SN < "MARINERCDBASEICPR" then
										if SN < "MARINERCDBASEFATE" then
											if SN == "MARINERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityM",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEFATE" then
											if SN == "MARINERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseFate",L);
										end
									elseif SN > "MARINERCDBASEICPR" then
										if SN < "MARINERCDBASEMORALE" then
											if SN == "MARINERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										elseif SN > "MARINERCDBASEMORALE" then
											if SN == "MARINERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMorale",L);
										end
									else
										return CalcStat("ClassBaseICPR",L);
									end
								else
									return 2;
								end
							else
								return 3;
							end
						else
							return -CalcStat("WillT",L,0.4);
						end
					elseif SN > "MARINERCDBASENCPR" then
						if SN < "MASTERYT" then
							if SN < "MARINERCDPHYMITTOCOMPHYMIT" then
								if SN < "MARINERCDFATETONCPR" then
									if SN < "MARINERCDBASEWILL" then
										if SN > "MARINERCDBASEPOWER" then
											if SN == "MARINERCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										elseif SN == "MARINERCDBASEPOWER" then
											return CalcStat("ClassBasePower",L);
										else
											return 0;
										end
									elseif SN > "MARINERCDBASEWILL" then
										if SN < "MARINERCDCALCTYPENONPHYMIT" then
											if SN == "MARINERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "MARINERCDCALCTYPENONPHYMIT" then
											if SN == "MARINERCDCALCTYPETACMIT" then
												return 26;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ClassBaseWillM",L);
									end
								elseif SN > "MARINERCDFATETONCPR" then
									if SN < "MARINERCDMIGHTTOFINESSE" then
										if SN < "MARINERCDHASPOWER" then
											if SN == "MARINERCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDHASPOWER" then
											if SN == "MARINERCDMIGHTTOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MARINERCDMIGHTTOFINESSE" then
										if SN < "MARINERCDMIGHTTOPARRY" then
											if SN == "MARINERCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDMIGHTTOPARRY" then
											if SN == "MARINERCDMIGHTTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1.5;
									end
								else
									return 0.07;
								end
							elseif SN > "MARINERCDPHYMITTOCOMPHYMIT" then
								if SN < "MARINERCDWILLTOOUTHEAL" then
									if SN < "MARINERCDVITALITYTOMORALE" then
										if SN < "MARINERCDTACMASTOOUTHEAL" then
											if SN == "MARINERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MARINERCDTACMASTOOUTHEAL" then
											if SN == "MARINERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MARINERCDVITALITYTOMORALE" then
										if SN < "MARINERCDWILLTOCRITHIT" then
											if SN == "MARINERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "MARINERCDWILLTOCRITHIT" then
											if SN == "MARINERCDWILLTOFINESSE" then
												return 1.5;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 4.5;
									end
								elseif SN > "MARINERCDWILLTOOUTHEAL" then
									if SN < "MASTERY" then
										if SN < "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTOPHYMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MARINERCDWILLTOPHYMIT" then
											if SN == "MARINERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MASTERY" then
										if SN < "MASTERYC" then
											if SN == "MASTERYADJ" then
												if Lm <= 499 then
													return 1;
												elseif Lm <= 500 then
													return 0.9;
												else
													return 247/300;
												end
											else
												return 0;
											end
										elseif SN > "MASTERYC" then
											if SN == "MASTERYOLD" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPMasteryC","CreepProg","ProgBMasteryC","",L,N,0);
										end
									else
										return StatLinInter("PntMPMastery","ItemProg","ProgBMastery","MasteryAdj",L,N,0);
									end
								else
									return 2;
								end
							else
								return 1;
							end
						elseif SN > "MASTERYT" then
							if SN < "MINSTRELCDAGILITYTOCRITHIT" then
								if SN < "MINCALLOROMEMELLIGHTMIT" then
									if SN < "MIGHTC" then
										if SN < "MATHOMLVLTOILVL" then
											if SN == "MASTERYTADJ" then
												if Lm <= 140 then
													return 1;
												elseif Lm <= 141 then
													return 0.9;
												else
													return 247/300;
												end
											else
												return 0;
											end
										elseif SN > "MATHOMLVLTOILVL" then
											if SN == "MIGHT" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										else
											return CalcStat("AwardLvlToILvl",L);
										end
									elseif SN > "MIGHTC" then
										if SN < "MINCALLEARENDILLIGHTMIT" then
											if SN == "MIGHTT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										elseif SN > "MINCALLEARENDILLIGHTMIT" then
											if SN == "MINCALLOROMELIGHTMIT" then
												return -CalcStat("LightMitT",L,1.2);
											else
												return 0;
											end
										else
											return -CalcStat("LightMitT",L,1.2);
										end
									else
										return CalcStat("MainC",L,N);
									end
								elseif SN > "MINCALLOROMEMELLIGHTMIT" then
									if SN < "MINECHOESBATTLECRITDEF" then
										if SN < "MINCOMPOSURETACMIT" then
											if SN == "MINCOMPOSURERESIST" then
												return CalcStat("ResistT",L,1.6);
											else
												return 0;
											end
										elseif SN > "MINCOMPOSURETACMIT" then
											if SN == "MINCOURAGERESIST" then
												return CalcStat("FearResistT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("TacMitT",L,1.0);
										end
									elseif SN > "MINECHOESBATTLECRITDEF" then
										if SN < "MINENDMORALE" then
											if SN == "MINECHOESBATTLERESIST" then
												return -CalcStat("SongResistT",L,1.0);
											else
												return 0;
											end
										elseif SN > "MINENDMORALE" then
											if SN == "MINPIERCINGBALFINESSE" then
												return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
											else
												return 0;
											end
										else
											return CalcStat("MoraleT",L,CalcStat("Trait12345Choice",N)*0.8);
										end
									else
										return -CalcStat("CritDefT",L,2.0);
									end
								else
									return -CalcStat("LightMitT",L,1.4);
								end
							elseif SN > "MINSTRELCDAGILITYTOCRITHIT" then
								if SN < "MINSTRELCDBASEAGILITY" then
									if SN < "MINSTRELCDARMOURTOCOMPHYMIT" then
										if SN < "MINSTRELCDAGILITYTOFINESSE" then
											if SN == "MINSTRELCDAGILITYTOEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDAGILITYTOFINESSE" then
											if SN == "MINSTRELCDAGILITYTOTACMAS" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MINSTRELCDARMOURTOCOMPHYMIT" then
										if SN < "MINSTRELCDARMOURTOTACMIT" then
											if SN == "MINSTRELCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDARMOURTOTACMIT" then
											if SN == "MINSTRELCDARMOURTYPE" then
												return 1;
											else
												return 0;
											end
										else
											return 0.2;
										end
									else
										return 1;
									end
								elseif SN > "MINSTRELCDBASEAGILITY" then
									if SN < "MINSTRELCDBASEMIGHT" then
										if SN < "MINSTRELCDBASEICMR" then
											if SN == "MINSTRELCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASEICMR" then
											if SN == "MINSTRELCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICMRL",L);
										end
									elseif SN > "MINSTRELCDBASEMIGHT" then
										if SN < "MINSTRELCDBASENCMR" then
											if SN == "MINSTRELCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										elseif SN > "MINSTRELCDBASENCMR" then
											if SN == "MINSTRELCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCMRL",L);
										end
									else
										return CalcStat("ClassBaseMightL",L);
									end
								else
									return CalcStat("ClassBaseAgilityM",L);
								end
							else
								return 2;
							end
						else
							return StatLinInter("PntMPMastery","TraitProg","ProgBMastery","MasteryTAdj",L,N,0);
						end
					else
						return CalcStat("ClassBaseNCPR",L);
					end
				else
					return 12;
				end
			else
				return 2;
			end
		else
			if Lm <= 0 then
				return 0;
			elseif Lm <= 10 then
				return LinFmod(1,1,1.2,1,10,L);
			elseif Lm <= 36 then
				return LinFmod(1,1.2,1.25,10,36,L);
			elseif Lm <= 41 then
				return LinFmod(1,1.25,1.3,36,41,L);
			elseif Lm <= 60 then
				return LinFmod(1,1.3,1.25,41,60,L);
			else
				return CalcStat("CreepAudacityDmgP",60);
			end
		end
	elseif SN > "MINSTRELCDBASEPOWER" then
		if SN < "TACMITLPRATPB" then
			if SN < "PNTMPDMGTYPEMIT" then
				if SN < "PARTBPEPRATPA" then
					if SN < "MITMEDIUMPRATPCAPR" then
						if SN < "MINTOTFATE" then
							if SN < "MINSTRELCDPHYMITTONONPHYMIT" then
								if SN < "MINSTRELCDFATETONCPR" then
									if SN < "MINSTRELCDCALCTYPECOMPHYMIT" then
										if SN > "MINSTRELCDBASEVITALITY" then
											if SN == "MINSTRELCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										elseif SN == "MINSTRELCDBASEVITALITY" then
											return CalcStat("ClassBaseVitality",L);
										else
											return 0;
										end
									elseif SN > "MINSTRELCDCALCTYPECOMPHYMIT" then
										if SN < "MINSTRELCDCALCTYPETACMIT" then
											if SN == "MINSTRELCDCALCTYPENONPHYMIT" then
												return 12;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDCALCTYPETACMIT" then
											if SN == "MINSTRELCDCANBLOCK" then
												if Lm <= 19 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										else
											return 25;
										end
									else
										return 12;
									end
								elseif SN > "MINSTRELCDFATETONCPR" then
									if SN < "MINSTRELCDMIGHTTOOUTHEAL" then
										if SN < "MINSTRELCDHASPOWER" then
											if SN == "MINSTRELCDFATETOPOWER" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDHASPOWER" then
											if SN == "MINSTRELCDMIGHTTOCRITHIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "MINSTRELCDMIGHTTOOUTHEAL" then
										if SN < "MINSTRELCDMIGHTTOTACMIT" then
											if SN == "MINSTRELCDMIGHTTOTACMAS" then
												return 2;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDMIGHTTOTACMIT" then
											if SN == "MINSTRELCDPHYMITTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 2;
									end
								else
									return 0.07;
								end
							elseif SN > "MINSTRELCDPHYMITTONONPHYMIT" then
								if SN < "MINSTRELCDWILLTOPHYMIT" then
									if SN < "MINSTRELCDVITALITYTONCMR" then
										if SN < "MINSTRELCDVITALITYTOICMR" then
											if SN == "MINSTRELCDTACMASTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDVITALITYTOICMR" then
											if SN == "MINSTRELCDVITALITYTOMORALE" then
												return 4.5;
											else
												return 0;
											end
										else
											return 0.012;
										end
									elseif SN > "MINSTRELCDVITALITYTONCMR" then
										if SN < "MINSTRELCDWILLTOCRITHIT" then
											if SN == "MINSTRELCDWILLTOBLOCK" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDWILLTOCRITHIT" then
											if SN == "MINSTRELCDWILLTOEVADE" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.12;
									end
								elseif SN > "MINSTRELCDWILLTOPHYMIT" then
									if SN < "MINTACMAS" then
										if SN < "MINSTRELCDWILLTOTACMAS" then
											if SN == "MINSTRELCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "MINSTRELCDWILLTOTACMAS" then
											if SN == "MINSTRELCDWILLTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									elseif SN > "MINTACMAS" then
										if SN < "MINTOTCRITHIT" then
											if SN == "MINTIMEECHOESBATTLERESIST" then
												return -CalcStat("SongResistT",L,0.6);
											else
												return 0;
											end
										elseif SN > "MINTOTCRITHIT" then
											if SN == "MINTOTCRITHITSEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0,0,0.4,0.6,0},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("CritHitT",L,CalcStat("MinToTCritHitSel",N));
										end
									else
										return CalcStat("TacMasT",L,CalcStat("Trait12345Choice",N)*0.4);
									end
								else
									return 1;
								end
							else
								return 1;
							end
						elseif SN > "MINTOTFATE" then
							if SN < "MITIGATION" then
								if SN < "MITHEAVYPBONUS" then
									if SN < "MINTOTRESIST" then
										if SN < "MINTOTFINESSE" then
											if SN == "MINTOTFATESEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.2,0.3,0.4,0.5,0.6,0},L);
												end
											else
												return 0;
											end
										elseif SN > "MINTOTFINESSE" then
											if SN == "MINTOTFINESSESEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0,0.2,0.4,0.6,0},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("MinToTFinesseSel",N));
										end
									elseif SN > "MINTOTRESIST" then
										if SN < "MINTOTVITALITY" then
											if SN == "MINTOTRESISTSEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0.2,0.3,0.4,0.5,0},L);
												end
											else
												return 0;
											end
										elseif SN > "MINTOTVITALITY" then
											if SN == "MINTOTVITALITYSEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0,0,0,0,0.4,0},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("VitalityT",L,CalcStat("MinToTVitalitySel",N));
										end
									else
										return CalcStat("ResistT",L,CalcStat("MinToTResistSel",N));
									end
								elseif SN > "MITHEAVYPBONUS" then
									if SN < "MITHEAVYPRATPB" then
										if SN < "MITHEAVYPRATP" then
											if SN == "MITHEAVYPPRAT" then
												return CalcRatAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "MITHEAVYPRATP" then
											if SN == "MITHEAVYPRATPA" then
												return 180;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("MitHeavyPRatPA",L),CalcStat("MitHeavyPRatPB",L),CalcStat("MitHeavyPRatPCap",L),N);
										end
									elseif SN > "MITHEAVYPRATPB" then
										if SN < "MITHEAVYPRATPCAP" then
											if SN == "MITHEAVYPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "MITHEAVYPRATPCAP" then
											if SN == "MITHEAVYPRATPCAPR" then
												return CalcStat("MitHeavyPRatPB",L)*CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										else
											return 60;
										end
									else
										return CalcStat("BRatRounded",L,"BRatMitHeavy");
									end
								else
									return 0;
								end
							elseif SN > "MITIGATION" then
								if SN < "MITLIGHTPRATPCAPR" then
									if SN < "MITLIGHTPRATPA" then
										if SN < "MITLIGHTPPRAT" then
											if SN == "MITLIGHTPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "MITLIGHTPPRAT" then
											if SN == "MITLIGHTPRATP" then
												return CalcPercAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("MitLightPRatPA",L),CalcStat("MitLightPRatPB",L),CalcStat("MitLightPRatPCapR",L),N);
										end
									elseif SN > "MITLIGHTPRATPA" then
										if SN < "MITLIGHTPRATPC" then
											if SN == "MITLIGHTPRATPB" then
												return CalcStat("BRatRounded",L,"BRatMitLight");
											else
												return 0;
											end
										elseif SN > "MITLIGHTPRATPC" then
											if SN == "MITLIGHTPRATPCAP" then
												return 40;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 120;
									end
								elseif SN > "MITLIGHTPRATPCAPR" then
									if SN < "MITMEDIUMPRATPA" then
										if SN < "MITMEDIUMPPRAT" then
											if SN == "MITMEDIUMPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "MITMEDIUMPPRAT" then
											if SN == "MITMEDIUMPRATP" then
												return CalcPercAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("MitMediumPRatPA",L),CalcStat("MitMediumPRatPB",L),CalcStat("MitMediumPRatPCapR",L),N);
										end
									elseif SN > "MITMEDIUMPRATPA" then
										if SN < "MITMEDIUMPRATPC" then
											if SN == "MITMEDIUMPRATPB" then
												return CalcStat("BRatRounded",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "MITMEDIUMPRATPC" then
											if SN == "MITMEDIUMPRATPCAP" then
												return 50;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 150;
									end
								else
									return CalcStat("MitLightPRatPB",L)*CalcStat("MitLightPRatPC",L);
								end
							else
								return StatLinInter("PntMPMitigation","ItemProg","ProgBMitigation","AdjUmbarItemMit",L,N,0);
							end
						else
							return CalcStat("FateT",L,CalcStat("MinToTFateSel",N));
						end
					elseif SN > "MITMEDIUMPRATPCAPR" then
						if SN < "OUTHEALPRATPCAPR" then
							if SN < "OUTDMGPBONUS" then
								if SN < "NCMRC" then
									if SN < "MORALET" then
										if SN > "MORALE" then
											if SN == "MORALEADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 79 then
													return 0.8;
												elseif Lm <= 80 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN == "MORALE" then
											return StatLinInter("PntMPMorale","ItemProgVital","ProgBMorale","MoraleAdj",L,N,0);
										else
											return 0;
										end
									elseif SN > "MORALET" then
										if SN < "N" then
											if SN == "MORALETADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 65 then
													return 0.8;
												elseif Lm <= 75 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "N" then
											if SN == "NCMR" then
												return StatLinInter("PntMPNCMR","ItemProgVital","ProgBNCMR","",L,N,1);
											else
												return 0;
											end
										else
											return N;
										end
									else
										return StatLinInter("PntMPMorale","TraitProgVital","ProgBMorale","MoraleTAdj",L,N,0);
									end
								elseif SN > "NCMRC" then
									if SN < "NCPRC" then
										if SN < "NCMRT" then
											if SN == "NCMRCRAW" then
												return StatLinInter("PntMPNCMRC","CreepProg","ProgBNCMRC","",L,N,99);
											else
												return 0;
											end
										elseif SN > "NCMRT" then
											if SN == "NCPR" then
												return StatLinInter("PntMPNCPR","ItemProgClassic","ProgBNCPR","",L,N,99);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPNCMR","TraitProgVital","ProgBNCMR","",L,N,1);
										end
									elseif SN > "NCPRC" then
										if SN < "NCPRT" then
											if SN == "NCPRCRAW" then
												return StatLinInter("PntMPNCPRC","CreepProg","ProgBNCPRC","",L,N,99);
											else
												return 0;
											end
										elseif SN > "NCPRT" then
											if SN == "OFFSET" then
												return L+N;
											else
												return 0;
											end
										else
											return StatLinInter("PntMPNCPR","TraitProgVital","ProgBNCPR","",L,N,99);
										end
									else
										return StatLinInter("PntMPNCPRC","CreepProg","ProgBNCPRC","",L,N,0);
									end
								else
									return StatLinInter("PntMPNCMRC","CreepProg","ProgBNCMRC","",L,N,0);
								end
							elseif SN > "OUTDMGPBONUS" then
								if SN < "OUTHEAL" then
									if SN < "OUTDMGPRATPB" then
										if SN < "OUTDMGPRATP" then
											if SN == "OUTDMGPPRAT" then
												return CalcRatAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCapR",L),N);
											else
												return 0;
											end
										elseif SN > "OUTDMGPRATP" then
											if SN == "OUTDMGPRATPA" then
												return 600;
											else
												return 0;
											end
										else
											return CalcPercAB(CalcStat("OutDmgPRatPA",L),CalcStat("OutDmgPRatPB",L),CalcStat("OutDmgPRatPCap",L),N);
										end
									elseif SN > "OUTDMGPRATPB" then
										if SN < "OUTDMGPRATPCAP" then
											if SN == "OUTDMGPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "OUTDMGPRATPCAP" then
											if SN == "OUTDMGPRATPCAPR" then
												return CalcStat("OutDmgPRatPB",L)*CalcStat("OutDmgPRatPC",L);
											else
												return 0;
											end
										else
											return 200;
										end
									else
										return CalcStat("BRatRounded",L,"BRatExtra");
									end
								elseif SN > "OUTHEAL" then
									if SN < "OUTHEALPRATPA" then
										if SN < "OUTHEALPPRAT" then
											if SN == "OUTHEALPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "OUTHEALPPRAT" then
											if SN == "OUTHEALPRATP" then
												return CalcPercAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("OutHealPRatPA",L),CalcStat("OutHealPRatPB",L),CalcStat("OutHealPRatPCapR",L),N);
										end
									elseif SN > "OUTHEALPRATPA" then
										if SN < "OUTHEALPRATPC" then
											if SN == "OUTHEALPRATPB" then
												return CalcStat("BRatRounded",L,"BRatOutHeal");
											else
												return 0;
											end
										elseif SN > "OUTHEALPRATPC" then
											if SN == "OUTHEALPRATPCAP" then
												return 70;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 210;
									end
								else
									return StatLinInter("PntMPOutHeal","ItemProg","ProgBOutHeal","AdjUmbarItem",L,N,0);
								end
							else
								return 0;
							end
						elseif SN > "OUTHEALPRATPCAPR" then
							if SN < "PARTBLOCKMITPRATPA" then
								if SN < "PARRYPRATPB" then
									if SN < "PARRYPBONUS" then
										if SN < "PARRY" then
											if SN == "OUTHEALT" then
												return StatLinInter("PntMPOutHeal","TraitProg","ProgBOutHeal","AdjUmbarTrait",L,N,0);
											else
												return 0;
											end
										elseif SN > "PARRY" then
											if SN == "PARRYC" then
												return CalcStat("BPEC",L,N);
											else
												return 0;
											end
										else
											return CalcStat("BPE",L,N);
										end
									elseif SN > "PARRYPBONUS" then
										if SN < "PARRYPRATP" then
											if SN == "PARRYPPRAT" then
												return CalcStat("BPEPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "PARRYPRATP" then
											if SN == "PARRYPRATPA" then
												return CalcStat("BPEPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatP",L,N);
										end
									else
										return CalcStat("BPEPBonus",L);
									end
								elseif SN > "PARRYPRATPB" then
									if SN < "PARRYT" then
										if SN < "PARRYPRATPCAP" then
											if SN == "PARRYPRATPC" then
												return CalcStat("BPEPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARRYPRATPCAP" then
											if SN == "PARRYPRATPCAPR" then
												return CalcStat("BPEPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("BPEPRatPCap",L);
										end
									elseif SN > "PARRYT" then
										if SN < "PARTBLOCKMITPPRAT" then
											if SN == "PARTBLOCKMITPBONUS" then
												return CalcStat("PartMitPBonus",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKMITPPRAT" then
											if SN == "PARTBLOCKMITPRATP" then
												return CalcStat("PartMitPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPPRat",L,N);
										end
									else
										return CalcStat("BPET",L,N);
									end
								else
									return CalcStat("BPEPRatPB",L);
								end
							elseif SN > "PARTBLOCKMITPRATPA" then
								if SN < "PARTBLOCKPRATPA" then
									if SN < "PARTBLOCKMITPRATPCAPR" then
										if SN < "PARTBLOCKMITPRATPC" then
											if SN == "PARTBLOCKMITPRATPB" then
												return CalcStat("PartMitPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKMITPRATPC" then
											if SN == "PARTBLOCKMITPRATPCAP" then
												return CalcStat("PartMitPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPC",L);
										end
									elseif SN > "PARTBLOCKMITPRATPCAPR" then
										if SN < "PARTBLOCKPPRAT" then
											if SN == "PARTBLOCKPBONUS" then
												return CalcStat("PartBPEPBonus",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPPRAT" then
											if SN == "PARTBLOCKPRATP" then
												return CalcStat("PartBPEPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPPRat",L,N);
										end
									else
										return CalcStat("PartMitPRatPCapR",L);
									end
								elseif SN > "PARTBLOCKPRATPA" then
									if SN < "PARTBLOCKPRATPCAPR" then
										if SN < "PARTBLOCKPRATPC" then
											if SN == "PARTBLOCKPRATPB" then
												return CalcStat("PartBPEPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PARTBLOCKPRATPC" then
											if SN == "PARTBLOCKPRATPCAP" then
												return CalcStat("PartBPEPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPC",L);
										end
									elseif SN > "PARTBLOCKPRATPCAPR" then
										if SN < "PARTBPEPPRAT" then
											if SN == "PARTBPEPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "PARTBPEPPRAT" then
											if SN == "PARTBPEPRATP" then
												return CalcPercAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("PartBPEPRatPA",L),CalcStat("PartBPEPRatPB",L),CalcStat("PartBPEPRatPCapR",L),N);
										end
									else
										return CalcStat("PartBPEPRatPCapR",L);
									end
								else
									return CalcStat("PartBPEPRatPA",L);
								end
							else
								return CalcStat("PartMitPRatPA",L);
							end
						else
							return CalcStat("OutHealPRatPB",L)*CalcStat("OutHealPRatPC",L);
						end
					else
						return CalcStat("MitMediumPRatPB",L)*CalcStat("MitMediumPRatPC",L);
					end
				elseif SN > "PARTBPEPRATPA" then
					if SN < "PERKNCPR" then
						if SN < "PARTFINESSEPRATP" then
							if SN < "PARTEVADEPRATP" then
								if SN < "PARTEVADEMITPRATP" then
									if SN < "PARTBPEPRATPCAP" then
										if SN > "PARTBPEPRATPB" then
											if SN == "PARTBPEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										elseif SN == "PARTBPEPRATPB" then
											return CalcStat("BRatRounded",L,"BRatPartBPE");
										else
											return 0;
										end
									elseif SN > "PARTBPEPRATPCAP" then
										if SN < "PARTEVADEMITPBONUS" then
											if SN == "PARTBPEPRATPCAPR" then
												return CalcStat("PartBPEPRatPB",L)*CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEMITPBONUS" then
											if SN == "PARTEVADEMITPPRAT" then
												return CalcStat("PartMitPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPBonus",L);
										end
									else
										return 25;
									end
								elseif SN > "PARTEVADEMITPRATP" then
									if SN < "PARTEVADEMITPRATPCAP" then
										if SN < "PARTEVADEMITPRATPB" then
											if SN == "PARTEVADEMITPRATPA" then
												return CalcStat("PartMitPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEMITPRATPB" then
											if SN == "PARTEVADEMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPB",L);
										end
									elseif SN > "PARTEVADEMITPRATPCAP" then
										if SN < "PARTEVADEPBONUS" then
											if SN == "PARTEVADEMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEPBONUS" then
											if SN == "PARTEVADEPPRAT" then
												return CalcStat("PartBPEPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPBonus",L);
										end
									else
										return CalcStat("PartMitPRatPCap",L);
									end
								else
									return CalcStat("PartMitPRatP",L,N);
								end
							elseif SN > "PARTEVADEPRATP" then
								if SN < "PARTFINESSEDMGPRATP" then
									if SN < "PARTEVADEPRATPCAP" then
										if SN < "PARTEVADEPRATPB" then
											if SN == "PARTEVADEPRATPA" then
												return CalcStat("PartBPEPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTEVADEPRATPB" then
											if SN == "PARTEVADEPRATPC" then
												return CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPB",L);
										end
									elseif SN > "PARTEVADEPRATPCAP" then
										if SN < "PARTFINESSEDMGPBONUS" then
											if SN == "PARTEVADEPRATPCAPR" then
												return CalcStat("PartBPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTFINESSEDMGPBONUS" then
											if SN == "PARTFINESSEDMGPPRAT" then
												return CalcRatAB(CalcStat("PartFinesseDmgPRatPA",L),CalcStat("PartFinesseDmgPRatPB",L),CalcStat("PartFinesseDmgPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return CalcStat("PartBPEPRatPCap",L);
									end
								elseif SN > "PARTFINESSEDMGPRATP" then
									if SN < "PARTFINESSEDMGPRATPCAP" then
										if SN < "PARTFINESSEDMGPRATPB" then
											if SN == "PARTFINESSEDMGPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN > "PARTFINESSEDMGPRATPB" then
											if SN == "PARTFINESSEDMGPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									elseif SN > "PARTFINESSEDMGPRATPCAP" then
										if SN < "PARTFINESSEPBONUS" then
											if SN == "PARTFINESSEDMGPRATPCAPR" then
												return CalcStat("PartFinesseDmgPRatPB",L)*CalcStat("PartFinesseDmgPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTFINESSEPBONUS" then
											if SN == "PARTFINESSEPPRAT" then
												return CalcRatAB(CalcStat("PartFinessePRatPA",L),CalcStat("PartFinessePRatPB",L),CalcStat("PartFinessePRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0;
										end
									else
										return 50;
									end
								else
									return CalcPercAB(CalcStat("PartFinesseDmgPRatPA",L),CalcStat("PartFinesseDmgPRatPB",L),CalcStat("PartFinesseDmgPRatPCap",L),N);
								end
							else
								return CalcStat("PartBPEPRatP",L,N);
							end
						elseif SN > "PARTFINESSEPRATP" then
							if SN < "PARTPARRYMITPRATP" then
								if SN < "PARTMITPRATP" then
									if SN < "PARTFINESSEPRATPCAP" then
										if SN < "PARTFINESSEPRATPB" then
											if SN == "PARTFINESSEPRATPA" then
												return 150;
											else
												return 0;
											end
										elseif SN > "PARTFINESSEPRATPB" then
											if SN == "PARTFINESSEPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatStandard");
										end
									elseif SN > "PARTFINESSEPRATPCAP" then
										if SN < "PARTMITPBONUS" then
											if SN == "PARTFINESSEPRATPCAPR" then
												return CalcStat("PartFinessePRatPB",L)*CalcStat("PartFinessePRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTMITPBONUS" then
											if SN == "PARTMITPPRAT" then
												return CalcRatAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCapR",L),N);
											else
												return 0;
											end
										else
											return 0.1;
										end
									else
										return 50;
									end
								elseif SN > "PARTMITPRATP" then
									if SN < "PARTMITPRATPCAP" then
										if SN < "PARTMITPRATPB" then
											if SN == "PARTMITPRATPA" then
												return 105;
											else
												return 0;
											end
										elseif SN > "PARTMITPRATPB" then
											if SN == "PARTMITPRATPC" then
												return 0.5;
											else
												return 0;
											end
										else
											return CalcStat("BRatRounded",L,"BRatPartBPE");
										end
									elseif SN > "PARTMITPRATPCAP" then
										if SN < "PARTPARRYMITPBONUS" then
											if SN == "PARTMITPRATPCAPR" then
												return CalcStat("PartMitPRatPB",L)*CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYMITPBONUS" then
											if SN == "PARTPARRYMITPPRAT" then
												return CalcStat("PartMitPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPBonus",L);
										end
									else
										return 35;
									end
								else
									return CalcPercAB(CalcStat("PartMitPRatPA",L),CalcStat("PartMitPRatPB",L),CalcStat("PartMitPRatPCap",L),N);
								end
							elseif SN > "PARTPARRYMITPRATP" then
								if SN < "PARTPARRYPRATP" then
									if SN < "PARTPARRYMITPRATPCAP" then
										if SN < "PARTPARRYMITPRATPB" then
											if SN == "PARTPARRYMITPRATPA" then
												return CalcStat("PartMitPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYMITPRATPB" then
											if SN == "PARTPARRYMITPRATPC" then
												return CalcStat("PartMitPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartMitPRatPB",L);
										end
									elseif SN > "PARTPARRYMITPRATPCAP" then
										if SN < "PARTPARRYPBONUS" then
											if SN == "PARTPARRYMITPRATPCAPR" then
												return CalcStat("PartMitPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYPBONUS" then
											if SN == "PARTPARRYPPRAT" then
												return CalcStat("PartBPEPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPBonus",L);
										end
									else
										return CalcStat("PartMitPRatPCap",L);
									end
								elseif SN > "PARTPARRYPRATP" then
									if SN < "PARTPARRYPRATPCAP" then
										if SN < "PARTPARRYPRATPB" then
											if SN == "PARTPARRYPRATPA" then
												return CalcStat("PartBPEPRatPA",L);
											else
												return 0;
											end
										elseif SN > "PARTPARRYPRATPB" then
											if SN == "PARTPARRYPRATPC" then
												return CalcStat("PartBPEPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("PartBPEPRatPB",L);
										end
									elseif SN > "PARTPARRYPRATPCAP" then
										if SN < "PERKMORALE" then
											if SN == "PARTPARRYPRATPCAPR" then
												return CalcStat("PartBPEPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "PERKMORALE" then
											if SN == "PERKNCMR" then
												return CalcStat("FoodNCMRL",L);
											else
												return 0;
											end
										else
											return DataTableValue({10,20,30,40},L);
										end
									else
										return CalcStat("PartBPEPRatPCap",L);
									end
								else
									return CalcStat("PartBPEPRatP",L,N);
								end
							else
								return CalcStat("PartMitPRatP",L,N);
							end
						else
							return CalcPercAB(CalcStat("PartFinessePRatPA",L),CalcStat("PartFinessePRatPB",L),CalcStat("PartFinessePRatPCap",L),N);
						end
					elseif SN > "PERKNCPR" then
						if SN < "PHYMITLPRATPC" then
							if SN < "PHYMAST" then
								if SN < "PHYDMGPRATPA" then
									if SN < "PERKTACMIT" then
										if SN > "PERKPHYMIT" then
											if SN == "PERKPOWER" then
												return DataTableValue({10,20,30,40},L);
											else
												return 0;
											end
										elseif SN == "PERKPHYMIT" then
											return CalcStat("PhyMitT",L,0.2*N);
										else
											return 0;
										end
									elseif SN > "PERKTACMIT" then
										if SN < "PHYDMGPPRAT" then
											if SN == "PHYDMGPBONUS" then
												return CalcStat("OutDmgPBonus",L);
											else
												return 0;
											end
										elseif SN > "PHYDMGPPRAT" then
											if SN == "PHYDMGPRATP" then
												return CalcStat("OutDmgPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPPRat",L,N);
										end
									else
										return CalcStat("TacMitT",L,0.2*N);
									end
								elseif SN > "PHYDMGPRATPA" then
									if SN < "PHYDMGPRATPCAPR" then
										if SN < "PHYDMGPRATPC" then
											if SN == "PHYDMGPRATPB" then
												return CalcStat("OutDmgPRatPB",L);
											else
												return 0;
											end
										elseif SN > "PHYDMGPRATPC" then
											if SN == "PHYDMGPRATPCAP" then
												return CalcStat("OutDmgPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatPC",L);
										end
									elseif SN > "PHYDMGPRATPCAPR" then
										if SN < "PHYMASC" then
											if SN == "PHYMAS" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMASC" then
											if SN == "PHYMASOLD" then
												return CalcStat("Mastery",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MasteryC",L,N);
										end
									else
										return CalcStat("OutDmgPRatPCapR",L);
									end
								else
									return CalcStat("OutDmgPRatPA",L);
								end
							elseif SN > "PHYMAST" then
								if SN < "PHYMITHPRATPC" then
									if SN < "PHYMITHPPRAT" then
										if SN < "PHYMITC" then
											if SN == "PHYMIT" then
												return StatLinInter("PntMPPhyMit","ItemProg","ProgBMitigation","AdjUmbarItemMit",L,N,0);
											else
												return 0;
											end
										elseif SN > "PHYMITC" then
											if SN == "PHYMITHPBONUS" then
												return CalcStat("MitHeavyPBonus",L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPPhyMitC","CreepProg","ProgBMitigationC","",L,N,0);
										end
									elseif SN > "PHYMITHPPRAT" then
										if SN < "PHYMITHPRATPA" then
											if SN == "PHYMITHPRATP" then
												return CalcStat("MitHeavyPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITHPRATPA" then
											if SN == "PHYMITHPRATPB" then
												return CalcStat("MitHeavyPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPA",L);
										end
									else
										return CalcStat("MitHeavyPPRat",L,N);
									end
								elseif SN > "PHYMITHPRATPC" then
									if SN < "PHYMITLPPRAT" then
										if SN < "PHYMITHPRATPCAPR" then
											if SN == "PHYMITHPRATPCAP" then
												return CalcStat("MitHeavyPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PHYMITHPRATPCAPR" then
											if SN == "PHYMITLPBONUS" then
												return CalcStat("MitLightPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPCapR",L);
										end
									elseif SN > "PHYMITLPPRAT" then
										if SN < "PHYMITLPRATPA" then
											if SN == "PHYMITLPRATP" then
												return CalcStat("MitLightPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITLPRATPA" then
											if SN == "PHYMITLPRATPB" then
												return CalcStat("MitLightPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatPA",L);
										end
									else
										return CalcStat("MitLightPPRat",L,N);
									end
								else
									return CalcStat("MitHeavyPRatPC",L);
								end
							else
								return CalcStat("MasteryT",L,N);
							end
						elseif SN > "PHYMITLPRATPC" then
							if SN < "PLAYERBASEPHYMAS" then
								if SN < "PHYMITMPRATPC" then
									if SN < "PHYMITMPPRAT" then
										if SN < "PHYMITLPRATPCAPR" then
											if SN == "PHYMITLPRATPCAP" then
												return CalcStat("MitLightPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PHYMITLPRATPCAPR" then
											if SN == "PHYMITMPBONUS" then
												return CalcStat("MitMediumPBonus",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatPCapR",L);
										end
									elseif SN > "PHYMITMPPRAT" then
										if SN < "PHYMITMPRATPA" then
											if SN == "PHYMITMPRATP" then
												return CalcStat("MitMediumPRatP",L,N);
											else
												return 0;
											end
										elseif SN > "PHYMITMPRATPA" then
											if SN == "PHYMITMPRATPB" then
												return CalcStat("MitMediumPRatPB",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPA",L);
										end
									else
										return CalcStat("MitMediumPPRat",L,N);
									end
								elseif SN > "PHYMITMPRATPC" then
									if SN < "PHYRESISTT" then
										if SN < "PHYMITMPRATPCAPR" then
											if SN == "PHYMITMPRATPCAP" then
												return CalcStat("MitMediumPRatPCap",L);
											else
												return 0;
											end
										elseif SN > "PHYMITMPRATPCAPR" then
											if SN == "PHYMITT" then
												return StatLinInter("PntMPPhyMit","TraitProg","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPCapR",L);
										end
									elseif SN > "PHYRESISTT" then
										if SN < "PLAYERBASEMORALE" then
											if SN == "PLAYERBASEEVADE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "PLAYERBASEMORALE" then
											if SN == "PLAYERBASEPARRY" then
												return 3;
											else
												return 0;
											end
										else
											return 52.5;
										end
									else
										return CalcStat("ResistAddT",L,N);
									end
								else
									return CalcStat("MitMediumPRatPC",L);
								end
							elseif SN > "PLAYERBASEPHYMAS" then
								if SN < "PNTMPBPEC" then
									if SN < "PNTMPARMOURPENT" then
										if SN < "PNTMPARMOUR" then
											if SN == "PLAYERBASETACMAS" then
												return 11.5;
											else
												return 0;
											end
										elseif SN > "PNTMPARMOUR" then
											if SN == "PNTMPARMOURC" then
												return 15840/6;
											else
												return 0;
											end
										else
											return 4.4/1200;
										end
									elseif SN > "PNTMPARMOURPENT" then
										if SN < "PNTMPARMOURVIRTUES" then
											if SN == "PNTMPARMOURT" then
												return 25/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPARMOURVIRTUES" then
											if SN == "PNTMPBPE" then
												return 42/1200;
											else
												return 0;
											end
										else
											return 24/1200;
										end
									else
										return 72/1200;
									end
								elseif SN > "PNTMPBPEC" then
									if SN < "PNTMPCRITDEF" then
										if SN < "PNTMPCLASSBASENCPR" then
											if SN == "PNTMPCLASSBASEICPR" then
												return 0.125;
											else
												return 0;
											end
										elseif SN > "PNTMPCLASSBASENCPR" then
											if SN == "PNTMPCOMBATDAMAGEMOD" then
												return 300/1200;
											else
												return 0;
											end
										else
											return 0.25;
										end
									elseif SN > "PNTMPCRITDEF" then
										if SN < "PNTMPCRITHIT" then
											if SN == "PNTMPCRITDEFC" then
												return 19070/6;
											else
												return 0;
											end
										elseif SN > "PNTMPCRITHIT" then
											if SN == "PNTMPCRITHITC" then
												return 28560/6;
											else
												return 0;
											end
										else
											return 20/1200;
										end
									else
										return 40/1200;
									end
								else
									return 19070/6;
								end
							else
								return 11.5;
							end
						else
							return CalcStat("MitLightPRatPC",L);
						end
					else
						return CalcStat("FoodNCPRL",L);
					end
				else
					return 75;
				end
			elseif SN > "PNTMPDMGTYPEMIT" then
				if SN < "RIVERHOBBITRDPSVONENAME" then
					if SN < "PROGBMAIN" then
						if SN < "PNTMPTACHPS" then
							if SN < "PNTMPMASTERYC" then
								if SN < "PNTMPICMR" then
									if SN < "PNTMPFINESSE" then
										if SN > "PNTMPFATE" then
											if SN == "PNTMPFATEC" then
												return 1683/6;
											else
												return 0;
											end
										elseif SN == "PNTMPFATE" then
											return 1.25;
										else
											return 0;
										end
									elseif SN > "PNTMPFINESSE" then
										if SN < "PNTMPFINESSET" then
											if SN == "PNTMPFINESSEC" then
												return 19070/6;
											else
												return 0;
											end
										elseif SN > "PNTMPFINESSET" then
											if SN == "PNTMPFOODRESIST" then
												return 30/1200;
											else
												return 0;
											end
										else
											return 36/1200;
										end
									else
										return 40/1200;
									end
								elseif SN > "PNTMPICMR" then
									if SN < "PNTMPINHEAL" then
										if SN < "PNTMPICPR" then
											if SN == "PNTMPICMRC" then
												return 41.2;
											else
												return 0;
											end
										elseif SN > "PNTMPICPR" then
											if SN == "PNTMPICPRC" then
												return 4.75;
											else
												return 0;
											end
										else
											return 0.0625;
										end
									elseif SN > "PNTMPINHEAL" then
										if SN < "PNTMPMAINC" then
											if SN == "PNTMPMAIN" then
												return 0.5;
											else
												return 0;
											end
										elseif SN > "PNTMPMAINC" then
											if SN == "PNTMPMASTERY" then
												return 17/1200;
											else
												return 0;
											end
										else
											return 1683/6;
										end
									else
										return 40/1200;
									end
								else
									return 0.015;
								end
							elseif SN > "PNTMPMASTERYC" then
								if SN < "PNTMPOUTHEAL" then
									if SN < "PNTMPNCMR" then
										if SN < "PNTMPMORALE" then
											if SN == "PNTMPMITIGATION" then
												return 28/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPMORALE" then
											if SN == "PNTMPMORALEVIRTUES" then
												return 0.75;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "PNTMPNCMR" then
										if SN < "PNTMPNCPR" then
											if SN == "PNTMPNCMRC" then
												return 164.8;
											else
												return 0;
											end
										elseif SN > "PNTMPNCPR" then
											if SN == "PNTMPNCPRC" then
												return 19;
											else
												return 0;
											end
										else
											return 0.5;
										end
									else
										return 0.15;
									end
								elseif SN > "PNTMPOUTHEAL" then
									if SN < "PNTMPPOWERT" then
										if SN < "PNTMPPHYMITC" then
											if SN == "PNTMPPHYMIT" then
												return 36/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPPHYMITC" then
											if SN == "PNTMPPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 15840/6;
										end
									elseif SN > "PNTMPPOWERT" then
										if SN < "PNTMPRESISTC" then
											if SN == "PNTMPRESIST" then
												return 36/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPRESISTC" then
											if SN == "PNTMPSHIELDBLOCK" then
												return 80/1200;
											else
												return 0;
											end
										else
											return 28560/6;
										end
									else
										return 0.6666;
									end
								else
									return 30/1200;
								end
							else
								return 28560/6;
							end
						elseif SN > "PNTMPTACHPS" then
							if SN < "PROGBCOMBATDAMAGEMOD" then
								if SN < "POWERT" then
									if SN < "PNTMPVITALITYC" then
										if SN < "PNTMPTACMITC" then
											if SN == "PNTMPTACMIT" then
												return 36/1200;
											else
												return 0;
											end
										elseif SN > "PNTMPTACMITC" then
											if SN == "PNTMPVITALITY" then
												return 0.175;
											else
												return 0;
											end
										else
											return 15840/6;
										end
									elseif SN > "PNTMPVITALITYC" then
										if SN < "POISONRESISTT" then
											if SN == "PNTMPVITALITYT" then
												return 0.225;
											else
												return 0;
											end
										elseif SN > "POISONRESISTT" then
											if SN == "POWER" then
												return StatLinInter("PntMPPower","ItemProgVital","ProgBPower","",L,N,0);
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										return 4190/6;
									end
								elseif SN > "POWERT" then
									if SN < "PROGBARMOURLIGHT" then
										if SN < "PROGBARMOURC" then
											if SN == "PROGBARMOUR" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "PROGBARMOURC" then
											if SN == "PROGBARMOURHEAVY" then
												return CalcStat("BRatProgB",L,"BRatMitHeavy");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									elseif SN > "PROGBARMOURLIGHT" then
										if SN < "PROGBBPE" then
											if SN == "PROGBARMOURMEDIUM" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "PROGBBPE" then
											if SN == "PROGBBPEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatStandard");
										end
									else
										return CalcStat("BRatProgB",L,"BRatMitLight");
									end
								else
									return StatLinInter("PntMPPowerT","TraitProgVital","ProgBPower","",L,N,0);
								end
							elseif SN > "PROGBCOMBATDAMAGEMOD" then
								if SN < "PROGBFINESSE" then
									if SN < "PROGBCRITHITC" then
										if SN < "PROGBCRITDEFC" then
											if SN == "PROGBCRITDEF" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										elseif SN > "PROGBCRITDEFC" then
											if SN == "PROGBCRITHIT" then
												return CalcStat("BRatProgB",L,"BRatExtra");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									elseif SN > "PROGBCRITHITC" then
										if SN < "PROGBFATE" then
											if SN == "PROGBENERGY" then
												if Lm <= 50 then
													return LinFmod(1,4,8,1,50,L);
												elseif Lm <= 60 then
													return LinFmod(1,8,10.6,50,60,L);
												elseif Lm <= 65 then
													return LinFmod(1,10.6,13.2,60,65,L);
												elseif Lm <= 75 then
													return LinFmod(1,13.2,19.8,65,75,L);
												elseif Lm <= 85 then
													return LinFmod(1,19.8,29.8,75,85,L);
												elseif Lm <= 95 then
													return LinFmod(1,29.8,39.6,85,95,L);
												elseif Lm <= 100 then
													return LinFmod(1,39.6,52,95,100,L);
												elseif Lm <= 105 then
													return LinFmod(1,52,70,100,105,L);
												elseif Lm <= 115 then
													return LinFmod(1,78,106,106,115,L);
												elseif Lm <= 120 then
													return LinFmod(1,122,132,116,120,L);
												elseif Lm <= 130 then
													return LinFmod(1,152,198,121,130,L);
												elseif Lm <= 140 then
													return LinFmod(1,228,396,131,140,L);
												elseif Lm <= 150 then
													return LinFmod(1,456,800,141,150,L);
												else
													return LinFmod(1,800,1600,151,160,L);
												end
											else
												return 0;
											end
										elseif SN > "PROGBFATE" then
											if SN == "PROGBFATEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									else
										return CalcStat("CreepProgB",L);
									end
								elseif SN > "PROGBFINESSE" then
									if SN < "PROGBICMRC" then
										if SN < "PROGBHEALTH" then
											if SN == "PROGBFINESSEC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										elseif SN > "PROGBHEALTH" then
											if SN == "PROGBICMR" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,8,60,1,50,L);
											elseif Lm <= 60 then
												return LinFmod(1,60,80,50,60,L);
											elseif Lm <= 65 then
												return LinFmod(1,80,100,60,65,L);
											elseif Lm <= 75 then
												return LinFmod(1,100,150,65,75,L);
											elseif Lm <= 85 then
												return LinFmod(1,150,226,75,85,L);
											elseif Lm <= 95 then
												return LinFmod(1,226,300,85,95,L);
											elseif Lm <= 100 then
												return LinFmod(1,300,450,95,100,L);
											elseif Lm <= 105 then
												return LinFmod(1,450,600,100,105,L);
											elseif Lm <= 115 then
												return LinFmod(1,660,900,106,115,L);
											elseif Lm <= 120 then
												return LinFmod(1,1040,1120,116,120,L);
											elseif Lm <= 130 then
												return LinFmod(1,1280,1680,121,130,L);
											elseif Lm <= 140 then
												return LinFmod(1,1940,3360,131,140,L);
											elseif Lm <= 150 then
												return LinFmod(1,3860,6800,141,150,L);
											else
												return LinFmod(1,6800,13600,151,160,L);
											end
										end
									elseif SN > "PROGBICMRC" then
										if SN < "PROGBICPRC" then
											if SN == "PROGBICPR" then
												return CalcStat("ProgBEnergy",L);
											else
												return 0;
											end
										elseif SN > "PROGBICPRC" then
											if SN == "PROGBINHEAL" then
												return CalcStat("BRatProgB",L,"BRatStandard");
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									else
										return CalcStat("CreepProgB",L);
									end
								else
									return CalcStat("BRatProgB",L,"BRatStandard");
								end
							else
								if Lm <= 25 then
									return LinFmod(1,5,45.58309,1,25,L);
								elseif Lm <= 50 then
									return LinFmod(1,45.58309,120,25,50,L);
								elseif Lm <= 60 then
									return LinFmod(1,120,175.56,50,60,L);
								elseif Lm <= 65 then
									return LinFmod(1,175.56,200.79,60,65,L);
								elseif Lm <= 75 then
									return LinFmod(1,200.79,292.5,65,75,L);
								elseif Lm <= 85 then
									return LinFmod(1,292.5,421.2,75,85,L);
								elseif Lm <= 95 then
									return LinFmod(1,421.2,600.3,85,95,L);
								elseif Lm <= 100 then
									return LinFmod(1,600.3,774,95,100,L);
								elseif Lm <= 105 then
									return LinFmod(1,774,985.5,100,105,L);
								elseif Lm <= 115 then
									return LinFmod(1,985.5,1350,105,115,L);
								elseif Lm <= 120 then
									return LinFmod(1,1350,1710,115,120,L);
								elseif Lm <= 130 then
									return LinFmod(1,1710,2330.25,120,130,L);
								elseif Lm <= 140 then
									return LinFmod(1,2330.25,3600,130,140,L);
								else
									return LinFmod(1,4010,5535,141,150,L);
								end
							end
						else
							return 0.0875;
						end
					elseif SN > "PROGBMAIN" then
						if SN < "REPAGILITYH" then
							if SN < "PROGBTACHPS" then
								if SN < "PROGBNCMR" then
									if SN < "PROGBMASTERYC" then
										if SN > "PROGBMAINC" then
											if SN == "PROGBMASTERY" then
												return CalcStat("BRatProgB",L,"BRatExtra");
											else
												return 0;
											end
										elseif SN == "PROGBMAINC" then
											return CalcStat("CreepProgB",L);
										else
											return 0;
										end
									elseif SN > "PROGBMASTERYC" then
										if SN < "PROGBMITIGATIONC" then
											if SN == "PROGBMITIGATION" then
												return CalcStat("BRatProgB",L,"BRatMitMedium");
											else
												return 0;
											end
										elseif SN > "PROGBMITIGATIONC" then
											if SN == "PROGBMORALE" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									else
										return CalcStat("CreepProgB",L);
									end
								elseif SN > "PROGBNCMR" then
									if SN < "PROGBOUTHEAL" then
										if SN < "PROGBNCPR" then
											if SN == "PROGBNCMRC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										elseif SN > "PROGBNCPR" then
											if SN == "PROGBNCPRC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("ProgBEnergy",L);
										end
									elseif SN > "PROGBOUTHEAL" then
										if SN < "PROGBRESIST" then
											if SN == "PROGBPOWER" then
												return CalcStat("ProgBEnergy",L);
											else
												return 0;
											end
										elseif SN > "PROGBRESIST" then
											if SN == "PROGBRESISTC" then
												return CalcStat("CreepProgB",L);
											else
												return 0;
											end
										else
											return CalcStat("BRatProgB",L,"BRatExtra");
										end
									else
										return CalcStat("BRatProgB",L,"BRatOutHeal");
									end
								else
									return CalcStat("ProgBHealth",L);
								end
							elseif SN > "PROGBTACHPS" then
								if SN < "PROGEXTLOWEXPRND" then
									if SN < "PROGEXTCOMHIGHRND" then
										if SN < "PROGBVITALITYC" then
											if SN == "PROGBVITALITY" then
												return CalcStat("ProgBHealth",L);
											else
												return 0;
											end
										elseif SN > "PROGBVITALITYC" then
											if SN == "PROGEXTCOMHIGHRAW" then
												if Lm <= 121 then
													return ExpFmod(N,121,20,L,nil);
												elseif Lm <= 125 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",121,N),122,5.5,L,nil);
												elseif Lm <= 126 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",125,N),126,20,L,nil);
												elseif Lm <= 130 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",126,N),127,5.5,L,nil);
												elseif Lm <= 131 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",130,N),131,20,L,nil);
												elseif Lm <= 150 then
													return ExpFmod(CalcStat("ProgExtComHighRaw",131,N),132,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",150,N);
												end
											else
												return 0;
											end
										else
											return CalcStat("CreepProgB",L);
										end
									elseif SN > "PROGEXTCOMHIGHRND" then
										if SN < "PROGEXTCOMLOWRND" then
											if SN == "PROGEXTCOMLOWRAW" then
												if Lm <= 116 then
													return ExpFmod(N,116,20,L,nil);
												elseif Lm <= 120 then
													return ExpFmod(CalcStat("ProgExtComLowRaw",116,N),117,5.5,L,nil);
												else
													return CalcStat("ProgExtComHighRaw",L,CalcStat("ProgExtComLowRaw",120,N));
												end
											else
												return 0;
											end
										elseif SN > "PROGEXTCOMLOWRND" then
											if SN == "PROGEXTHIGHLINEXPRND" then
												if Lm <= 115 then
													return LinFmod(N,2,4,106,115,L,-1);
												else
													return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtHighLinExpRnd",115,N));
												end
											else
												return 0;
											end
										else
											if Lm <= 116 then
												return ExpFmod(N,116,20,L,0);
											elseif Lm <= 120 then
												return ExpFmod(CalcStat("ProgExtComLowRnd",116,N),117,5.5,L,0);
											else
												return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtComLowRnd",120,N));
											end
										end
									else
										if Lm <= 121 then
											return ExpFmod(N,121,20,L,0);
										elseif Lm <= 125 then
											return ExpFmod(CalcStat("ProgExtComHighRnd",121,N),122,5.5,L,0);
										elseif Lm <= 126 then
											return ExpFmod(CalcStat("ProgExtComHighRnd",125,N),126,20,L,0);
										elseif Lm <= 130 then
											return ExpFmod(CalcStat("ProgExtComHighRnd",126,N),127,5.5,L,0);
										elseif Lm <= 131 then
											return ExpFmod(CalcStat("ProgExtComHighRnd",130,N),131,20,L,0);
										elseif Lm <= 150 then
											return ExpFmod(CalcStat("ProgExtComHighRnd",131,N),132,5.5,L,0);
										else
											return CalcStat("ProgExtComHighRnd",150,N);
										end
									end
								elseif SN > "PROGEXTLOWEXPRND" then
									if SN < "REAVERCDCALCTYPECOMPHYMIT" then
										if SN < "PROGEXTMPEXPRND" then
											if SN == "PROGEXTMEDEXPRAW" then
												if Lm <= 106 then
													return ExpFmod(N,106,10,L,nil);
												elseif Lm <= 115 then
													return ExpFmod(CalcStat("ProgExtMedExpRaw",106,N),107,5.5,L,nil);
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("ProgExtMedExpRaw",115,N));
												end
											else
												return 0;
											end
										elseif SN > "PROGEXTMPEXPRND" then
											if SN == "REAVERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 114 then
												return ExpFmod(N,106,7.5,L,0);
											elseif Lm <= 115 then
												return 2*N;
											elseif Lm <= 119 then
												return ExpFmod(CalcStat("ProgExtMpExpRnd",115,N),116,25,L,0);
											elseif Lm <= 120 then
												return RoundDbl((16/3)*N);
											else
												return CalcStat("ProgExtComHighRnd",L,CalcStat("ProgExtMpExpRnd",120,N));
											end
										end
									elseif SN > "REAVERCDCALCTYPECOMPHYMIT" then
										if SN < "REAVERCDCALCTYPETACMIT" then
											if SN == "REAVERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "REAVERCDCALCTYPETACMIT" then
											if SN == "REAVERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									else
										return 13;
									end
								else
									if Lm <= 115 then
										return ExpFmod(N,106,5.5,L,0);
									else
										return CalcStat("ProgExtComLowRnd",L,CalcStat("ProgExtLowExpRnd",115,N));
									end
								end
							else
								return CalcStat("ProgBHealth",L);
							end
						elseif SN > "REPAGILITYH" then
							if SN < "REPWILLH" then
								if SN < "REPMAINL" then
									if SN < "REPFATEH" then
										if SN < "REPCRITDEF" then
											if SN == "REPAGILITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "REPCRITDEF" then
											if SN == "REPCRITHIT" then
												if Lm <= 50 then
													return LinFmod(1,300,496,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,496,636,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,636,716,85,105,L);
												else
													return LinFmod(1,716,776,105,120,L);
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,900,1488,1,50,L);
											elseif Lm <= 85 then
												return LinFmod(1,1488,1908,50,85,L);
											elseif Lm <= 105 then
												return LinFmod(1,1908,2148,85,105,L);
											else
												return LinFmod(1,2148,2328,105,120,L);
											end
										end
									elseif SN > "REPFATEH" then
										if SN < "REPFINESSE" then
											if SN == "REPFATEL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "REPFINESSE" then
											if SN == "REPMAINH" then
												if Lm <= 50 then
													return RoundDblDown(LinFmod(1,80,153,1,50,L));
												elseif Lm <= 85 then
													return RoundDblDown(LinFmod(1,153,206,50,85,L));
												elseif Lm <= 105 then
													return RoundDblDown(LinFmod(1,206,236,85,105,L));
												else
													return RoundDblDown(LinFmod(1,236,258,105,120,L));
												end
											else
												return 0;
											end
										else
											if Lm <= 50 then
												return LinFmod(1,322,557,1,50,L);
											elseif Lm <= 85 then
												return LinFmod(1,557,749.9477,50,85,L);
											elseif Lm <= 105 then
												return LinFmod(1,749.9477,859.1634,85,105,L);
											else
												return LinFmod(1,859.1634,939.2549,105,120,L);
											end
										end
									else
										return CalcStat("RepMainH",L);
									end
								elseif SN > "REPMAINL" then
									if SN < "REPPOWER" then
										if SN < "REPMIGHTL" then
											if SN == "REPMIGHTH" then
												return CalcStat("RepMainH",L);
											else
												return 0;
											end
										elseif SN > "REPMIGHTL" then
											if SN == "REPMORALE" then
												if Lm <= 50 then
													return LinFmod(1,187,427,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,427,599,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,599,697,85,105,L);
												else
													return LinFmod(1,697,770,105,120,L);
												end
											else
												return 0;
											end
										else
											return CalcStat("RepMainL",L);
										end
									elseif SN > "REPPOWER" then
										if SN < "REPVITALITYH" then
											if SN == "REPTACMIT" then
												if Lm <= 50 then
													return LinFmod(1,675,1116,1,50,L);
												elseif Lm <= 85 then
													return LinFmod(1,1116,1431,50,85,L);
												elseif Lm <= 105 then
													return LinFmod(1,1431,1611,85,105,L);
												else
													return LinFmod(1,1611,1746,105,120,L);
												end
											else
												return 0;
											end
										elseif SN > "REPVITALITYH" then
											if SN == "REPVITALITYL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										else
											return CalcStat("RepMainH",L);
										end
									else
										if Lm <= 50 then
											return LinFmod(1,94,212,1,50,L);
										elseif Lm <= 85 then
											return LinFmod(1,212,296,50,85,L);
										elseif Lm <= 105 then
											return LinFmod(1,296,344,85,105,L);
										else
											return LinFmod(1,344,380,105,120,L);
										end
									end
								else
									if Lm <= 50 then
										return RoundDblDown(LinFmod(1,53,102,1,50,L));
									elseif Lm <= 85 then
										return RoundDblDown(LinFmod(1,102,137,50,85,L));
									elseif Lm <= 105 then
										return RoundDblDown(LinFmod(1,137,157,85,105,L));
									else
										return RoundDblDown(LinFmod(1,157,172,105,120,L));
									end
								end
							elseif SN > "REPWILLH" then
								if SN < "RESISTPRATPA" then
									if SN < "RESISTC" then
										if SN < "RESIST" then
											if SN == "REPWILLL" then
												return CalcStat("RepMainL",L);
											else
												return 0;
											end
										elseif SN > "RESIST" then
											if SN == "RESISTADDT" then
												return CalcStat("ResistT",L,N);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPResist","ItemProg","ProgBResist","AdjUmbarItem",L,N,0);
										end
									elseif SN > "RESISTC" then
										if SN < "RESISTPPRAT" then
											if SN == "RESISTPBONUS" then
												return 0;
											else
												return 0;
											end
										elseif SN > "RESISTPPRAT" then
											if SN == "RESISTPRATP" then
												return CalcPercAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCap",L),N);
											else
												return 0;
											end
										else
											return CalcRatAB(CalcStat("ResistPRatPA",L),CalcStat("ResistPRatPB",L),CalcStat("ResistPRatPCapR",L),N);
										end
									else
										return StatLinInter("PntMPResistC","CreepProg","ProgBResistC","",L,N,0);
									end
								elseif SN > "RESISTPRATPA" then
									if SN < "RESISTPRATPCAPR" then
										if SN < "RESISTPRATPC" then
											if SN == "RESISTPRATPB" then
												return CalcStat("BRatRounded",L,"BRatExtra");
											else
												return 0;
											end
										elseif SN > "RESISTPRATPC" then
											if SN == "RESISTPRATPCAP" then
												return 50;
											else
												return 0;
											end
										else
											return 0.5;
										end
									elseif SN > "RESISTPRATPCAPR" then
										if SN < "RESISTT" then
											if SN == "RESISTRKALLRES" then
												return StatLinInter("PntMPResist","ItemProg","ProgBResist","AdjUmbarItem",L,N,0);
											else
												return 0;
											end
										elseif SN > "RESISTT" then
											if SN == "REVERSE" then
												return ReverseCalc(C,L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPResist","TraitProg","ProgBResist","AdjUmbarTrait",L,N,0);
										end
									else
										return CalcStat("ResistPRatPB",L)*CalcStat("ResistPRatPC",L);
									end
								else
									return 150;
								end
							else
								return CalcStat("RepMainH",L);
							end
						else
							return CalcStat("RepMainH",L);
						end
					else
						return CalcStat("StdProg",L,1.75);
					end
				elseif SN > "RIVERHOBBITRDPSVONENAME" then
					if SN < "STALKERCDCALCTYPECOMPHYMIT" then
						if SN < "RUNEKEEPERCDBASEVITALITY" then
							if SN < "RUNEKEEPERCDAGILITYTOEVADE" then
								if SN < "RIVHOBHARDYHOLBMORALE" then
									if SN < "RIVERHOBBITRDTRAITAGILITY" then
										if SN > "RIVERHOBBITRDPSVONEWILL" then
											if SN == "RIVERHOBBITRDPSVTWONAME" then
												return "";
											else
												return 0;
											end
										elseif SN == "RIVERHOBBITRDPSVONEWILL" then
											return CalcStat("RivHobSeenWorldWill",L);
										else
											return 0;
										end
									elseif SN > "RIVERHOBBITRDTRAITAGILITY" then
										if SN < "RIVERHOBBITRDTRAITMORALE" then
											if SN == "RIVERHOBBITRDTRAITFROSTMITP" then
												return CalcStat("RivHobSwimmerFrostMitP",L);
											else
												return 0;
											end
										elseif SN > "RIVERHOBBITRDTRAITMORALE" then
											if SN == "RIVERHOBBITRDTRAITWILL" then
												return CalcStat("RivHobSeclusionWill",L);
											else
												return 0;
											end
										else
											return CalcStat("RivHobHardyHolbMorale",L);
										end
									else
										return CalcStat("RivHobSlipperyAgility",L);
									end
								elseif SN > "RIVHOBHARDYHOLBMORALE" then
									if SN < "RIVHOBSWIMMERFROSTMITP" then
										if SN < "RIVHOBSEENWORLDWILL" then
											if SN == "RIVHOBSECLUSIONWILL" then
												return -CalcStat("WillT",L,0.4);
											else
												return 0;
											end
										elseif SN > "RIVHOBSEENWORLDWILL" then
											if SN == "RIVHOBSLIPPERYAGILITY" then
												return CalcStat("AgilityT",L,1.0);
											else
												return 0;
											end
										else
											return CalcStat("WillT",L,1.0);
										end
									elseif SN > "RIVHOBSWIMMERFROSTMITP" then
										if SN < "RKFORTUNESMILESFATE" then
											if SN == "RKDETERMINATIONWILL" then
												return CalcStat("WillT",L,CalcStat("Trait567810Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "RKFORTUNESMILESFATE" then
											if SN == "RUNEKEEPERCDAGILITYTOCRITHIT" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("FateT",L,CalcStat("Trait567810Choice",N)*0.4);
										end
									else
										return 1;
									end
								else
									return CalcStat("MoraleT",L,1.0);
								end
							elseif SN > "RUNEKEEPERCDAGILITYTOEVADE" then
								if SN < "RUNEKEEPERCDBASEFATE" then
									if SN < "RUNEKEEPERCDARMOURTONONPHYMIT" then
										if SN < "RUNEKEEPERCDAGILITYTOTACMAS" then
											if SN == "RUNEKEEPERCDAGILITYTOFINESSE" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDAGILITYTOTACMAS" then
											if SN == "RUNEKEEPERCDARMOURTOCOMPHYMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "RUNEKEEPERCDARMOURTONONPHYMIT" then
										if SN < "RUNEKEEPERCDARMOURTYPE" then
											if SN == "RUNEKEEPERCDARMOURTOTACMIT" then
												return 0.2;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDARMOURTYPE" then
											if SN == "RUNEKEEPERCDBASEAGILITY" then
												return CalcStat("ClassBaseAgilityL",L);
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 0.2;
									end
								elseif SN > "RUNEKEEPERCDBASEFATE" then
									if SN < "RUNEKEEPERCDBASEMORALE" then
										if SN < "RUNEKEEPERCDBASEICPR" then
											if SN == "RUNEKEEPERCDBASEICMR" then
												return CalcStat("ClassBaseICMRL",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASEICPR" then
											if SN == "RUNEKEEPERCDBASEMIGHT" then
												return CalcStat("ClassBaseMightM",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseICPR",L);
										end
									elseif SN > "RUNEKEEPERCDBASEMORALE" then
										if SN < "RUNEKEEPERCDBASENCPR" then
											if SN == "RUNEKEEPERCDBASENCMR" then
												return CalcStat("ClassBaseNCMRL",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDBASENCPR" then
											if SN == "RUNEKEEPERCDBASEPOWER" then
												return CalcStat("ClassBasePower",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseNCPR",L);
										end
									else
										return CalcStat("ClassBaseMorale",L);
									end
								else
									return CalcStat("ClassBaseFate",L);
								end
							else
								return 1;
							end
						elseif SN > "RUNEKEEPERCDBASEVITALITY" then
							if SN < "RUNEKEEPERCDVITALITYTOMORALE" then
								if SN < "RUNEKEEPERCDMIGHTTOCRITHIT" then
									if SN < "RUNEKEEPERCDCALCTYPETACMIT" then
										if SN < "RUNEKEEPERCDCALCTYPECOMPHYMIT" then
											if SN == "RUNEKEEPERCDBASEWILL" then
												return CalcStat("ClassBaseWillH",L);
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDCALCTYPECOMPHYMIT" then
											if SN == "RUNEKEEPERCDCALCTYPENONPHYMIT" then
												return 12;
											else
												return 0;
											end
										else
											return 12;
										end
									elseif SN > "RUNEKEEPERCDCALCTYPETACMIT" then
										if SN < "RUNEKEEPERCDFATETOPOWER" then
											if SN == "RUNEKEEPERCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDFATETOPOWER" then
											if SN == "RUNEKEEPERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 25;
									end
								elseif SN > "RUNEKEEPERCDMIGHTTOCRITHIT" then
									if SN < "RUNEKEEPERCDPHYMITTOCOMPHYMIT" then
										if SN < "RUNEKEEPERCDMIGHTTOTACMAS" then
											if SN == "RUNEKEEPERCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDMIGHTTOTACMAS" then
											if SN == "RUNEKEEPERCDMIGHTTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 2;
										end
									elseif SN > "RUNEKEEPERCDPHYMITTOCOMPHYMIT" then
										if SN < "RUNEKEEPERCDTACMASTOOUTHEAL" then
											if SN == "RUNEKEEPERCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDTACMASTOOUTHEAL" then
											if SN == "RUNEKEEPERCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							elseif SN > "RUNEKEEPERCDVITALITYTOMORALE" then
								if SN < "SHADOWMIT" then
									if SN < "RUNEKEEPERCDWILLTOPHYMIT" then
										if SN < "RUNEKEEPERCDWILLTOCRITHIT" then
											if SN == "RUNEKEEPERCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDWILLTOCRITHIT" then
											if SN == "RUNEKEEPERCDWILLTOEVADE" then
												return 2;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "RUNEKEEPERCDWILLTOPHYMIT" then
										if SN < "RUNEKEEPERCDWILLTOTACMAS" then
											if SN == "RUNEKEEPERCDWILLTORESIST" then
												return 1;
											else
												return 0;
											end
										elseif SN > "RUNEKEEPERCDWILLTOTACMAS" then
											if SN == "RUNEKEEPERCDWILLTOTACMIT" then
												return 1;
											else
												return 0;
											end
										else
											return 3;
										end
									else
										return 1;
									end
								elseif SN > "SHADOWMIT" then
									if SN < "SKILLPOWERCOST" then
										if SN < "SHIELDBLOCK" then
											if SN == "SHADOWMITT" then
												return CalcStat("DmgTypeMitT",L,N);
											else
												return 0;
											end
										elseif SN > "SHIELDBLOCK" then
											if SN == "SHIELDBRAWLERBLOCK" then
												return CalcStat("DwarfShieldBrwlBlock",L);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPShieldBlock","ItemProg","ProgBBPE","AdjUmbarItem",L,N,0);
										end
									elseif SN > "SKILLPOWERCOST" then
										if SN < "SONGRESISTT" then
											if SN == "SKILLPOWERCOSTMOUNTED" then
												return CalcStat("CombatDamageModEnergy",L,0.64*N);
											else
												return 0;
											end
										elseif SN > "SONGRESISTT" then
											if SN == "STALKERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										if Lm <= 140 then
											return CalcStat("CombatDamageModEnergy",L,0.64*N);
										else
											return CalcStat("CombatDamageModEnergy",L,1.05*0.64*N);
										end
									end
								else
									return CalcStat("DmgTypeMit",L,N);
								end
							else
								return 4.5;
							end
						else
							return CalcStat("ClassBaseVitality",L);
						end
					elseif SN > "STALKERCDCALCTYPECOMPHYMIT" then
						if SN < "STOUTWRBLACKLMIGHT" then
							if SN < "STOUTAXERDPSVTWONAME" then
								if SN < "STDMORALEM" then
									if SN < "STALKERCDHASPOWER" then
										if SN > "STALKERCDCALCTYPENONPHYMIT" then
											if SN == "STALKERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										elseif SN == "STALKERCDCALCTYPENONPHYMIT" then
											return 14;
										else
											return 0;
										end
									elseif SN > "STALKERCDHASPOWER" then
										if SN < "STDMORALEH" then
											if SN == "STATC" then
												return CalcStat(C,L);
											else
												return 0;
											end
										elseif SN > "STDMORALEH" then
											if SN == "STDMORALEL" then
												if Lm <= 105 then
													return RoundDbl(CalcStat("StdMoraleRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleL",105));
												end
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return RoundDbl(2*CalcStat("StdMoraleRaw",L));
											else
												return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleH",105));
											end
										end
									else
										return 1;
									end
								elseif SN > "STDMORALEM" then
									if SN < "STDPOWERRAW" then
										if SN < "STDPOWERH" then
											if SN == "STDMORALERAW" then
												if Lm <= 50 then
													return LinFmod(1,7.51,227.59,1,50,L);
												else
													return LinFmod(1,227.59,1327.5,50,105,L);
												end
											else
												return 0;
											end
										elseif SN > "STDPOWERH" then
											if SN == "STDPOWERL" then
												if Lm <= 105 then
													return RoundDbl(CalcStat("StdPowerRaw",L));
												else
													return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerL",105));
												end
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return RoundDbl(2*CalcStat("StdPowerRaw",L));
											else
												return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdPowerH",105));
											end
										end
									elseif SN > "STDPOWERRAW" then
										if SN < "STOUTAXERDPSVONENAME" then
											if SN == "STDPROG" then
												if Lm <= 50 then
													return LinFmod(N,1,10,1,50,L,"P");
												elseif Lm <= 60 then
													return LinFmod(CalcStat("StdProg",50,N),10/10,15/10,50,60,L,"P");
												elseif Lm <= 65 then
													return LinFmod(CalcStat("StdProg",60,N),15/15,20/15,60,65,L,"P");
												elseif Lm <= 75 then
													return LinFmod(CalcStat("StdProg",65,N),20/20,30/20,65,75,L,"P");
												elseif Lm <= 85 then
													return LinFmod(CalcStat("StdProg",75,N),30/30,45/30,75,85,L,"P");
												elseif Lm <= 95 then
													return LinFmod(CalcStat("StdProg",85,N),45/45,65/45,85,95,L,"P");
												elseif Lm <= 100 then
													return LinFmod(CalcStat("StdProg",95,N),65/65,90.3/65,95,100,L,"P");
												elseif Lm <= 105 then
													return LinFmod(CalcStat("StdProg",100,N),90.3/90.3,120.3/90.3,100,105,L,"P");
												elseif Lm <= 115 then
													return LinFmod(CalcStat("StdProg",105,N),1.1,1.5,106,115,L,"P");
												elseif Lm <= 120 then
													return LinFmod(CalcStat("StdProg",115,N),1.15,1.25,116,120,L,"P");
												elseif Lm <= 130 then
													return LinFmod(CalcStat("StdProg",120,N),1.15,1.5,121,130,L,"P");
												elseif Lm <= 140 then
													return LinFmod(CalcStat("StdProg",130,N),1.15,2,131,140,L,"P");
												elseif Lm <= 150 then
													return LinFmod(CalcStat("StdProg",140,N),1.3,2.205,141,150,L,"P");
												else
													return LinFmod(CalcStat("StdProg",150,N),1,2,151,160,L,"P");
												end
											else
												return 0;
											end
										elseif SN > "STOUTAXERDPSVONENAME" then
											if SN == "STOUTAXERDPSVONEVITALITY" then
												return CalcStat("StoutUnwritDestVitality",L);
											else
												return 0;
											end
										else
											return "Unwritten Destiny";
										end
									else
										if Lm <= 50 then
											return LinFmod(1,3,91.0375,1,50,L);
										else
											return LinFmod(1,91.0375,421,50,105,L);
										end
									end
								else
									if Lm <= 105 then
										return RoundDbl(1.5*CalcStat("StdMoraleRaw",L));
									else
										return CalcStat("ProgExtLowExpRnd",L,CalcStat("StdMoraleM",105));
									end
								end
							elseif SN > "STOUTAXERDPSVTWONAME" then
								if SN < "STOUTAXERDTRAITWILL" then
									if SN < "STOUTAXERDTRAITMIGHT" then
										if SN < "STOUTAXERDTRAITDISEASERESISTP" then
											if SN == "STOUTAXERDTRAITAGILITY" then
												return CalcStat("StoutWrBlackLAgility",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDTRAITDISEASERESISTP" then
											if SN == "STOUTAXERDTRAITFATE" then
												return CalcStat("StoutDoomDrasaFate",L);
											else
												return 0;
											end
										else
											return CalcStat("StoutWrBlackLDiseaseResistP",L);
										end
									elseif SN > "STOUTAXERDTRAITMIGHT" then
										if SN < "STOUTAXERDTRAITSHADOWMITP" then
											if SN == "STOUTAXERDTRAITPHYMITP" then
												return CalcStat("StoutUnyieldingPhyMitP",L);
											else
												return 0;
											end
										elseif SN > "STOUTAXERDTRAITSHADOWMITP" then
											if SN == "STOUTAXERDTRAITVITALITY" then
												return CalcStat("StoutShadowEyeVitality",L);
											else
												return 0;
											end
										else
											return CalcStat("StoutWrBlackLShadowMitP",L);
										end
									else
										return CalcStat("StoutWrBlackLMight",L);
									end
								elseif SN > "STOUTAXERDTRAITWILL" then
									if SN < "STOUTUNYIELDINGPHYMITP" then
										if SN < "STOUTSHADOWEYEVITALITY" then
											if SN == "STOUTDOOMDRASAFATE" then
												return -CalcStat("FateT",L,0.4);
											else
												return 0;
											end
										elseif SN > "STOUTSHADOWEYEVITALITY" then
											if SN == "STOUTUNWRITDESTVITALITY" then
												return CalcStat("VitalityT",L,1.0);
											else
												return 0;
											end
										else
											return -CalcStat("VitalityT",L,0.4);
										end
									elseif SN > "STOUTUNYIELDINGPHYMITP" then
										if SN < "STOUTWRBLACKLAGILITY" then
											if SN == "STOUTUNYIELDINGWILL" then
												return CalcStat("WillT",L,1.0);
											else
												return 0;
											end
										elseif SN > "STOUTWRBLACKLAGILITY" then
											if SN == "STOUTWRBLACKLDISEASERESISTP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("AgilityT",L,1.0);
										end
									else
										return 1;
									end
								else
									return CalcStat("StoutUnyieldingWill",L);
								end
							else
								return "";
							end
						elseif SN > "STOUTWRBLACKLMIGHT" then
							if SN < "TACMASOLD" then
								if SN < "TACDMGPRATP" then
									if SN < "T2PENMIT" then
										if SN < "T2PENARMOUR" then
											if SN == "STOUTWRBLACKLSHADOWMITP" then
												return 1;
											else
												return 0;
											end
										elseif SN > "T2PENARMOUR" then
											if SN == "T2PENBPE" then
												if Lm <= 115 then
													return (-40)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenBPE",115));
												end
											else
												return 0;
											end
										else
											return CalcStat("T2penMit",L);
										end
									elseif SN > "T2PENMIT" then
										if SN < "TACDMGPBONUS" then
											if SN == "T2PENRESIST" then
												if Lm <= 115 then
													return (-90)*L;
												else
													return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenResist",115));
												end
											else
												return 0;
											end
										elseif SN > "TACDMGPBONUS" then
											if SN == "TACDMGPPRAT" then
												return CalcStat("OutDmgPPRat",L,N);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPBonus",L);
										end
									else
										if Lm <= 115 then
											return RoundDblDown(L*13.5)*-5;
										else
											return CalcStat("ProgExtComLowRaw",L,CalcStat("T2PenMit",115));
										end
									end
								elseif SN > "TACDMGPRATP" then
									if SN < "TACDMGPRATPCAP" then
										if SN < "TACDMGPRATPB" then
											if SN == "TACDMGPRATPA" then
												return CalcStat("OutDmgPRatPA",L);
											else
												return 0;
											end
										elseif SN > "TACDMGPRATPB" then
											if SN == "TACDMGPRATPC" then
												return CalcStat("OutDmgPRatPC",L);
											else
												return 0;
											end
										else
											return CalcStat("OutDmgPRatPB",L);
										end
									elseif SN > "TACDMGPRATPCAP" then
										if SN < "TACMAS" then
											if SN == "TACDMGPRATPCAPR" then
												return CalcStat("OutDmgPRatPCapR",L);
											else
												return 0;
											end
										elseif SN > "TACMAS" then
											if SN == "TACMASC" then
												return CalcStat("MasteryC",L,N);
											else
												return 0;
											end
										else
											return CalcStat("Mastery",L,N);
										end
									else
										return CalcStat("OutDmgPRatPCap",L);
									end
								else
									return CalcStat("OutDmgPRatP",L,N);
								end
							elseif SN > "TACMASOLD" then
								if SN < "TACMITHPRATPB" then
									if SN < "TACMITHPBONUS" then
										if SN < "TACMIT" then
											if SN == "TACMAST" then
												return CalcStat("MasteryT",L,N);
											else
												return 0;
											end
										elseif SN > "TACMIT" then
											if SN == "TACMITC" then
												return StatLinInter("PntMPTacMitC","CreepProg","ProgBMitigationC","",L,N,0);
											else
												return 0;
											end
										else
											return StatLinInter("PntMPTacMit","ItemProg","ProgBMitigation","AdjUmbarItemMit",L,N,0);
										end
									elseif SN > "TACMITHPBONUS" then
										if SN < "TACMITHPRATP" then
											if SN == "TACMITHPPRAT" then
												return CalcStat("MitHeavyPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "TACMITHPRATP" then
											if SN == "TACMITHPRATPA" then
												return CalcStat("MitHeavyPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatP",L,N);
										end
									else
										return CalcStat("MitHeavyPBonus",L);
									end
								elseif SN > "TACMITHPRATPB" then
									if SN < "TACMITLPBONUS" then
										if SN < "TACMITHPRATPCAP" then
											if SN == "TACMITHPRATPC" then
												return CalcStat("MitHeavyPRatPC",L);
											else
												return 0;
											end
										elseif SN > "TACMITHPRATPCAP" then
											if SN == "TACMITHPRATPCAPR" then
												return CalcStat("MitHeavyPRatPCapR",L);
											else
												return 0;
											end
										else
											return CalcStat("MitHeavyPRatPCap",L);
										end
									elseif SN > "TACMITLPBONUS" then
										if SN < "TACMITLPRATP" then
											if SN == "TACMITLPPRAT" then
												return CalcStat("MitLightPPRat",L,N);
											else
												return 0;
											end
										elseif SN > "TACMITLPRATP" then
											if SN == "TACMITLPRATPA" then
												return CalcStat("MitLightPRatPA",L);
											else
												return 0;
											end
										else
											return CalcStat("MitLightPRatP",L,N);
										end
									else
										return CalcStat("MitLightPBonus",L);
									end
								else
									return CalcStat("MitHeavyPRatPB",L);
								end
							else
								return CalcStat("Mastery",L,N);
							end
						else
							return CalcStat("MightT",L,1.0);
						end
					else
						return 13;
					end
				else
					return "Seen the World";
				end
			else
				return 60/1200;
			end
		elseif SN > "TACMITLPRATPB" then
			if SN < "WARDENCDMIGHTTOPHYMAS" then
				if SN < "VIRTWISDOMTACMAS" then
					if SN < "VIRTDETERMINATIONVPPHYMAS" then
						if SN < "TRAIT12345CHOICE" then
							if SN < "TOMEFATEDEC" then
								if SN < "TACMITMPRATPA" then
									if SN < "TACMITLPRATPCAPR" then
										if SN > "TACMITLPRATPC" then
											if SN == "TACMITLPRATPCAP" then
												return CalcStat("MitLightPRatPCap",L);
											else
												return 0;
											end
										elseif SN == "TACMITLPRATPC" then
											return CalcStat("MitLightPRatPC",L);
										else
											return 0;
										end
									elseif SN > "TACMITLPRATPCAPR" then
										if SN < "TACMITMPPRAT" then
											if SN == "TACMITMPBONUS" then
												return CalcStat("MitMediumPBonus",L);
											else
												return 0;
											end
										elseif SN > "TACMITMPPRAT" then
											if SN == "TACMITMPRATP" then
												return CalcStat("MitMediumPRatP",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPPRat",L,N);
										end
									else
										return CalcStat("MitLightPRatPCapR",L);
									end
								elseif SN > "TACMITMPRATPA" then
									if SN < "TACMITMPRATPCAPR" then
										if SN < "TACMITMPRATPC" then
											if SN == "TACMITMPRATPB" then
												return CalcStat("MitMediumPRatPB",L);
											else
												return 0;
											end
										elseif SN > "TACMITMPRATPC" then
											if SN == "TACMITMPRATPCAP" then
												return CalcStat("MitMediumPRatPCap",L);
											else
												return 0;
											end
										else
											return CalcStat("MitMediumPRatPC",L);
										end
									elseif SN > "TACMITMPRATPCAPR" then
										if SN < "TACRESISTT" then
											if SN == "TACMITT" then
												return StatLinInter("PntMPTacMit","TraitProg","ProgBMitigation","AdjUmbarTraitMit",L,N,0);
											else
												return 0;
											end
										elseif SN > "TACRESISTT" then
											if SN == "TOMEFATE" then
												return CalcStat("TomeFateDec",RomanRankDecode(C));
											else
												return 0;
											end
										else
											return CalcStat("ResistAddT",L,N);
										end
									else
										return CalcStat("MitMediumPRatPCapR",L);
									end
								else
									return CalcStat("MitMediumPRatPA",L);
								end
							elseif SN > "TOMEFATEDEC" then
								if SN < "TOMETOTALVITALITY" then
									if SN < "TOMETOTALFATEDEC" then
										if SN < "TOMEMAINDEC" then
											if SN == "TOMEMAIN" then
												return CalcStat("TomeMainDec",RomanRankDecode(C));
											else
												return 0;
											end
										elseif SN > "TOMEMAINDEC" then
											if SN == "TOMETOTALFATE" then
												return CalcStat("TomeTotalFateDec",RomanRankDecode(C));
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalMainDec",L)-CalcStat("TomeTotalMainDec",L-1);
										end
									elseif SN > "TOMETOTALFATEDEC" then
										if SN < "TOMETOTALMAIN" then
											if SN == "TOMETOTALLEVEL" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({4,15,27,38,48,54,60,63,67,71,75,78,85,90,95,98,101,104,106,110,114,116,121},L);
												end
											else
												return 0;
											end
										elseif SN > "TOMETOTALMAIN" then
											if SN == "TOMETOTALMAINDEC" then
												return CalcStat("MainT",CalcStat("TomeTotalLevel",L),2.0);
											else
												return 0;
											end
										else
											return CalcStat("TomeTotalMainDec",RomanRankDecode(C));
										end
									else
										return CalcStat("FateT",CalcStat("TomeTotalLevel",L),2.0);
									end
								elseif SN > "TOMETOTALVITALITY" then
									if SN < "TPENARMOUR" then
										if SN < "TOMEVITALITY" then
											if SN == "TOMETOTALVITALITYDEC" then
												return CalcStat("VitalityT",CalcStat("TomeTotalLevel",L),2.0);
											else
												return 0;
											end
										elseif SN > "TOMEVITALITY" then
											if SN == "TOMEVITALITYDEC" then
												return CalcStat("TomeTotalVitalityDec",L)-CalcStat("TomeTotalVitalityDec",L-1);
											else
												return 0;
											end
										else
											return CalcStat("TomeVitalityDec",RomanRankDecode(C));
										end
									elseif SN > "TPENARMOUR" then
										if SN < "TPENCHOICE" then
											if SN == "TPENBPE" then
												return -CalcStat("BPET",L,CalcStat("TpenChoice",N));
											else
												return 0;
											end
										elseif SN > "TPENCHOICE" then
											if SN == "TPENRESIST" then
												return -CalcStat("ResistT",L,CalcStat("TpenChoice",N)*2);
											else
												return 0;
											end
										else
											return DataTableValue({0,1,2},L);
										end
									else
										return -CalcStat("ArmourPenT",L,CalcStat("TpenChoice",N));
									end
								else
									return CalcStat("TomeTotalVitalityDec",RomanRankDecode(C));
								end
							else
								return CalcStat("TomeTotalFateDec",L)-CalcStat("TomeTotalFateDec",L-1);
							end
						elseif SN > "TRAIT12345CHOICE" then
							if SN < "VIRTCHARITYPHYMIT" then
								if SN < "TRAIT47101316CHOICE" then
									if SN < "TRAIT13510CHOICE" then
										if SN < "TRAIT1234CHOICE" then
											if SN == "TRAIT12347CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,7},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT1234CHOICE" then
											if SN == "TRAIT123CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,2,3,4},L);
											end
										end
									elseif SN > "TRAIT13510CHOICE" then
										if SN < "TRAIT234CHOICE" then
											if SN == "TRAIT23456CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({2,3,4,5,6},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT234CHOICE" then
											if SN == "TRAIT357912CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({3,5,7,9,12},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({2,3,4},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({1,3,5,10},L);
										end
									end
								elseif SN > "TRAIT47101316CHOICE" then
									if SN < "TRAITPROG" then
										if SN < "TRAIT56789CHOICE" then
											if SN == "TRAIT567810CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({5,6,7,8,10},L);
												end
											else
												return 0;
											end
										elseif SN > "TRAIT56789CHOICE" then
											if SN == "TRAIT58121620CHOICE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({5,8,12,16,20},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({5,6,7,8,9},L);
											end
										end
									elseif SN > "TRAITPROG" then
										if SN < "U371LEGACYSTATFIX" then
											if SN == "TRAITPROGVITAL" then
												return {{1,25,50,60,65,75,85,95,100,105,115,120,130,140,141,150},{1,25,50,60,65,75,85,95,100,105,115,120,130,140,141,150}};
											else
												return 0;
											end
										elseif SN > "U371LEGACYSTATFIX" then
											if SN == "VARMOUR" then
												return RoundDbl(StatLinInter("PntMPArmourVirtues","ItemProg","ProgBArmourLight","ArmourAdjTypeOther",L,N,0));
											else
												return 0;
											end
										else
											if Lm <= 2 then
												return 0;
											else
												return RoundDblUp(CalcStat(C,L-2)*0.07);
											end
										end
									else
										return {{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141,150},{1,25,50,60,65,75,85,95,100,105,115,120,130,131,140,141,150}};
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({4,7,10,13,16},L);
									end
								end
							elseif SN > "VIRTCHARITYPHYMIT" then
								if SN < "VIRTCONFIDENCECRITHIT" then
									if SN < "VIRTCOMPASSIONARMOUR" then
										if SN < "VIRTCHARITYVITALITY" then
											if SN == "VIRTCHARITYRESIST" then
												return CalcStat("VSResistH",L);
											else
												return 0;
											end
										elseif SN > "VIRTCHARITYVITALITY" then
											if SN == "VIRTCHARITYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVitalityL",L);
										end
									elseif SN > "VIRTCOMPASSIONARMOUR" then
										if SN < "VIRTCOMPASSIONTACMIT" then
											if SN == "VIRTCOMPASSIONPHYMIT" then
												return CalcStat("VSPhyMitH",L);
											else
												return 0;
											end
										elseif SN > "VIRTCOMPASSIONTACMIT" then
											if SN == "VIRTCOMPASSIONVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMitM",L);
										end
									else
										return CalcStat("VSArmourL",L);
									end
								elseif SN > "VIRTCONFIDENCECRITHIT" then
									if SN < "VIRTCONFIDENCEVPTACMAS" then
										if SN < "VIRTCONFIDENCEFINESSE" then
											if SN == "VIRTCONFIDENCEEVADE" then
												return CalcStat("VSEvadeL",L);
											else
												return 0;
											end
										elseif SN > "VIRTCONFIDENCEFINESSE" then
											if SN == "VIRTCONFIDENCEVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSFinesseM",L);
										end
									elseif SN > "VIRTCONFIDENCEVPTACMAS" then
										if SN < "VIRTDETERMINATIONCRITHIT" then
											if SN == "VIRTDETERMINATIONAGILITY" then
												return CalcStat("VSAgilityH",L);
											else
												return 0;
											end
										elseif SN > "VIRTDETERMINATIONCRITHIT" then
											if SN == "VIRTDETERMINATIONPHYMAS" then
												return CalcStat("VSPhyMasM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									else
										return CalcStat("VSVPTacMas",L);
									end
								else
									return CalcStat("VSCritHitH",L);
								end
							else
								return CalcStat("VSPhyMitM",L);
							end
						else
							if Lm <= 0 then
								return 0;
							else
								return DataTableValue({1,2,3,4,5},L);
							end
						end
					elseif SN > "VIRTDETERMINATIONVPPHYMAS" then
						if SN < "VIRTINNOCENCEPHYMIT" then
							if SN < "VIRTFORTITUDEMORALE" then
								if SN < "VIRTEMPATHYCRITDEF" then
									if SN < "VIRTDISCIPLINEPHYMIT" then
										if SN > "VIRTDETERMINATIONVPTACMAS" then
											if SN == "VIRTDISCIPLINEINHEAL" then
												return CalcStat("VSInHealM",L);
											else
												return 0;
											end
										elseif SN == "VIRTDETERMINATIONVPTACMAS" then
											return CalcStat("VSVPTacMas",L);
										else
											return 0;
										end
									elseif SN > "VIRTDISCIPLINEPHYMIT" then
										if SN < "VIRTDISCIPLINEVPMORALE" then
											if SN == "VIRTDISCIPLINERESIST" then
												return CalcStat("VSResistH",L);
											else
												return 0;
											end
										elseif SN > "VIRTDISCIPLINEVPMORALE" then
											if SN == "VIRTEMPATHYARMOUR" then
												return CalcStat("VSArmourH",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSPhyMitL",L);
									end
								elseif SN > "VIRTEMPATHYCRITDEF" then
									if SN < "VIRTFIDELITYTACMIT" then
										if SN < "VIRTEMPATHYVPMORALE" then
											if SN == "VIRTEMPATHYRESIST" then
												return CalcStat("VSResistL",L);
											else
												return 0;
											end
										elseif SN > "VIRTEMPATHYVPMORALE" then
											if SN == "VIRTFIDELITYPHYMIT" then
												return CalcStat("VSPhyMitL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTFIDELITYTACMIT" then
										if SN < "VIRTFIDELITYVPMORALE" then
											if SN == "VIRTFIDELITYVITALITY" then
												return CalcStat("VSVitalityM",L);
											else
												return 0;
											end
										elseif SN > "VIRTFIDELITYVPMORALE" then
											if SN == "VIRTFORTITUDECRITDEF" then
												return CalcStat("VSCritDefM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									else
										return CalcStat("VSTacMitH",L);
									end
								else
									return CalcStat("VSCritDefM",L);
								end
							elseif SN > "VIRTFORTITUDEMORALE" then
								if SN < "VIRTHONOURCRITDEF" then
									if SN < "VIRTHONESTYTACMAS" then
										if SN < "VIRTFORTITUDEVPMORALE" then
											if SN == "VIRTFORTITUDERESIST" then
												return CalcStat("VSResistL",L);
											else
												return 0;
											end
										elseif SN > "VIRTFORTITUDEVPMORALE" then
											if SN == "VIRTHONESTYCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPMorale",L);
										end
									elseif SN > "VIRTHONESTYTACMAS" then
										if SN < "VIRTHONESTYVPTACMAS" then
											if SN == "VIRTHONESTYVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONESTYVPTACMAS" then
											if SN == "VIRTHONESTYWILL" then
												return CalcStat("VSWillM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									else
										return CalcStat("VSTacMasH",L);
									end
								elseif SN > "VIRTHONOURCRITDEF" then
									if SN < "VIRTIDEALISMFATE" then
										if SN < "VIRTHONOURTACMIT" then
											if SN == "VIRTHONOURMORALE" then
												return CalcStat("VSMoraleH",L);
											else
												return 0;
											end
										elseif SN > "VIRTHONOURTACMIT" then
											if SN == "VIRTHONOURVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMitM",L);
										end
									elseif SN > "VIRTIDEALISMFATE" then
										if SN < "VIRTIDEALISMMORALE" then
											if SN == "VIRTIDEALISMINHEAL" then
												return CalcStat("VSInHealM",L);
											else
												return 0;
											end
										elseif SN > "VIRTIDEALISMMORALE" then
											if SN == "VIRTIDEALISMVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSMoraleL",L);
										end
									else
										return CalcStat("VSFateH",L);
									end
								else
									return CalcStat("VSCritDefL",L);
								end
							else
								return CalcStat("VSMoraleH",L);
							end
						elseif SN > "VIRTINNOCENCEPHYMIT" then
							if SN < "VIRTPATIENCECRITHIT" then
								if SN < "VIRTLOYALTYARMOUR" then
									if SN < "VIRTJUSTICEICMR" then
										if SN < "VIRTINNOCENCETACMIT" then
											if SN == "VIRTINNOCENCERESIST" then
												return CalcStat("VSResistM",L);
											else
												return 0;
											end
										elseif SN > "VIRTINNOCENCETACMIT" then
											if SN == "VIRTINNOCENCEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMitL",L);
										end
									elseif SN > "VIRTJUSTICEICMR" then
										if SN < "VIRTJUSTICETACMIT" then
											if SN == "VIRTJUSTICEMORALE" then
												return CalcStat("VSMoraleM",L);
											else
												return 0;
											end
										elseif SN > "VIRTJUSTICETACMIT" then
											if SN == "VIRTJUSTICEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSTacMitL",L);
										end
									else
										return CalcStat("VSICMRH",L);
									end
								elseif SN > "VIRTLOYALTYARMOUR" then
									if SN < "VIRTMERCYEVADE" then
										if SN < "VIRTLOYALTYVITALITY" then
											if SN == "VIRTLOYALTYINHEAL" then
												return CalcStat("VSInHealL",L);
											else
												return 0;
											end
										elseif SN > "VIRTLOYALTYVITALITY" then
											if SN == "VIRTLOYALTYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVitalityH",L);
										end
									elseif SN > "VIRTMERCYEVADE" then
										if SN < "VIRTMERCYVITALITY" then
											if SN == "VIRTMERCYFATE" then
												return CalcStat("VSFateM",L);
											else
												return 0;
											end
										elseif SN > "VIRTMERCYVITALITY" then
											if SN == "VIRTMERCYVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVitalityL",L);
										end
									else
										return CalcStat("VSEvadeH",L);
									end
								else
									return CalcStat("VSArmourM",L);
								end
							elseif SN > "VIRTPATIENCECRITHIT" then
								if SN < "VIRTTOLERANCETACMIT" then
									if SN < "VIRTRNKCOST" then
										if SN < "VIRTPATIENCEPOWER" then
											if SN == "VIRTPATIENCEEVADE" then
												return CalcStat("VSEvadeM",L);
											else
												return 0;
											end
										elseif SN > "VIRTPATIENCEPOWER" then
											if SN == "VIRTPATIENCEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPowerH",L);
										end
									elseif SN > "VIRTRNKCOST" then
										if SN < "VIRTTOLERANCEPHYMIT" then
											if SN == "VIRTRNKCOSTTOT" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat("VirtRnkCostTot",L-1)+CalcStat("VirtRnkCost",L);
												end
											else
												return 0;
											end
										elseif SN > "VIRTTOLERANCEPHYMIT" then
											if SN == "VIRTTOLERANCERESIST" then
												return CalcStat("VSResistM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSPhyMitL",L);
										end
									else
										if Lm <= 0 then
											return 0;
										elseif Lm <= 10 then
											return 1000;
										elseif Lm <= 60 then
											return RoundDbl(18*L+878,-2);
										elseif Lm <= 73 then
											return RoundDbl(18.75*L+878,-2);
										elseif Lm <= 90 then
											return RoundDbl(17.45*L+878,-2);
										else
											return 2500;
										end
									end
								elseif SN > "VIRTTOLERANCETACMIT" then
									if SN < "VIRTVALOURPHYMAS" then
										if SN < "VIRTVALOURCRITHIT" then
											if SN == "VIRTTOLERANCEVPMORALE" then
												return CalcStat("VSVPMorale",L);
											else
												return 0;
											end
										elseif SN > "VIRTVALOURCRITHIT" then
											if SN == "VIRTVALOURFINESSE" then
												return CalcStat("VSFinesseM",L);
											else
												return 0;
											end
										else
											return CalcStat("VSCritHitL",L);
										end
									elseif SN > "VIRTVALOURPHYMAS" then
										if SN < "VIRTVALOURVPTACMAS" then
											if SN == "VIRTVALOURVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTVALOURVPTACMAS" then
											if SN == "VIRTWISDOMFINESSE" then
												return CalcStat("VSFinesseL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									else
										return CalcStat("VSPhyMasH",L);
									end
								else
									return CalcStat("VSTacMitH",L);
								end
							else
								return CalcStat("VSCritHitL",L);
							end
						else
							return CalcStat("VSPhyMitH",L);
						end
					else
						return CalcStat("VSVPPhyMas",L);
					end
				elseif SN > "VIRTWISDOMTACMAS" then
					if SN < "VSMORALEH" then
						if SN < "VRNKLVLCAP" then
							if SN < "VITALITY" then
								if SN < "VIRTWITTACMAS" then
									if SN < "VIRTWISDOMWILL" then
										if SN > "VIRTWISDOMVPPHYMAS" then
											if SN == "VIRTWISDOMVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										elseif SN == "VIRTWISDOMVPPHYMAS" then
											return CalcStat("VSVPPhyMas",L);
										else
											return 0;
										end
									elseif SN > "VIRTWISDOMWILL" then
										if SN < "VIRTWITFINESSE" then
											if SN == "VIRTWITCRITHIT" then
												return CalcStat("VSCritHitM",L);
											else
												return 0;
											end
										elseif SN > "VIRTWITFINESSE" then
											if SN == "VIRTWITPHYMAS" then
												return CalcStat("VSPhyMasL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSFinesseH",L);
										end
									else
										return CalcStat("VSWillH",L);
									end
								elseif SN > "VIRTWITTACMAS" then
									if SN < "VIRTZEALMIGHT" then
										if SN < "VIRTWITVPTACMAS" then
											if SN == "VIRTWITVPPHYMAS" then
												return CalcStat("VSVPPhyMas",L);
											else
												return 0;
											end
										elseif SN > "VIRTWITVPTACMAS" then
											if SN == "VIRTZEALCRITHIT" then
												return CalcStat("VSCritHitL",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPTacMas",L);
										end
									elseif SN > "VIRTZEALMIGHT" then
										if SN < "VIRTZEALVPPHYMAS" then
											if SN == "VIRTZEALPHYMAS" then
												return CalcStat("VSPhyMasM",L);
											else
												return 0;
											end
										elseif SN > "VIRTZEALVPPHYMAS" then
											if SN == "VIRTZEALVPTACMAS" then
												return CalcStat("VSVPTacMas",L);
											else
												return 0;
											end
										else
											return CalcStat("VSVPPhyMas",L);
										end
									else
										return CalcStat("VSMightH",L);
									end
								else
									return CalcStat("VSTacMasL",L);
								end
							elseif SN > "VITALITY" then
								if SN < "VMHIGH" then
									if SN < "VITALITYTADJ" then
										if SN < "VITALITYC" then
											if SN == "VITALITYADJ" then
												if Lm <= 25 then
													return 0.5;
												elseif Lm <= 50 then
													return 0.6;
												elseif Lm <= 60 then
													return 0.7;
												elseif Lm <= 79 then
													return 0.8;
												elseif Lm <= 80 then
													return 0.9;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "VITALITYC" then
											if SN == "VITALITYT" then
												return RoundDblDown(StatLinInter("PntMPVitalityT","TraitProgVital","ProgBVitality","VitalityTAdj",L,N,0));
											else
												return 0;
											end
										else
											return RoundDblDown(StatLinInter("PntMPVitalityC","CreepProg","ProgBVitalityC","",L,N,0));
										end
									elseif SN > "VITALITYTADJ" then
										if SN < "VMASTERYADJ" then
											if SN == "VMASTERY" then
												return StatLinInter("PntMPMastery","ItemProgVirtueMastery","ProgBMastery","VMasteryAdj",L,N,0);
											else
												return 0;
											end
										elseif SN > "VMASTERYADJ" then
											if SN == "VMASTERYOLD" then
												return CalcStat("VMastery",L,N);
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return CalcStat("MasteryAdj",L);
											end
										end
									else
										if Lm <= 25 then
											return 0.5;
										elseif Lm <= 50 then
											return 0.6;
										elseif Lm <= 60 then
											return 0.7;
										elseif Lm <= 65 then
											return 0.8;
										elseif Lm <= 75 then
											return 0.9;
										else
											return 1;
										end
									end
								elseif SN > "VMHIGH" then
									if SN < "VMMORALEPSV" then
										if SN < "VMMASTERYPSV" then
											if SN == "VMLOW" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat(C,CalcStat("VRnkToILvl",L),0.6);
												end
											else
												return 0;
											end
										elseif SN > "VMMASTERYPSV" then
											if SN == "VMMEDIUM" then
												if Lm <= 0 then
													return 0;
												else
													return CalcStat(C,CalcStat("VRnkToILvl",L),1.0);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return CalcStat("VMastery",CalcStat("VRnkToILvl",L),0.2);
											end
										end
									elseif SN > "VMMORALEPSV" then
										if SN < "VMORALEADJ" then
											if SN == "VMORALE" then
												return StatLinInter("PntMPMoraleVirtues","ItemProgVirtueMorale","ProgBMorale","VMoraleAdj",L,N,0);
											else
												return 0;
											end
										elseif SN > "VMORALEADJ" then
											if SN == "VRNKCAP" then
												return 90;
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											elseif Lm <= 2 then
												return 1;
											elseif Lm <= 50 then
												return 2;
											elseif Lm <= 80 then
												return 1.5;
											elseif Lm <= 450 then
												return 1;
											elseif Lm <= 499 then
												return 0.9475;
											else
												return 1;
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return CalcStat("VMorale",CalcStat("VRnkToILvl",L),0.3);
										end
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return CalcStat(C,CalcStat("VRnkToILvl",L),2.0);
									end
								end
							else
								return RoundDblDown(StatLinInter("PntMPVitality","ItemProgVital","ProgBVitality","VitalityAdj",L,N,0));
							end
						elseif SN > "VRNKLVLCAP" then
							if SN < "VSEVADEM" then
								if SN < "VSCRITDEFH" then
									if SN < "VSAGILITYM" then
										if SN < "VSAGILITYH" then
											if SN == "VRNKTOILVL" then
												if Lm <= 0 then
													return 0;
												elseif Lm <= 38 then
													return LinFmod(1,4,78,1,38,L);
												elseif Lm <= 48 then
													return LinFmod(1,78,178,38,48,L);
												elseif Lm <= 49 then
													return LinFmod(1,178,190,48,49,L);
												elseif Lm <= 50 then
													return LinFmod(1,190,210,49,50,L);
												elseif Lm <= 51 then
													return LinFmod(1,210,222,50,51,L);
												elseif Lm <= 52 then
													return LinFmod(1,222,236,51,52,L);
												elseif Lm <= 53 then
													return LinFmod(1,236,260,52,53,L);
												elseif Lm <= 55 then
													return LinFmod(1,260,292,53,55,L);
												elseif Lm <= 68 then
													return LinFmod(1,292,396,55,68,L);
												else
													return LinFmod(1,396,706,68,130,L);
												end
											else
												return 0;
											end
										elseif SN > "VSAGILITYH" then
											if SN == "VSAGILITYL" then
												return CalcStat("VMLow",L,"Agility");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"Agility");
										end
									elseif SN > "VSAGILITYM" then
										if SN < "VSARMOURL" then
											if SN == "VSARMOURH" then
												return CalcStat("VMHigh",L,"Varmour");
											else
												return 0;
											end
										elseif SN > "VSARMOURL" then
											if SN == "VSARMOURM" then
												return CalcStat("VMMedium",L,"Varmour");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Varmour");
										end
									else
										return CalcStat("VMMedium",L,"Agility");
									end
								elseif SN > "VSCRITDEFH" then
									if SN < "VSCRITHITL" then
										if SN < "VSCRITDEFM" then
											if SN == "VSCRITDEFL" then
												return CalcStat("VMLow",L,"CritDef");
											else
												return 0;
											end
										elseif SN > "VSCRITDEFM" then
											if SN == "VSCRITHITH" then
												return CalcStat("VMHigh",L,"CritHit");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"CritDef");
										end
									elseif SN > "VSCRITHITL" then
										if SN < "VSEVADEH" then
											if SN == "VSCRITHITM" then
												return CalcStat("VMMedium",L,"CritHit");
											else
												return 0;
											end
										elseif SN > "VSEVADEH" then
											if SN == "VSEVADEL" then
												return CalcStat("VMLow",L,"Evade");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"Evade");
										end
									else
										return CalcStat("VMLow",L,"CritHit");
									end
								else
									return CalcStat("VMHigh",L,"CritDef");
								end
							elseif SN > "VSEVADEM" then
								if SN < "VSICMRL" then
									if SN < "VSFINESSEH" then
										if SN < "VSFATEL" then
											if SN == "VSFATEH" then
												return CalcStat("VMHigh",L,"Fate");
											else
												return 0;
											end
										elseif SN > "VSFATEL" then
											if SN == "VSFATEM" then
												return CalcStat("VMMedium",L,"Fate");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Fate");
										end
									elseif SN > "VSFINESSEH" then
										if SN < "VSFINESSEM" then
											if SN == "VSFINESSEL" then
												return CalcStat("VMLow",L,"Finesse");
											else
												return 0;
											end
										elseif SN > "VSFINESSEM" then
											if SN == "VSICMRH" then
												return CalcStat("VMHigh",L,"ICMR");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Finesse");
										end
									else
										return CalcStat("VMHigh",L,"Finesse");
									end
								elseif SN > "VSICMRL" then
									if SN < "VSINHEALM" then
										if SN < "VSINHEALH" then
											if SN == "VSICMRM" then
												return CalcStat("VMMedium",L,"ICMR");
											else
												return 0;
											end
										elseif SN > "VSINHEALH" then
											if SN == "VSINHEALL" then
												return CalcStat("VMLow",L,"InHeal");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"InHeal");
										end
									elseif SN > "VSINHEALM" then
										if SN < "VSMIGHTL" then
											if SN == "VSMIGHTH" then
												return CalcStat("VMHigh",L,"Might");
											else
												return 0;
											end
										elseif SN > "VSMIGHTL" then
											if SN == "VSMIGHTM" then
												return CalcStat("VMMedium",L,"Might");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Might");
										end
									else
										return CalcStat("VMMedium",L,"InHeal");
									end
								else
									return CalcStat("VMLow",L,"ICMR");
								end
							else
								return CalcStat("VMMedium",L,"Evade");
							end
						else
							if Lm <= 4 then
								return 2;
							elseif Lm <= 110 then
								return RoundDblDown(L/2);
							elseif Lm <= 139 then
								return L-55;
							elseif Lm <= 140 then
								return L-54;
							elseif Lm <= 149 then
								return L-53;
							else
								return 98;
							end
						end
					elseif SN > "VSMORALEH" then
						if SN < "WARDENCDAGILITYTOCRITHIT" then
							if SN < "VSTACMASH" then
								if SN < "VSPHYMITL" then
									if SN < "VSPHYMASH" then
										if SN > "VSMORALEL" then
											if SN == "VSMORALEM" then
												return CalcStat("VMMedium",L,"VMorale");
											else
												return 0;
											end
										elseif SN == "VSMORALEL" then
											return CalcStat("VMLow",L,"VMorale");
										else
											return 0;
										end
									elseif SN > "VSPHYMASH" then
										if SN < "VSPHYMASM" then
											if SN == "VSPHYMASL" then
												return CalcStat("VMLow",L,"PhyMas");
											else
												return 0;
											end
										elseif SN > "VSPHYMASM" then
											if SN == "VSPHYMITH" then
												return CalcStat("VMHigh",L,"PhyMit");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"PhyMas");
										end
									else
										return CalcStat("VMHigh",L,"PhyMas");
									end
								elseif SN > "VSPHYMITL" then
									if SN < "VSPOWERM" then
										if SN < "VSPOWERH" then
											if SN == "VSPHYMITM" then
												return CalcStat("VMMedium",L,"PhyMit");
											else
												return 0;
											end
										elseif SN > "VSPOWERH" then
											if SN == "VSPOWERL" then
												return CalcStat("VMLow",L,"Power");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"Power");
										end
									elseif SN > "VSPOWERM" then
										if SN < "VSRESISTL" then
											if SN == "VSRESISTH" then
												return CalcStat("VMHigh",L,"Resist");
											else
												return 0;
											end
										elseif SN > "VSRESISTL" then
											if SN == "VSRESISTM" then
												return CalcStat("VMMedium",L,"Resist");
											else
												return 0;
											end
										else
											return CalcStat("VMLow",L,"Resist");
										end
									else
										return CalcStat("VMMedium",L,"Power");
									end
								else
									return CalcStat("VMLow",L,"PhyMit");
								end
							elseif SN > "VSTACMASH" then
								if SN < "VSVITALITYM" then
									if SN < "VSTACMITL" then
										if SN < "VSTACMASM" then
											if SN == "VSTACMASL" then
												return CalcStat("VMLow",L,"TacMas");
											else
												return 0;
											end
										elseif SN > "VSTACMASM" then
											if SN == "VSTACMITH" then
												return CalcStat("VMHigh",L,"TacMit");
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"TacMas");
										end
									elseif SN > "VSTACMITL" then
										if SN < "VSVITALITYH" then
											if SN == "VSTACMITM" then
												return CalcStat("VMMedium",L,"TacMit");
											else
												return 0;
											end
										elseif SN > "VSVITALITYH" then
											if SN == "VSVITALITYL" then
												return CalcStat("VMLow",L,"Vitality");
											else
												return 0;
											end
										else
											return CalcStat("VMHigh",L,"Vitality");
										end
									else
										return CalcStat("VMLow",L,"TacMit");
									end
								elseif SN > "VSVITALITYM" then
									if SN < "VSWILLH" then
										if SN < "VSVPPHYMAS" then
											if SN == "VSVPMORALE" then
												return CalcStat("VMMoralePsv",L);
											else
												return 0;
											end
										elseif SN > "VSVPPHYMAS" then
											if SN == "VSVPTACMAS" then
												return CalcStat("VMMasteryPsv",L);
											else
												return 0;
											end
										else
											return CalcStat("VMMasteryPsv",L);
										end
									elseif SN > "VSWILLH" then
										if SN < "VSWILLM" then
											if SN == "VSWILLL" then
												return CalcStat("VMLow",L,"Will");
											else
												return 0;
											end
										elseif SN > "VSWILLM" then
											if SN == "WARDENCDAGILITYTOBLOCK" then
												return 2;
											else
												return 0;
											end
										else
											return CalcStat("VMMedium",L,"Will");
										end
									else
										return CalcStat("VMHigh",L,"Will");
									end
								else
									return CalcStat("VMMedium",L,"Vitality");
								end
							else
								return CalcStat("VMHigh",L,"TacMas");
							end
						elseif SN > "WARDENCDAGILITYTOCRITHIT" then
							if SN < "WARDENCDBASENCMR" then
								if SN < "WARDENCDARMOURTOTACMIT" then
									if SN < "WARDENCDAGILITYTOPHYMIT" then
										if SN < "WARDENCDAGILITYTOPARRY" then
											if SN == "WARDENCDAGILITYTOOUTHEAL" then
												return 3;
											else
												return 0;
											end
										elseif SN > "WARDENCDAGILITYTOPARRY" then
											if SN == "WARDENCDAGILITYTOPHYMAS" then
												return 3;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "WARDENCDAGILITYTOPHYMIT" then
										if SN < "WARDENCDARMOURTOCOMPHYMIT" then
											if SN == "WARDENCDAGILITYTOTACMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDARMOURTOCOMPHYMIT" then
											if SN == "WARDENCDARMOURTONONPHYMIT" then
												return 0.2;
											else
												return 0;
											end
										else
											return 1;
										end
									else
										return 1;
									end
								elseif SN > "WARDENCDARMOURTOTACMIT" then
									if SN < "WARDENCDBASEICMR" then
										if SN < "WARDENCDBASEAGILITY" then
											if SN == "WARDENCDARMOURTYPE" then
												return 2;
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEAGILITY" then
											if SN == "WARDENCDBASEFATE" then
												return CalcStat("ClassBaseFate",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseAgilityH",L);
										end
									elseif SN > "WARDENCDBASEICMR" then
										if SN < "WARDENCDBASEMIGHT" then
											if SN == "WARDENCDBASEICPR" then
												return CalcStat("ClassBaseICPR",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEMIGHT" then
											if SN == "WARDENCDBASEMORALE" then
												return CalcStat("ClassBaseMorale",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBaseMightM",L);
										end
									else
										return CalcStat("ClassBaseICMRH",L);
									end
								else
									return 0.2;
								end
							elseif SN > "WARDENCDBASENCMR" then
								if SN < "WARDENCDCANBLOCK" then
									if SN < "WARDENCDBASEWILL" then
										if SN < "WARDENCDBASEPOWER" then
											if SN == "WARDENCDBASENCPR" then
												return CalcStat("ClassBaseNCPR",L);
											else
												return 0;
											end
										elseif SN > "WARDENCDBASEPOWER" then
											if SN == "WARDENCDBASEVITALITY" then
												return CalcStat("ClassBaseVitality",L);
											else
												return 0;
											end
										else
											return CalcStat("ClassBasePower",L);
										end
									elseif SN > "WARDENCDBASEWILL" then
										if SN < "WARDENCDCALCTYPENONPHYMIT" then
											if SN == "WARDENCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "WARDENCDCALCTYPENONPHYMIT" then
											if SN == "WARDENCDCALCTYPETACMIT" then
												return 26;
											else
												return 0;
											end
										else
											return 13;
										end
									else
										return CalcStat("ClassBaseWillL",L);
									end
								elseif SN > "WARDENCDCANBLOCK" then
									if SN < "WARDENCDMIGHTTOBLOCK" then
										if SN < "WARDENCDFATETOPOWER" then
											if SN == "WARDENCDFATETONCPR" then
												return 0.07;
											else
												return 0;
											end
										elseif SN > "WARDENCDFATETOPOWER" then
											if SN == "WARDENCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "WARDENCDMIGHTTOBLOCK" then
										if SN < "WARDENCDMIGHTTOFINESSE" then
											if SN == "WARDENCDMIGHTTOCRITHIT" then
												return 1.5;
											else
												return 0;
											end
										elseif SN > "WARDENCDMIGHTTOFINESSE" then
											if SN == "WARDENCDMIGHTTOOUTHEAL" then
												return 2;
											else
												return 0;
											end
										else
											return 1.5;
										end
									else
										return 1;
									end
								else
									return 1;
								end
							else
								return CalcStat("ClassBaseNCMRH",L);
							end
						else
							return 1;
						end
					else
						return CalcStat("VMHigh",L,"VMorale");
					end
				else
					return CalcStat("VSTacMasM",L);
				end
			elseif SN > "WARDENCDMIGHTTOPHYMAS" then
				if SN < "WORTHTABN" then
					if SN < "WORTHTABAU" then
						if SN < "WORTHMPB" then
							if SN < "WARLEADERCDCALCTYPECOMPHYMIT" then
								if SN < "WARDENCDWILLTOFINESSE" then
									if SN < "WARDENCDTACMASTOOUTHEAL" then
										if SN > "WARDENCDPHYMITTOCOMPHYMIT" then
											if SN == "WARDENCDPHYMITTONONPHYMIT" then
												return 1;
											else
												return 0;
											end
										elseif SN == "WARDENCDPHYMITTOCOMPHYMIT" then
											return 1;
										else
											return 0;
										end
									elseif SN > "WARDENCDTACMASTOOUTHEAL" then
										if SN < "WARDENCDVITALITYTOMORALE" then
											if SN == "WARDENCDVITALITYTOICMR" then
												return 0.012;
											else
												return 0;
											end
										elseif SN > "WARDENCDVITALITYTOMORALE" then
											if SN == "WARDENCDVITALITYTONCMR" then
												return 0.12;
											else
												return 0;
											end
										else
											return 4.5;
										end
									else
										return 1;
									end
								elseif SN > "WARDENCDWILLTOFINESSE" then
									if SN < "WARDENCDWILLTOTACMIT" then
										if SN < "WARDENCDWILLTOPHYMAS" then
											if SN == "WARDENCDWILLTOOUTHEAL" then
												return 1;
											else
												return 0;
											end
										elseif SN > "WARDENCDWILLTOPHYMAS" then
											if SN == "WARDENCDWILLTOPHYMIT" then
												return 1.5;
											else
												return 0;
											end
										else
											return 1;
										end
									elseif SN > "WARDENCDWILLTOTACMIT" then
										if SN < "WARDINGLORETACMIT" then
											if SN == "WARDINGLOREPHYMIT" then
												if Lm <= 105 then
													return CalcStat("Mitigation",L,1.6);
												elseif Lm <= 119 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 120 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 129 then
													return CalcStat("PhyMitT",L,1.2);
												elseif Lm <= 130 then
													return CalcStat("PhyMitT",L,1.6);
												elseif Lm <= 139 then
													return CalcStat("PhyMitT",L,1.2);
												else
													return CalcStat("PhyMitT",L,1.6);
												end
											else
												return 0;
											end
										elseif SN > "WARDINGLORETACMIT" then
											if SN == "WARLEADERCANBLOCK" then
												return 1;
											else
												return 0;
											end
										else
											if Lm <= 105 then
												return CalcStat("Mitigation",L,1.6);
											elseif Lm <= 119 then
												return CalcStat("TacMitT",L,1.2);
											elseif Lm <= 120 then
												return CalcStat("TacMitT",L,1.6);
											elseif Lm <= 129 then
												return CalcStat("TacMitT",L,1.2);
											elseif Lm <= 130 then
												return CalcStat("TacMitT",L,1.6);
											elseif Lm <= 139 then
												return CalcStat("TacMitT",L,1.2);
											else
												return CalcStat("TacMitT",L,1.6);
											end
										end
									else
										return 1.5;
									end
								else
									return 1;
								end
							elseif SN > "WARLEADERCDCALCTYPECOMPHYMIT" then
								if SN < "WEAVERCDHASPOWER" then
									if SN < "WEAVERCANBLOCK" then
										if SN < "WARLEADERCDCALCTYPETACMIT" then
											if SN == "WARLEADERCDCALCTYPENONPHYMIT" then
												return 14;
											else
												return 0;
											end
										elseif SN > "WARLEADERCDCALCTYPETACMIT" then
											if SN == "WARLEADERCDHASPOWER" then
												return 1;
											else
												return 0;
											end
										else
											return 27;
										end
									elseif SN > "WEAVERCANBLOCK" then
										if SN < "WEAVERCDCALCTYPENONPHYMIT" then
											if SN == "WEAVERCDCALCTYPECOMPHYMIT" then
												return 13;
											else
												return 0;
											end
										elseif SN > "WEAVERCDCALCTYPENONPHYMIT" then
											if SN == "WEAVERCDCALCTYPETACMIT" then
												return 27;
											else
												return 0;
											end
										else
											return 14;
										end
									else
										return 1;
									end
								elseif SN > "WEAVERCDHASPOWER" then
									if SN < "WORTHEXT" then
										if SN < "WILLC" then
											if SN == "WILL" then
												return CalcStat("Main",L,N);
											else
												return 0;
											end
										elseif SN > "WILLC" then
											if SN == "WILLT" then
												return CalcStat("MainT",L,N);
											else
												return 0;
											end
										else
											return CalcStat("MainC",L,N);
										end
									elseif SN > "WORTHEXT" then
										if SN < "WORTHEXT8LIN" then
											if SN == "WORTHEXT4LIN" then
												if Lm <= 501 then
													return CalcStat("WorthExt",L,C);
												elseif Lm <= 601 then
													return CalcStat("WorthExt",501,C)+(L-501)*4;
												else
													return 0;
												end
											else
												return 0;
											end
										elseif SN > "WORTHEXT8LIN" then
											if SN == "WORTHMPA" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.1,1.15,1.2,1.3},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 501 then
												return CalcStat("WorthExt",L,C);
											elseif Lm <= 601 then
												return CalcStat("WorthExt",501,C)+(L-501)*8;
											else
												return 0;
											end
										end
									else
										if Lm <= 360 then
											return RoundDbl(CalcStat("StatC",L,C));
										elseif Lm <= 601 then
											return RoundDbl(CalcStat("StatC",L-1,C));
										else
											return 0;
										end
									end
								else
									return 1;
								end
							else
								return 14;
							end
						elseif SN > "WORTHMPB" then
							if SN < "WORTHTABAF" then
								if SN < "WORTHMPJ" then
									if SN < "WORTHMPF" then
										if SN < "WORTHMPD" then
											if SN == "WORTHMPC" then
												if Lm <= 0 then
													return 0;
												else
													return 1;
												end
											else
												return 0;
											end
										elseif SN > "WORTHMPD" then
											if SN == "WORTHMPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.1,1.15,1.2,1.25},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,1,1,2,3},L);
											end
										end
									elseif SN > "WORTHMPF" then
										if SN < "WORTHMPH" then
											if SN == "WORTHMPG" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,3,4,5},L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHMPH" then
											if SN == "WORTHMPI" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,2,2.5,3,10},L);
												end
											else
												return 0;
											end
										else
											if Lm <= 0 then
												return 0;
											else
												return DataTableValue({1,1.2,1.8,3.2,5},L);
											end
										end
									else
										if Lm <= 0 then
											return 0;
										else
											return DataTableValue({0.5,1,1.25,1.5,2},L);
										end
									end
								elseif SN > "WORTHMPJ" then
									if SN < "WORTHTABAB" then
										if SN < "WORTHTABA" then
											if SN == "WORTHMPK" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({1,1.2,1.3,1.35,1.4},L);
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABA" then
											if SN == "WORTHTABAA" then
												return CalcStat("WorthTabD",L)+20;
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 1;
											else
												return CalcStat("WorthTabAF",L);
											end
										end
									elseif SN > "WORTHTABAB" then
										if SN < "WORTHTABAD" then
											if SN == "WORTHTABAC" then
												return CalcStat("WorthTabE",L)-30;
											else
												return 0;
											end
										elseif SN > "WORTHTABAD" then
											if SN == "WORTHTABAE" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										else
											return 7*L+100;
										end
									else
										return 7*L+50;
									end
								else
									if Lm <= 0 then
										return 0;
									else
										return DataTableValue({1,1,1,1,5},L);
									end
								end
							elseif SN > "WORTHTABAF" then
								if SN < "WORTHTABAM" then
									if SN < "WORTHTABAJ" then
										if SN < "WORTHTABAH" then
											if SN == "WORTHTABAG" then
												return CalcStat("WorthTabAF",L)+25;
											else
												return 0;
											end
										elseif SN > "WORTHTABAH" then
											if SN == "WORTHTABAI" then
												if Lm <= 16 then
													return 10.73*L+54.3;
												elseif Lm <= 34 then
													return 10.735*L+54;
												elseif Lm <= 35 then
													return 429;
												elseif Lm <= 41 then
													return 10.65*L+57.1;
												elseif Lm <= 49 then
													return 10.7*L+55.3;
												else
													return 9*L+141;
												end
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L+7)-66;
											else
												return 9*L+72;
											end
										end
									elseif SN > "WORTHTABAJ" then
										if SN < "WORTHTABAL" then
											if SN == "WORTHTABAK" then
												if Lm <= 49 then
													return CalcStat("WorthTabALBase",L+1);
												else
													return 9*L+78;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAL" then
											if SN == "WORTHTABALBASE" then
												return 9.86*L+23.51+RoundDbl(L*0.1+0.3)*0.4;
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabALBase",L);
											else
												return 9*L+69;
											end
										end
									else
										return CalcStat("WorthTabAH",L);
									end
								elseif SN > "WORTHTABAM" then
									if SN < "WORTHTABAQ" then
										if SN < "WORTHTABAO" then
											if SN == "WORTHTABAN" then
												if Lm <= 50 then
													return 8.25*L+27.75;
												else
													return 8*L+40;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABAO" then
											if SN == "WORTHTABAP" then
												return CalcStat("WorthTabAQ",L);
											else
												return 0;
											end
										else
											return CalcStat("WorthTabE",L);
										end
									elseif SN > "WORTHTABAQ" then
										if SN < "WORTHTABAS" then
											if SN == "WORTHTABAR" then
												return CalcStat("WorthTabD",L)+37.5;
											else
												return 0;
											end
										elseif SN > "WORTHTABAS" then
											if SN == "WORTHTABAT" then
												if Lm <= 4 then
													return RoundDbl(3*L+20,-1);
												elseif Lm <= 7 then
													return 60;
												elseif Lm <= 10 then
													return 100;
												elseif Lm <= 13 then
													return 180;
												elseif Lm <= 16 then
													return 270;
												elseif Lm <= 19 then
													return 310;
												elseif Lm <= 40 then
													return RoundDbl(3.2*L+274,-1);
												elseif Lm <= 43 then
													return 400;
												elseif Lm <= 53 then
													return RoundDbl(1.9*L+334,-1);
												elseif Lm <= 56 then
													return 460;
												elseif Lm <= 59 then
													return 480;
												else
													return 20*L-680;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabU",L);
										end
									else
										if Lm <= 10 then
											return 7.5*L+60;
										elseif Lm <= 49 then
											return 5.5*L+80;
										else
											return 6*L+55;
										end
									end
								else
									return CalcStat("WorthTabB",L);
								end
							else
								if Lm <= 49 then
									return 2.5*L;
								else
									return 3*L-25;
								end
							end
						else
							if Lm <= 0 then
								return 0;
							else
								return DataTableValue({1,1.2,2,3,4},L);
							end
						end
					elseif SN > "WORTHTABAU" then
						if SN < "WORTHTABBX" then
							if SN < "WORTHTABBH" then
								if SN < "WORTHTABB" then
									if SN < "WORTHTABAX" then
										if SN > "WORTHTABAV" then
											if SN == "WORTHTABAW" then
												if Lm <= 1 then
													return 50;
												elseif Lm <= 49 then
													return 2.5*L+102.5;
												elseif Lm <= 80 then
													return 3*L+78;
												elseif Lm <= 120 then
													return 2.97*L+79.5;
												else
													return 3*L+75;
												end
											else
												return 0;
											end
										elseif SN == "WORTHTABAV" then
											if Lm <= 4 then
												return RoundDbl(3*L+20,-1);
											elseif Lm <= 7 then
												return 70;
											elseif Lm <= 10 then
												return 140;
											elseif Lm <= 13 then
												return 270;
											elseif Lm <= 16 then
												return 400;
											elseif Lm <= 19 then
												return 460;
											elseif Lm <= 25 then
												return RoundDbl(3*L+447.5,-1);
											elseif Lm <= 46 then
												return RoundDbl(3.2*L+455,-1);
											elseif Lm <= 50 then
												return 620;
											else
												return 20*L-360;
											end
										else
											return 0;
										end
									elseif SN > "WORTHTABAX" then
										if SN < "WORTHTABAZ" then
											if SN == "WORTHTABAY" then
												return 5*L;
											else
												return 0;
											end
										elseif SN > "WORTHTABAZ" then
											if SN == "WORTHTABAZBASE" then
												if Lm <= 44 then
													return RoundDbl(0.0525*L+0.7);
												elseif Lm <= 65 then
													return RoundDbl(0.19*L-4.85);
												elseif Lm <= 81 then
													return RoundDbl(0.19*L+28.15);
												else
													return RoundDbl(0.19*L+28.15+RoundDbl(L*0.05-4.55)*0.2);
												end
											else
												return 0;
											end
										else
											if Lm <= 10 then
												return CalcStat("WorthTabAZBase",L)*62.5;
											elseif Lm <= 65 then
												return CalcStat("WorthTabAZBase",L)*125;
											else
												return CalcStat("WorthTabAZBase",L)*25;
											end
										end
									else
										if Lm <= 16 then
											return 10.75*L+54;
										elseif Lm <= 34 then
											return 10.75*L+53.7;
										elseif Lm <= 35 then
											return 429;
										elseif Lm <= 41 then
											return 10.65*L+57.1;
										elseif Lm <= 49 then
											return 10.68*L+56.25;
										else
											return 12*L-9;
										end
									end
								elseif SN > "WORTHTABB" then
									if SN < "WORTHTABBD" then
										if SN < "WORTHTABBB" then
											if SN == "WORTHTABBA" then
												return 5*L+385;
											else
												return 0;
											end
										elseif SN > "WORTHTABBB" then
											if SN == "WORTHTABBC" then
												if Lm <= 34 then
													return RoundDbl(L/15+0.8)*3750+1250;
												elseif Lm <= 35 then
													return 100000;
												else
													return 125500;
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 40;
											elseif Lm <= 10 then
												return RoundDbl(0.35*L-0.1)*20;
											elseif Lm <= 13 then
												return 100;
											elseif Lm <= 19 then
												return RoundDbl(0.35*L+1.6)*20;
											elseif Lm <= 25 then
												return RoundDbl(2*L+150,-1);
											elseif Lm <= 40 then
												return RoundDbl(2*L+164,-1);
											elseif Lm <= 49 then
												return RoundDbl(L+201,-1);
											else
												return 20*L-740;
											end
										end
									elseif SN > "WORTHTABBD" then
										if SN < "WORTHTABBF" then
											if SN == "WORTHTABBE" then
												if Lm <= 1 then
													return 60;
												elseif Lm <= 7 then
													return RoundDbl(L/3)*40+30;
												elseif Lm <= 13 then
													return RoundDbl(L/3)*130-210;
												elseif Lm <= 19 then
													return RoundDbl(L/3)*60+140;
												elseif Lm <= 25 then
													return RoundDbl(L/3)*10+480;
												elseif Lm <= 46 then
													return RoundDbl(L/3)*10+490;
												elseif Lm <= 50 then
													return 660;
												else
													return 20*L-320;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBF" then
											if SN == "WORTHTABBG" then
												if Lm <= 7 then
													return RoundDbl(0.2*L-0.4)*2+1;
												elseif Lm <= 16 then
													return RoundDbl(L/3-2)*6;
												elseif Lm <= 22 then
													return RoundDbl(L/3+1)*3;
												elseif Lm <= 25 then
													return RoundDbl(L/3+1)*3-2;
												elseif Lm <= 80 then
													return RoundDbl(L/3+18)-2;
												else
													return L-37;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabAV",L)+20;
										end
									else
										return CalcStat("WorthTabK",L)+1;
									end
								else
									if Lm <= 50 then
										return 8.25*L+22.25;
									else
										return 8*L+35;
									end
								end
							elseif SN > "WORTHTABBH" then
								if SN < "WORTHTABBP" then
									if SN < "WORTHTABBL" then
										if SN < "WORTHTABBJ" then
											if SN == "WORTHTABBI" then
												return CalcStat("WorthTabG",L)-25;
											else
												return 0;
											end
										elseif SN > "WORTHTABBJ" then
											if SN == "WORTHTABBK" then
												if Lm <= 10 then
													return RoundDbl(0.1*L+0.45);
												else
													return RoundDbl(0.1*L-0.6)*2;
												end
											else
												return 0;
											end
										else
											return RoundDbl(0.1*L+0.45)*3;
										end
									elseif SN > "WORTHTABBL" then
										if SN < "WORTHTABBN" then
											if SN == "WORTHTABBM" then
												if Lm <= 1 then
													return 10;
												else
													return 20*L-20;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBN" then
											if SN == "WORTHTABBO" then
												if Lm <= 10 then
													return 10;
												elseif Lm <= 40 then
													return RoundDbl(0.1*L-0.55)*50-25;
												elseif Lm <= 80 then
													return RoundDbl(0.1*L-0.55)*50;
												else
													return RoundDbl(0.1*L-0.55)*25+175;
												end
											else
												return 0;
											end
										else
											if Lm <= 1 then
												return 400;
											else
												return 20*L+800;
											end
										end
									else
										if Lm <= 35 then
											return 126600;
										elseif Lm <= 46 then
											return 150600;
										elseif Lm <= 47 then
											return 180720;
										else
											return 150600;
										end
									end
								elseif SN > "WORTHTABBP" then
									if SN < "WORTHTABBT" then
										if SN < "WORTHTABBR" then
											if SN == "WORTHTABBQ" then
												if Lm <= 1 then
													return 50;
												elseif Lm <= 49 then
													return RoundDbl(2.5*L+100);
												else
													return 3*L+75;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBR" then
											if SN == "WORTHTABBS" then
												return CalcStat("WorthTabBR",L)*2;
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return CalcStat("WorthTabBQ",L)*2;
											else
												return 4*L+250;
											end
										end
									elseif SN > "WORTHTABBT" then
										if SN < "WORTHTABBV" then
											if SN == "WORTHTABBU" then
												if Lm <= 80 then
													return 20*L+300;
												else
													return 10*L+1100;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABBV" then
											if SN == "WORTHTABBW" then
												if Lm <= 1 then
													return 645;
												elseif Lm <= 9 then
													return 20*L+980;
												else
													return 50*L+920;
												end
											else
												return 0;
											end
										else
											if Lm <= 29 then
												return 1250;
											else
												return RoundDbl(0.05*L-1)*2500;
											end
										end
									else
										return 62500;
									end
								else
									if Lm <= 10 then
										return 156.25*L+1562.5;
									elseif Lm <= 20 then
										return 312.5*L;
									elseif Lm <= 30 then
										return 625*L-6250;
									elseif Lm <= 40 then
										return 1250*L-25000;
									else
										return 2500*L-75000;
									end
								end
							else
								if Lm <= 80 then
									return RoundDbl(0.05*L-0.05)+RoundDbl(0.05*L+0.45);
								else
									return RoundDbl(0.05*L-0.1)+RoundDbl(0.05*L+0.45);
								end
							end
						elseif SN > "WORTHTABBX" then
							if SN < "WORTHTABCM" then
								if SN < "WORTHTABCE" then
									if SN < "WORTHTABCA" then
										if SN < "WORTHTABBZ" then
											if SN == "WORTHTABBY" then
												return 90000;
											else
												return 0;
											end
										elseif SN > "WORTHTABBZ" then
											if SN == "WORTHTABC" then
												return CalcStat("WorthTabD",L)+20;
											else
												return 0;
											end
										else
											if Lm <= 10 then
												return 27.42*L+273.1;
											elseif Lm <= 20 then
												return 54.69*L;
											elseif Lm <= 30 then
												return 109.35*L-1093;
											elseif Lm <= 40 then
												return 218.75*L-4375;
											elseif Lm <= 80 then
												return 437.5*L-13125;
											else
												return 437*L-13085;
											end
										end
									elseif SN > "WORTHTABCA" then
										if SN < "WORTHTABCC" then
											if SN == "WORTHTABCB" then
												return 0.1;
											else
												return 0;
											end
										elseif SN > "WORTHTABCC" then
											if SN == "WORTHTABCD" then
												return CalcStat("WorthTabBK",L)*25;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabBK",L)*5;
										end
									else
										if Lm <= 10 then
											return 7.25*L+72.25;
										elseif Lm <= 20 then
											return 14.49*L+0.11;
										elseif Lm <= 30 then
											return 29*L-290;
										elseif Lm <= 35 then
											return 57.99*L-1160.16;
										elseif Lm <= 40 then
											return 57.99*L-1160.12;
										elseif Lm <= 80 then
											return 115.94*L-3478.2;
										else
											return 116*L-3483;
										end
									end
								elseif SN > "WORTHTABCE" then
									if SN < "WORTHTABCI" then
										if SN < "WORTHTABCG" then
											if SN == "WORTHTABCF" then
												if Lm <= 20 then
													return RoundDbl(0.1*L-0.55)*600+200;
												elseif Lm <= 40 then
													return RoundDbl(0.1*L-0.55)*1000-500;
												elseif Lm <= 60 then
													return RoundDbl(0.1*L-0.55)*2000-4000;
												elseif Lm <= 70 then
													return 8500;
												elseif Lm <= 140 then
													return RoundDbl(0.1*L-0.55)*2500-7500;
												elseif Lm <= 200 then
													return RoundDbl(0.1*L-0.55)*3000-14000;
												else
													return RoundDbl(0.1*L-0.55)*2500-3500;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABCG" then
											if SN == "WORTHTABCH" then
												return 50000;
											else
												return 0;
											end
										else
											return 20;
										end
									elseif SN > "WORTHTABCI" then
										if SN < "WORTHTABCK" then
											if SN == "WORTHTABCJ" then
												return 20000;
											else
												return 0;
											end
										elseif SN > "WORTHTABCK" then
											if SN == "WORTHTABCL" then
												return 12500;
											else
												return 0;
											end
										else
											return 15000;
										end
									else
										return 40000;
									end
								else
									return CalcStat("WorthTabBN",L)*10;
								end
							elseif SN > "WORTHTABCM" then
								if SN < "WORTHTABF" then
									if SN < "WORTHTABCQ" then
										if SN < "WORTHTABCO" then
											if SN == "WORTHTABCN" then
												return RoundDbl(0.1*L+0.5)*10000;
											else
												return 0;
											end
										elseif SN > "WORTHTABCO" then
											if SN == "WORTHTABCP" then
												return 1;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabAB",L);
										end
									elseif SN > "WORTHTABCQ" then
										if SN < "WORTHTABD" then
											if SN == "WORTHTABCR" then
												return CalcStat("WorthTabM",L)-15;
											else
												return 0;
											end
										elseif SN > "WORTHTABD" then
											if SN == "WORTHTABE" then
												if Lm <= 9 then
													return 30*L+50;
												else
													return 7*L+280;
												end
											else
												return 0;
											end
										else
											if Lm <= 49 then
												return 7.5*L;
											else
												return 8*L-25;
											end
										end
									else
										if Lm <= 10 then
											return RoundDbl(0.3*L+0.2)*2;
										elseif Lm <= 16 then
											return RoundDbl(0.25*L-1)*6;
										elseif Lm <= 22 then
											return RoundDbl(0.3*L+5.5)*2;
										elseif Lm <= 50 then
											return RoundDbl((1/6)*L+8.75)*2;
										elseif Lm <= 80 then
											return RoundDbl((1/3)*L-11.5)*6;
										else
											return RoundDbl(0.25*L-4.6)*6;
										end
									end
								elseif SN > "WORTHTABF" then
									if SN < "WORTHTABJ" then
										if SN < "WORTHTABH" then
											if SN == "WORTHTABG" then
												if Lm <= 1 then
													return 50;
												else
													return CalcStat("WorthTabAF",L)+100;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABH" then
											if SN == "WORTHTABI" then
												if Lm <= 4 then
													return 1;
												elseif Lm <= 10 then
													return 0.7*L-1;
												elseif Lm <= 15 then
													return 1.4*L-7;
												elseif Lm <= 24 then
													return 1.25*L-5;
												elseif Lm <= 49 then
													return 2.5*L-35;
												else
													return 3*L-60;
												end
											else
												return 0;
											end
										else
											if Lm <= 2 then
												return L;
											elseif Lm <= 9 then
												return 1.48*L-2.85;
											elseif Lm <= 11 then
												return 2.5*L-13;
											elseif Lm <= 24 then
												return 2.5*L-10;
											else
												return 5*L-70;
											end
										end
									elseif SN > "WORTHTABJ" then
										if SN < "WORTHTABL" then
											if SN == "WORTHTABK" then
												if Lm <= 9 then
													return 12*L+24;
												else
													return 7*L+74;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABL" then
											if SN == "WORTHTABM" then
												if Lm <= 9 then
													return 10*L+11;
												elseif Lm <= 49 then
													return 4*L+65;
												else
													return 5*L+15;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabB",L)+19.25;
										end
									else
										if Lm <= 4 then
											return 2.5*L+7.5;
										elseif Lm <= 5 then
											return 23;
										else
											return 4*L;
										end
									end
								else
									return 0.1;
								end
							else
								return 7500;
							end
						else
							if Lm <= 10 then
								return RoundDbl(0.1*L+0.45)*3;
							elseif Lm <= 140 then
								return RoundDbl(0.1*L-0.6)*6;
							else
								return RoundDbl(0.1*L+0.4)*4+22;
							end
						end
					else
						return CalcStat("WorthTabBN",L)*1.25;
					end
				elseif SN > "WORTHTABN" then
					if SN < "WORTHVALBW" then
						if SN < "WORTHVALAR" then
							if SN < "WORTHVALAB" then
								if SN < "WORTHTABU" then
									if SN < "WORTHTABQ" then
										if SN > "WORTHTABO" then
											if SN == "WORTHTABP" then
												return CalcStat("WorthTabE",L);
											else
												return 0;
											end
										elseif SN == "WORTHTABO" then
											return CalcStat("WorthTabB",L)+11;
										else
											return 0;
										end
									elseif SN > "WORTHTABQ" then
										if SN < "WORTHTABS" then
											if SN == "WORTHTABR" then
												if Lm <= 9 then
													return 12*L+30;
												else
													return 7*L+75;
												end
											else
												return 0;
											end
										elseif SN > "WORTHTABS" then
											if SN == "WORTHTABT" then
												if Lm <= 1 then
													return 54;
												elseif Lm <= 17 then
													return 10.73*L+43.57;
												elseif Lm <= 35 then
													return 10.735*L+43.265;
												elseif Lm <= 36 then
													return 429;
												elseif Lm <= 42 then
													return 10.65*L+46.45;
												elseif Lm <= 49 then
													return 10.7*L+44.6;
												else
													return 9*L+130;
												end
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+17.5;
										end
									else
										return 7*L+25;
									end
								elseif SN > "WORTHTABU" then
									if SN < "WORTHTABY" then
										if SN < "WORTHTABW" then
											if SN == "WORTHTABV" then
												return CalcStat("WorthTabB",L)-2.75;
											else
												return 0;
											end
										elseif SN > "WORTHTABW" then
											if SN == "WORTHTABX" then
												return CalcStat("WorthTabAF",L)+75;
											else
												return 0;
											end
										else
											return CalcStat("WorthTabD",L)+25;
										end
									elseif SN > "WORTHTABY" then
										if SN < "WORTHVALA" then
											if SN == "WORTHTABZ" then
												return CalcStat("WorthTabR",L)-15;
											else
												return 0;
											end
										elseif SN > "WORTHVALA" then
											if SN == "WORTHVALAA" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAA"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabA"));
										end
									else
										return CalcStat("WorthTabD",L)+30;
									end
								else
									if Lm <= 10 then
										return 9*L+20;
									elseif Lm <= 49 then
										return 5.5*L+55;
									else
										return 6*L+30;
									end
								end
							elseif SN > "WORTHVALAB" then
								if SN < "WORTHVALAJ" then
									if SN < "WORTHVALAF" then
										if SN < "WORTHVALAD" then
											if SN == "WORTHVALAC" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAC"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAD" then
											if SN == "WORTHVALAE" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAE"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAD"));
										end
									elseif SN > "WORTHVALAF" then
										if SN < "WORTHVALAH" then
											if SN == "WORTHVALAG" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAG"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAH" then
											if SN == "WORTHVALAI" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAI"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAH"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAF"));
									end
								elseif SN > "WORTHVALAJ" then
									if SN < "WORTHVALAN" then
										if SN < "WORTHVALAL" then
											if SN == "WORTHVALAK" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAK"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAL" then
											if SN == "WORTHVALAM" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAM"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAL"));
										end
									elseif SN > "WORTHVALAN" then
										if SN < "WORTHVALAP" then
											if SN == "WORTHVALAO" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAO"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAP" then
											if SN == "WORTHVALAQ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAQ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAP"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAN"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabAJ"));
								end
							else
								return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAB"));
							end
						elseif SN > "WORTHVALAR" then
							if SN < "WORTHVALBG" then
								if SN < "WORTHVALAZ" then
									if SN < "WORTHVALAV" then
										if SN < "WORTHVALAT" then
											if SN == "WORTHVALAS" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAS"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAT" then
											if SN == "WORTHVALAU" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAU"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAT"));
										end
									elseif SN > "WORTHVALAV" then
										if SN < "WORTHVALAX" then
											if SN == "WORTHVALAW" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAW"));
											else
												return 0;
											end
										elseif SN > "WORTHVALAX" then
											if SN == "WORTHVALAY" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabAY"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAX"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAV"));
									end
								elseif SN > "WORTHVALAZ" then
									if SN < "WORTHVALBC" then
										if SN < "WORTHVALBA" then
											if SN == "WORTHVALB" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabB"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBA" then
											if SN == "WORTHVALBB" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBB"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBA"));
										end
									elseif SN > "WORTHVALBC" then
										if SN < "WORTHVALBE" then
											if SN == "WORTHVALBD" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBD"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBE" then
											if SN == "WORTHVALBF" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBF"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpJ",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBE"));
										end
									else
										return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBC"));
									end
								else
									return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabAZ"));
								end
							elseif SN > "WORTHVALBG" then
								if SN < "WORTHVALBO" then
									if SN < "WORTHVALBK" then
										if SN < "WORTHVALBI" then
											if SN == "WORTHVALBH" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBH"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBI" then
											if SN == "WORTHVALBJ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBJ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBI"));
										end
									elseif SN > "WORTHVALBK" then
										if SN < "WORTHVALBM" then
											if SN == "WORTHVALBL" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBL"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBM" then
											if SN == "WORTHVALBN" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBN"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBM"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBK"));
									end
								elseif SN > "WORTHVALBO" then
									if SN < "WORTHVALBS" then
										if SN < "WORTHVALBQ" then
											if SN == "WORTHVALBP" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBP"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBQ" then
											if SN == "WORTHVALBR" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBR"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBQ"));
										end
									elseif SN > "WORTHVALBS" then
										if SN < "WORTHVALBU" then
											if SN == "WORTHVALBT" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBT"));
											else
												return 0;
											end
										elseif SN > "WORTHVALBU" then
											if SN == "WORTHVALBV" then
												return RoundDbl(CalcStat("WorthMpD",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBV"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpF",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBU"));
										end
									else
										return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBS"));
									end
								else
									return RoundDbl(CalcStat("WorthMpG",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBO"));
								end
							else
								return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBG"));
							end
						else
							return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabAR"));
						end
					elseif SN > "WORTHVALBW" then
						if SN < "WORTHVALL" then
							if SN < "WORTHVALCK" then
								if SN < "WORTHVALCC" then
									if SN < "WORTHVALBZ" then
										if SN > "WORTHVALBX" then
											if SN == "WORTHVALBY" then
												return RoundDbl(CalcStat("WorthMpE",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBY"));
											else
												return 0;
											end
										elseif SN == "WORTHVALBX" then
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabBX"));
										else
											return 0;
										end
									elseif SN > "WORTHVALBZ" then
										if SN < "WORTHVALCA" then
											if SN == "WORTHVALC" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabC"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCA" then
											if SN == "WORTHVALCB" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCB"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCA"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBZ"));
									end
								elseif SN > "WORTHVALCC" then
									if SN < "WORTHVALCG" then
										if SN < "WORTHVALCE" then
											if SN == "WORTHVALCD" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCD"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCE" then
											if SN == "WORTHVALCF" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCF"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpH",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCE"));
										end
									elseif SN > "WORTHVALCG" then
										if SN < "WORTHVALCI" then
											if SN == "WORTHVALCH" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCH"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCI" then
											if SN == "WORTHVALCJ" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCJ"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCI"));
										end
									else
										return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCG"));
									end
								else
									return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabCC"));
								end
							elseif SN > "WORTHVALCK" then
								if SN < "WORTHVALD" then
									if SN < "WORTHVALCO" then
										if SN < "WORTHVALCM" then
											if SN == "WORTHVALCL" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCL"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCM" then
											if SN == "WORTHVALCN" then
												return RoundDbl(CalcStat("WorthMpI",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCN"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCM"));
										end
									elseif SN > "WORTHVALCO" then
										if SN < "WORTHVALCQ" then
											if SN == "WORTHVALCP" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCP"));
											else
												return 0;
											end
										elseif SN > "WORTHVALCQ" then
											if SN == "WORTHVALCR" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCR"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpK",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCQ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCO"));
									end
								elseif SN > "WORTHVALD" then
									if SN < "WORTHVALH" then
										if SN < "WORTHVALF" then
											if SN == "WORTHVALE" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabE"));
											else
												return 0;
											end
										elseif SN > "WORTHVALF" then
											if SN == "WORTHVALG" then
												return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabG"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabF"));
										end
									elseif SN > "WORTHVALH" then
										if SN < "WORTHVALJ" then
											if SN == "WORTHVALI" then
												return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabI"));
											else
												return 0;
											end
										elseif SN > "WORTHVALJ" then
											if SN == "WORTHVALK" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabK"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabJ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpA",QualityCodeIndex(C))*CalcStat("WorthExt4Lin",L,"WorthTabH"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabD"));
								end
							else
								return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabCK"));
							end
						elseif SN > "WORTHVALL" then
							if SN < "WPNDMGMAX" then
								if SN < "WORTHVALT" then
									if SN < "WORTHVALP" then
										if SN < "WORTHVALN" then
											if SN == "WORTHVALM" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabM"));
											else
												return 0;
											end
										elseif SN > "WORTHVALN" then
											if SN == "WORTHVALO" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabO"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabN"));
										end
									elseif SN > "WORTHVALP" then
										if SN < "WORTHVALR" then
											if SN == "WORTHVALQ" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabQ"));
											else
												return 0;
											end
										elseif SN > "WORTHVALR" then
											if SN == "WORTHVALS" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabS"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabR"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabP"));
									end
								elseif SN > "WORTHVALT" then
									if SN < "WORTHVALX" then
										if SN < "WORTHVALV" then
											if SN == "WORTHVALU" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabU"));
											else
												return 0;
											end
										elseif SN > "WORTHVALV" then
											if SN == "WORTHVALW" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabW"));
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabV"));
										end
									elseif SN > "WORTHVALX" then
										if SN < "WORTHVALZ" then
											if SN == "WORTHVALY" then
												return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabY"));
											else
												return 0;
											end
										elseif SN > "WORTHVALZ" then
											if SN == "WOUNDRESISTT" then
												return CalcStat("ResistAddT",L,N);
											else
												return 0;
											end
										else
											return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabZ"));
										end
									else
										return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabX"));
									end
								else
									return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt8Lin",L,"WorthTabT"));
								end
							elseif SN > "WPNDMGMAX" then
								if SN < "WRDPHYMAS" then
									if SN < "WRDBATSTRIKESCRITDEF" then
										if SN < "WPNDPS" then
											if SN == "WPNDMGMIN" then
												return ((2-2*CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2)))/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
											else
												return 0;
											end
										elseif SN > "WPNDPS" then
											if SN == "WPNDPSVARIANCETYPE" then
												if Lm <= 0 then
													return 0;
												else
													return DataTableValue({0.25,0.25,0.25},L);
												end
											else
												return 0;
											end
										else
											return CalcStat("CombatBase",L,C);
										end
									elseif SN > "WRDBATSTRIKESCRITDEF" then
										if SN < "WRDFINESSE" then
											if SN == "WRDCRITDEF" then
												return CalcStat("CritDefT",L,1.0);
											else
												return 0;
											end
										elseif SN > "WRDFINESSE" then
											if SN == "WRDIMPRBLADESPARRY" then
												return CalcStat("ParryT",L,2.8);
											else
												return 0;
											end
										else
											return CalcStat("FinesseT",L,CalcStat("Trait12345Choice",N)*0.4);
										end
									else
										return CalcStat("CritDefT",L,CalcStat("Trait123Choice",N)*0.4);
									end
								elseif SN > "WRDPHYMAS" then
									if SN < "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSHIELDMASBLOCK" then
											if SN == "WRDRECKLESSNCRITHIT" then
												return CalcStat("CritHitT",L,2.4);
											else
												return 0;
											end
										elseif SN > "WRDSHIELDMASBLOCK" then
											if SN == "WRDSHIELDTACTCRITDEF" then
												return CalcStat("CritDefT",L,2);
											else
												return 0;
											end
										else
											return CalcStat("BlockT",L,2.8);
										end
									elseif SN > "WRDSSSHPIERCERBLOCK" then
										if SN < "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRBLOCK" then
												return CalcStat("BlockT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										elseif SN > "WRDSTDYOURGREVADE" then
											if SN == "WRDSTDYOURGRPARRY" then
												return CalcStat("ParryT",L,CalcStat("Trait1234Choice",N)*0.4);
											else
												return 0;
											end
										else
											return -CalcStat("EvadeT",L,CalcStat("Trait1234Choice",N)*0.4);
										end
									else
										return -CalcStat("BlockT",L,4);
									end
								else
									return CalcStat("PhyMasT",L,CalcStat("Trait12345Choice",N)*0.4);
								end
							else
								return (2/(2-CalcStat("WpnDPSVarianceType",WpnCodeIndex(C,2))))*CalcStat("CombatBase",L,C);
							end
						else
							return RoundDbl(CalcStat("WorthMpB",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabL"));
						end
					else
						return RoundDbl(CalcStat("WorthMpC",QualityCodeIndex(C))*CalcStat("WorthExt",L,"WorthTabBW"));
					end
				else
					return CalcStat("WorthTabAF",L)+25;
				end
			else
				return 2;
			end
		else
			return CalcStat("MitLightPRatPB",L);
		end
	else
		return CalcStat("ClassBasePower",L);
	end

end

-- Support functions for CalcStat. These consist of implementations of more complex calculation types, decode methods for parameter "C" and rounding/min/max/compare functions for floating point numbers.

-- ****************** Calculation Type support functions ******************

-- DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
-- DataTableValue: Takes a value from an array table.

function DataTableValue(vDataArray, dIndex)

	local lIndex = RoundDbl(dIndex);
	if lIndex <= 1 then
		return vDataArray[1];
	elseif lIndex > #vDataArray then
		return vDataArray[#vDataArray];
	else
		return vDataArray[lIndex];
	end

end

-- EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
-- ExpFmod: Exponential function based on percentage.
-- Common percentage values are around ~5.5% for between levels and ~20% jumps between level segments.

function ExpFmod(dVal, dLstart, dPlvl, dLvl, vDec, vAdd)

	local dRng = dLvl-dLstart+1;
	if dRng <= DblCalcDev then
		return dVal;
	else
		local dFac = 1+dPlvl/100;
		local dAdd;
		if vAdd == nil then
			dAdd = 0;
		else
			dAdd = vAdd;
		end
		if vDec == nil then
			local dFacExp = dFac^dRng;
			return dVal*dFacExp+dAdd*((dFacExp-1)/(dFac-1));
		else
			local dResult = dVal;
			local dLm = dLstart-DblCalcDev;
			while dLm <= dLvl do
				dResult = RoundDbl(dResult*dFac+dAdd,vDec);
				dLm = dLm+1;
			end
			return dResult;
		end
	end

end

-- PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
-- CalcPercAB: Calculates the percentage out of a rating based on the AB formula.

function CalcPercAB(dA, dB, dPCap, dR)

	if dR <= DblCalcDev then
		return 0.0;
	else
		local dResult = dA/(1+dB/dR);
		if dResult >= dPCap-DblCalcDev then
			return dPCap;
		else
			return dResult;
		end
	end

end

-- RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
-- CalcRatAB: Calculates the rating out of a percentage based on the AB formula.

function CalcRatAB(dA, dB, dCapR, dP)

	if dP <= DblCalcDev then
		return 0.0;
	else
		local dResult = dB/(dA/dP-1);
		if dResult >= dCapR-DblCalcDev then
			return dCapR;
		else
			return dResult;
		end
	end

end

-- SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS
-- StatLinInter: (Normalized) Stat Linear Interpolating

function StatLinInter(sPntMP, sProgScheme, sProgBase, sAdj, dLvl, vNorC, vRoundType)

	-- parameter processing
	local dN = 1;
	local sC = "";
	if type(vNorC) == "number" then
		dN = vNorC;
	elseif type(vNorC) == "string" then
		sC = vNorC;
	end

	local dRoundType = 0;
	if type(vRoundType) == "number" then
		dRoundType = vRoundType;
	end
	
	local dProgScheme = CalcStat(sProgScheme,dLvl);
	if type(dProgScheme) ~= "table" then
		return 0.0;
	end

	-- find level interval
	local dLvlMinus = dLvl-DblCalcDev;
	local iPointIndexHigh = 2;
	local iPointIndexMax = #dProgScheme[1];
	while iPointIndexHigh < iPointIndexMax do
		if dLvlMinus <= dProgScheme[1][iPointIndexHigh] then
			break;
		end
		iPointIndexHigh = iPointIndexHigh+1;
	end
	local iPointIndexLow = iPointIndexHigh-1;
		
	local dAccessLvlLow = dProgScheme[1][iPointIndexLow];
	local dAccessLvlHigh = dProgScheme[1][iPointIndexHigh];
	local dBaseLvlLow = dProgScheme[2][iPointIndexLow];
	local dBaseLvlHigh = dProgScheme[2][iPointIndexHigh];
	
	-- get values from base progression
	local dValLow = CalcStat(sProgBase,dBaseLvlLow,sC);
	local dValHigh = CalcStat(sProgBase,dBaseLvlHigh,sC);

	-- graph point multiplications
	if type(sPntMP) == "string" and sPntMP ~= "" then
		dValLow = dValLow*CalcStat(sPntMP,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sPntMP,dAccessLvlHigh,sC);
	end
	if type(sAdj) == "string" and sAdj ~= "" then
		dValLow = dValLow*CalcStat(sAdj,dAccessLvlLow,sC);
		dValHigh = dValHigh*CalcStat(sAdj,dAccessLvlHigh,sC);
	end
	dValLow = dValLow*dN;
	dValHigh = dValHigh*dN;

	-- graph point roundings
	if dRoundType == 0 then
		dValLow = RoundDblLotro(dValLow);
		dValHigh = RoundDblLotro(dValHigh);
	elseif dRoundType == 1 then
		if -100.0 <= dValLow and dValLow <= 100.0 then
			dValLow = RoundDblUp(dValLow,2);
		elseif -1000.0 <= dValLow and dValLow <= 1000.0 then
			dValLow = RoundDblUp(dValLow,1);
		else
			dValLow = RoundDblLotro(dValLow);
		end
		if -100.0 <= dValHigh and dValHigh <= 100.0 then
			dValHigh = RoundDblUp(dValHigh,2);
		elseif -1000.0 <= dValHigh and dValHigh <= 1000.0 then
			dValHigh = RoundDblUp(dValHigh,1);
		else
			dValHigh = RoundDblLotro(dValHigh);
		end
	elseif dRoundType == 2 then
		dValLow = RoundDblLotro(dValLow);
		if dValLow == -1 then
			dValLow = -2;
		end
		dValHigh = RoundDblLotro(dValHigh);
		if dValHigh == -1 then
			dValHigh = -2;
		end
	end

	-- return interpolated value from the calculated graph points
	return LinFmod(1,dValLow,dValHigh,dAccessLvlLow,dAccessLvlHigh,dLvl);

end

-- TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT
-- LinFmod: Linear line function between 2 points with some optional modifications.
-- Connects point (dLstart,dVal*dFstart) with (dLend,dVal*dFend).
-- Usually used with dVal=1 and dFstart/dFend containing unrelated points or dVal=# and dFstart/dFend containing multiplier factors.
-- Modification for in-between points on the line: rounding.

function LinFmod(dVal, dFstart, dFend, dLstart, dLend, dLvl, vDec)

	if type(vDec) == "string" then
		local sRoundType = string.upper(vDec);
		if sRoundType == "P" then
			return LinFmod(1,RoundDblProg(dVal*dFstart),RoundDblProg(dVal*dFend),dLstart,dLend,dLvl);
		elseif sRoundType == "L" then
			return LinFmod(1,RoundDblLotro(dVal*dFstart),RoundDblLotro(dVal*dFend),dLstart,dLend,dLvl);
		else
			return LinFmod(1,dVal*dFstart,dVal*dFend,dLstart,dLend,dLvl);
		end
	end
	if dLstart-DblCalcDev <= dLvl and dLvl <= dLstart+DblCalcDev then
		return dVal*dFstart;
	elseif dLend-DblCalcDev <= dLvl and dLvl <= dLend+DblCalcDev then
		return dVal*dFend;
	elseif dLstart == dLend then
		return 0.0;
	elseif vDec == nil then
		return dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart);
	else
		return RoundDbl(dVal*(dFstart*(dLend-dLvl)+(dLvl-dLstart)*dFend)/(dLend-dLstart),vDec);
	end

end

-- ****************** Parameter "C" decode support functions ******************

-- ArmCodeIndex: returns a specified index from an Armour Code.
-- sACode string:
-- 1st position: H=heavy, M=medium, L=light
-- 2nd position: H=head, S=shoulders, CL=cloak/back, C=chest, G=gloves, L=leggings, B=boots, Sh=shield
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic
-- Note: no such thing exists as a heavy, medium or light cloak, so no H/M/L in cloak codes (cloaks go automatically in the M class since U23, although historically this was L)

function ArmCodeIndex(sACode, iI)

	if sACode == nil or iI == nil then
		return 0;
	end
	if type(sACode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sArmCode = string.match(sACode,"(%a+)");
	if not sArmCode then
		return 0;
	end
	sArmCode = string.upper(sArmCode).."    ";
	
	local sArmCat = string.sub(sArmCode,1,1);
	local sArmType = string.sub(sArmCode,2,2);
	local sArmCol = string.sub(sArmCode,3,3);

	if sArmType == "S" and sArmCol == "H" then
		sArmType = "SH";
		sArmCol = string.sub(sArmCode,4,4);
	elseif sArmCat == "C" and sArmType == "L" then
		sArmCat = "M";
		sArmType = "CL";
	elseif sArmType then
		sArmType = " "..sArmType;
	end
	
	local result = 0;
	if iI == 1 then
		if sArmCat then
			result = string.find("HML",sArmCat);
		end
	elseif iI == 2 then
		if sArmType then
			result = string.find(" H SCL C G L BSH",sArmType);
			if result then
				result = (result+1)/2;
			end
		end
	elseif iI == 3 then
		if sArmCol then
			result = string.find("WYPTG",sArmCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- QualityCodeIndex: returns a quality index from a Quality Code.
-- sQCode string: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function QualityCodeIndex(sQCode)

	if sQCode == nil then
		return 0;
	end
	if type(sQCode) ~= "string" then
		return 0;
	end

	local sQtyCode = string.match(sQCode,"(%a+)");
	if not sQtyCode then
		return 0;
	end
	sQtyCode = string.upper(sQtyCode).." ";

	local sQtyCol = string.sub(sQtyCode,1,1);
	
	local result = 0;
	if sQtyCol then
		result = string.find("WYPTG",sQtyCol);
	end
	
	if result then
		return result;
	else
		return 0;
	end
	
end

-- WpnCodeIndex: returns a specified index from a Weapon Code.
-- sWCode string:
-- 1st position: H=heavy, L=light
-- 2nd position: O=one-handed, T=two-handed, B=bow
-- 3rd position: W=white/common, Y=yellow/uncommon, P=purple/rare, T=teal/blue/incomparable, G=gold/legendary/epic

function WpnCodeIndex(sWCode, iI)

	if sWCode == nil or iI == nil then
		return 0;
	end
	if type(sWCode) ~= "string" or type(iI) ~= "number" then
		return 0;
	end

	local sWpnCode = string.match(sWCode,"(%a+)");
	if not sWpnCode then
		return 0;
	end
	sWpnCode = string.upper(sWpnCode).."   ";
	
	local sWpnCat = string.sub(sWpnCode,1,1);
	local sWpnType = string.sub(sWpnCode,2,2);
	local sWpnCol = string.sub(sWpnCode,3,3);

	local result = 0;
	if iI == 1 then
		if sWpnCat then
			result = string.find("HL",sWpnCat);
		end
	elseif iI == 2 then
		if sWpnType then
			result = string.find("OTB",sWpnType);
		end
	elseif iI == 3 then
		if sWpnCol then
			result = string.find("WYPTG",sWpnCol);
		end
	end

	if result then
		return result;
	else
		return 0;
	end
	
end

-- RomanRankDecode: converts a string with a Roman number in characters, to an integer number.
-- used for Legendary Item Title calculation.

local RomanCharsToValues = {["M"]=1000,["CM"]=900,["D"]=500,["CD"]=400,["C"]=100,["XC"]=90,["L"]=50,["XL"]=40,["X"]=10,["IX"]=9,["V"]=5,["IV"]=4,["I"]=1};

function RomanRankDecode(sNumber)

	if sNumber ~= nil and type(sNumber) == "string" and sNumber ~= "" then
		for sRomanRankChar, iRomanRankValue in pairs(RomanCharsToValues) do
			if string.sub(string.upper(sNumber),1,string.len(sRomanRankChar)) == sRomanRankChar then
				return iRomanRankValue+RomanRankDecode(string.sub(sNumber,string.len(sRomanRankChar)+1));
			end
		end
	end
	
	return 0;
	
end

-- ReverseCalc: tries to calculate back a calculation result to the original (integer) level.
-- Does not support N.

function ReverseCalc(sStat, dNum)

	if string.len(string.match(sStat,"([-%w]+)")) > 0 then
		local dNumMinus = dNum-DblCalcDev;
		local dNumPlus = dNum+DblCalcDev;
		local minlvl = 1;
		local maxlvl = 549;
		local devlvl = 3;
	
		local left = minlvl-1;
		local right = maxlvl;
		local middle;
		
		local count = minlvl;

		while right > left+1 and count <= maxlvl do
			count = count+1;
			middle = math.floor((left+right)/2);
			if CalcStat(sStat,middle) >= dNumMinus then
				right = middle;
			else
				left = middle;
			end
		end

		local mintest = right-devlvl;
		if mintest < minlvl then
			mintest = minlvl;
		end
		local maxtest = right+devlvl;
		if maxtest > maxlvl then
			maxtest = maxlvl;
		end

		local dFound;
	
		-- we check nearby in case the progression is not completely ascending/sorted.
		for test = mintest, maxtest do
			dFound = CalcStat(sStat,test);
			if dNumMinus <= dFound and dFound <= dNumPlus then
				return test;
			end
		end
	end

	return 0;

end

-- ****************** Misc. floating point support functions ******************

-- Misc. functions for floating point rounding.
-- 2nd parameter is number of decimals.

function RoundDbl(dNum, vDec)

	local dCorrection = 0.5+DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblDown(dNum, vDec)

	local dCorrection = DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblUp(dNum, vDec)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	if vDec == nil or (-DblCalcDev <= vDec and vDec <= DblCalcDev) then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^vDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end
	
end

function RoundDblLotro(dNum)

	local dCorrection = 1-DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end

	local iNumCeiled = math.floor(iSign*dNum+dCorrection);
	if iNumCeiled <= 1000 then
		return iSign*iNumCeiled;
	end

	local iFactor = 1;
	local iTestNum = math.floor(iNumCeiled/1000);
	while iTestNum > 0 do
		iTestNum = math.floor(iTestNum/10);
		iFactor = iFactor*10;
	end

	return iSign*math.floor(iNumCeiled/iFactor+dCorrection)*iFactor;

end

function RoundDblProg(dNum)

	local dCorrection = 0.5+DblCalcDev;
	local iSign = 1;
	if dNum < 0 then
		iSign = -1;
	end
	
	local dTestNum = dNum/(0.5*(iSign*63));
	local dDec = -math.floor(math.log10(dTestNum));

	if -DblCalcDev <= dDec and dDec <= DblCalcDev then
		return iSign*math.floor(iSign*dNum+dCorrection);
	else
		local dFactor = 10^dDec;
		return iSign*math.floor(iSign*dNum*dFactor+dCorrection)/dFactor;
	end

end

-- ******************************* End CalcStat *******************************